/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.com.rpc.annotation;

import de.starface.com.rpc.client.RpcClient;
import de.starface.com.rpc.server.RpcObjectRegistry;

import java.lang.annotation.Documented;
import java.lang.annotation.Inherited;
import java.lang.annotation.Retention;
import java.lang.annotation.Target;

import static java.lang.annotation.ElementType.METHOD;
import static java.lang.annotation.RetentionPolicy.RUNTIME;

/**
 * Annotation to define custom identifiers for remotely callable methods via RPC.
 * <p>
 * When the {@link RpcObjectRegistry} registers methods of an interface for remote calling or an
 * {@link RpcClient} creates an RPC proxy, this annotation is evaluated. Usually all methods are
 * registered for their names, but in case of overloading of methods this attempt will fail (as this
 * feature is not supported by this RPC framework.
 * </p>
 *
 * <p>
 * If such a annotation is present, the identifier will be set to the value provided by this
 * annotation. In the presence of a {@link RpcInterface} annotation, the value provided by this
 * annotation will be used as a prefix.
 * </p>
 *
 * @author may
 */
@Documented
@Inherited
@Retention(RUNTIME)
@Target(METHOD)
public @interface RpcProcedure {
	/** The name the method should be registered for. */
	String value();
}
