/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.com.rpc.annotation;

import de.starface.com.rpc.RpcException;

import java.lang.annotation.Inherited;
import java.lang.annotation.Retention;
import java.lang.annotation.Target;

import static java.lang.annotation.ElementType.METHOD;
import static java.lang.annotation.ElementType.TYPE;
import static java.lang.annotation.RetentionPolicy.RUNTIME;

/**
 * Defines the maximum processing time for a synchronous RPC remote procedure call. If the call does
 * not return in time the client will throw an {@link RpcException} with code
 * {@link RpcException.Codes.Client#NO_RESPONSE_FROM_SERVER}
 *
 * @author may
 */
@Inherited
@Retention(RUNTIME)
@Target({METHOD, TYPE})
public @interface RpcTimeout {
	/**
	 * Specifies the timeout for all calls without an annotation to be 30 seconds. Change this value
	 * to change the behavior for all these calls.
	 */
	public static long DEFAULT_CALL_TIMEOUT = 30 * 1000L;

	/**
	 * Specifies the timeout for the call. If no explicit value is given 30 seconds is used.
	 */
	long value() default 30 * 1000L;
}
