/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.com.rpc.client;

import de.starface.com.rpc.RpcException;
import de.starface.com.rpc.annotation.RpcTimeout;
import de.starface.com.rpc.common.ReflectionUtils;
import de.starface.com.rpc.common.RpcTransportToken;

/**
 * A base class for the implementation of {@link OutgoingRequestProcessor}s on the client side.
 *
 * @author tilmann
 *
 * @param <TransportToken>
 *        the transport token type of the requests processed
 */
public abstract class OutgoingRequestProcessorBase<TransportToken extends RpcTransportToken> implements
	OutgoingRequestProcessor<TransportToken> {
	protected long getTimeoutForCall() {
		RpcTimeout annotation = ReflectionUtils.findRpcAnnotation(RpcTimeout.class, request.getRpcInterface(),
			request.getRpcMethod());
		if (annotation != null) {
			return annotation.value();
		} else {
			return RpcTimeout.DEFAULT_CALL_TIMEOUT;
		}
	}

	protected OutgoingRequest<TransportToken> request = null;

	/**
	 * Actual processing logic to be implemented by derived classes
	 */
	protected abstract void processRequest() throws RpcException;

	/**
	 * @see de.starface.com.rpc.common.RequestProcessor#processRequest(de.starface.com.rpc.common.RpcRequest)
	 */
	@Override
	public void processRequest(OutgoingRequest<TransportToken> request) throws RpcException {
		this.request = request;
		processRequest();
	}
}
