/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.com.rpc.common;

import de.starface.com.rpc.RpcException;

/**
 * Implementations of this interface are able to process {@link RpcRequest}s. This implementation of
 * the strategy pattern hides the actual consumer of the request from the preprocessing.
 *
 * @author tilmann
 *
 * @param <Request>
 *        the type of request of this request processor
 */
public interface RequestProcessor<Request extends RpcRequest<?>> {
	/**
	 * Process the given request. On the client side this could be the sending of the request over a
	 * particular transport protocol. On the server side the calling of the request at the
	 * registered target method.
	 *
	 * @param request
	 *        the request to be processed
	 * @throws RpcException
	 *         if an exceptional condition is encountered
	 */
	public void processRequest(Request request) throws RpcException;
}
