/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.com.rpc.common;

import org.apache.commons.lang3.Validate;

import java.lang.reflect.Method;

/**
 * This class defines common information about all RPC requests
 *
 * @author tilmann
 *
 * @param <TransportToken>
 *        the transport token type of the request
 */
public class RpcRequest<TransportToken extends RpcTransportToken> {
	/** The method name for this RPC request */
	private final String rpcMethodName;

	private final Method rpcMethod;

	private final Class<?> rpcInterface;

	/** The parameters for the RPC request */
	private final Object[] parameters;

	/** The return value of the RPC request */
	private Object returnValue;

	/**
	 * Returns the name of the remote method to call
	 *
	 * @return The method name
	 */
	public String getRpcMethodName() {
		return rpcMethodName;
	}

	/**
	 * Creates a new RpcRequest
	 *
	 * @param rpcMethodName
	 *        The method name used for the RPC request
	 * @param params
	 *        The parameters used in the RPC request (may be null if no arguments are provided)
	 *
	 * @throws IllegalArgumentException
	 *         if no rpcMethodName name was provided
	 */
	protected RpcRequest(String rpcMethodName, Class<?> rpcInterface, Method rpcMethod, Object[] params)
		throws IllegalArgumentException {
		Validate.notEmpty(rpcMethodName, "rpcMethodName=empty");
		this.rpcMethodName = rpcMethodName;
		this.parameters = params;
		this.rpcInterface = rpcInterface;
		this.rpcMethod = rpcMethod;
	}

	public Object[] getParameters() {
		return parameters;
	}

	public Object getReturnValue() {
		return returnValue;
	}

	public void setReturnValue(Object returnValue) {
		this.returnValue = returnValue;
	}

	public Method getRpcMethod() {
		return rpcMethod;
	}

	public Class<?> getRpcInterface() {
		return rpcInterface;
	}
}
