/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.com.rpc.common.interceptor;

import de.starface.com.rpc.RpcException;
import de.starface.com.rpc.common.RpcRequest;

/**
 * A RpcRequestInterceptor is a class implementing a single aspect during processing of an RPC
 * request such as logging, value translation, authentication for example.
 *
 * @author tilmann
 *
 * @param <Request>
 *        the type of request this interceptor is intended for
 */
public interface RpcRequestInterceptor<Request extends RpcRequest<?>> {
	/**
	 * Intercept the given request. The implementing interceptor is supposed to call
	 * {@link RequestInterceptorChainLink#processRequest()} to continue the processing of the
	 * request. The implementation of this request interceptors aspect can take place before and
	 * after the call to processRequest().
	 *
	 * @param request
	 *        the request to be intercepted
	 * @param nextChainLink
	 *        the chain link used to continue request processing
	 * @throws RpcException
	 *         if an exceptional condition is encountered
	 */
	public void interceptRequest(Request request, RequestInterceptorChainLink nextChainLink) throws RpcException;
}
