/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.com.rpc.common.valuetranslation;

import de.starface.com.rpc.services.filetransfer.RpcFile;

/**
 * Implementations of FileTranslator are use during RPC value translation to decouple the automatic
 * file transfer process that actually transfers the files between client and server from the RPC
 * value translation implementation.
 *
 * @author tilmann
 */
public interface FileTranslator {
	/**
	 * Translates the given file to a fileId that can be used by the file translator on the side of
	 * the communication partner to obtain the transferred file using
	 * {@link #translateFileToFileId(RpcFile)}.
	 *
	 * @param file
	 *        the file to be transferred to the communication partner
	 * @return the id of the transferred file
	 */
	public String translateFileToFileId(RpcFile file);

	/**
	 * Retrieves a transferred file with the given id.
	 *
	 * @param fileId
	 *        the id of the transferred file
	 * @return the file transferred from the communication partner
	 */
	public RpcFile translateFileIdToFile(String fileId);
}
