/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */

package de.starface.com.rpc.server;

import de.starface.com.rpc.common.RpcRequest;
import de.starface.com.rpc.common.RpcTransportToken;

import org.apache.commons.lang3.Validate;

import java.lang.reflect.Method;

/**
 * An Incoming {@link RpcRequest} in the server implementation
 *
 * @author tilmann
 *
 * @param <TransportToken>
 *        the transport token type of the request
 */
public class IncomingRequest<TransportToken extends RpcTransportToken> extends RpcRequest<TransportToken> {
	/** The used transport token for this implementation */
	private TransportToken sourceTransportToken;

	/**
	 * Creates a new IncomingRequest with an {@link RpcTransportToken} identifying the source of the
	 * request
	 *
	 * @param rpcMethodName
	 *        The method name used for the remote call
	 * @param rpcInterface
	 *        the underlying Java interface the rpcMethod is defined in if present
	 * @param rpcMethod
	 *        the underlying Java method invoked for this request if present
	 * @param params
	 *        The parameters used in the remote call (may be null if no arguments are provided)
	 * @param sourceTransportToken
	 *        the {@link RpcTransportToken} identifying the source of this request
	 *
	 * @throws IllegalArgumentException
	 *         if no method name was provided or the given transport token is null
	 */
	public IncomingRequest(String rpcMethodName, Class<?> rpcInterface, Method rpcMethod, Object[] params,
						   TransportToken sourceTransportToken) throws IllegalArgumentException {
		super(rpcMethodName, rpcInterface, rpcMethod, params);
		Validate.notNull(sourceTransportToken, "sourceTransportToken=null");
		this.sourceTransportToken = sourceTransportToken;
	}

	/**
	 * Returns the {@link RpcTransportToken} which identifies the source of this request
	 *
	 * @return The TransportToken
	 */
	public TransportToken getSourceTransportToken() {
		return sourceTransportToken;
	}
}
