/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.com.rpc.services;

import de.starface.com.rpc.client.RpcClientBase;
import de.starface.com.rpc.common.RpcTransportToken;
import de.starface.com.rpc.common.authentication.RpcAuthToken;

/**
 * An Interface to represent RPC "agents" which assemble a collection of functionality on a set of
 * remote procedures. Agents work as a counterpart to {@link RpcService}s and encapsulate the
 * communication of a RPC Client with an RPC Server has the corresponding service registered.
 *
 * @author may
 * @param <RpcInterface>
 *        Type of the remote service to correspond with
 *
 */
public interface RpcAgent<RpcInterface> {

	/**
	 * Returns an interface which declares the methods this agent calls on the RPC server. This is
	 * essential as it is used during initialization of a agent in the {@link RpcClientBase}
	 *
	 * @return The RPC interface
	 *
	 * @see RpcClientBase#createRpcAgent(String, Class, RpcAuthToken)
	 * @see RpcClientBase#createRpcAgent(String, Class, RpcTransportToken)
	 */
	public Class<RpcInterface> getProxyType();

	/**
	 * Sets the proxy for RPC-Calls during initialization of this Agent in the {@link RpcClientBase}
	 *
	 * @param proxy
	 *        The proxy created for RPC-Calls
	 *
	 * @see RpcClientBase#createRpcAgent(String, Class, RpcAuthToken)
	 * @see RpcClientBase#createRpcAgent(String, Class, RpcTransportToken)
	 */
	public void setProxy(RpcInterface proxy);
}
