/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */

package de.starface.com.rpc.services.filetransfer;

/**
 * Event signaling that a file transfer failed (terminated unsuccessfully) with corresponding reason
 *
 * @author may
 *
 */
public interface FileTransferFailedEvent {
	/**
	 * Integer codes that represent distinct reasons a file transfer can fail for.
	 *
	 * @author tilmann
	 */
	public static class Codes {
		/**
		 * The file transfer was rejected by the server
		 */
		public static final int DENIED_BY_SERVER = 1;

		/**
		 * The client has aborted the file transfer
		 */
		public static final int ABORTED_BY_CLIENT = 2;

		/**
		 * The server has aborted the file transfer
		 */
		public static final int ABORTED_BY_SERVER = 3;

		/**
		 * The file has been corrupted during transfer
		 */
		public static final int FILE_CORRUPT = 4;

		/**
		 * The RPC used to transfer the file caused an Exception
		 */
		public static final int RPC_EXCEPTION = 5;

		/**
		 * File input caused an Exception
		 */
		public static final int FILE_IO_FAILURE = 6;

		/**
		 * The server encountered a timeout between two client requests
		 */
		public static final int TIMEOUT = 7;
	}

	/**
	 * The file transfer information of the transfer that caused the failure.
	 *
	 * @return Returns the file.
	 */
	public FileTransfer getFileTransfer();

	/**
	 * A message containing the reason for the failure.
	 *
	 * @return Returns the reason.
	 */
	public String getMessage();

	/**
	 * The code with one of the values of {@link Codes} specifies the reason for the failure
	 *
	 * @return the code of the reason
	 */
	public int getCode();

	/**
	 * A throwable, probably an exception, that lead to the failure
	 *
	 * @return the exception causing the file transfer to fail
	 */
	public Throwable getCause();
}
