/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.com.rpc.services.filetransfer;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.Validate;

import java.io.File;
import java.util.UUID;

/**
 * This class implements a wrapper for a Java file from an RPC file transfer point of view.
 *
 * @author tilmann
 */
public class RpcFile {
	private String id;

	private File file;

	private FileInfo fileInfo;

	public RpcFile(FileInfo fileInfo) {
		Validate.notNull(fileInfo, "fileInfo");
		init(null, null, fileInfo);
	}

	public RpcFile(String id, FileInfo fileInfo) {
		Validate.notNull(fileInfo, "fileInfo");
		Validate.notEmpty(id, "id=empty");
		init(id, null, fileInfo);
	}

	public RpcFile(File file, FileInfo fileInfo) {
		Validate.notNull(fileInfo, "fileInfo=null");
		Validate.notNull(file, "file=null");
		init(null, file, fileInfo);
	}

	public RpcFile(String id, File file, FileInfo fileInfo) {
		Validate.notEmpty(id, "id=empty");
		Validate.notNull(fileInfo, "fileInfo=null");
		Validate.notNull(file, "file=null");
		init(id, file, fileInfo);
	}

	public RpcFile(File file) {
		Validate.notNull(file, "file=null");
		init(null, file, null);
	}

	private void init(String id, File file, FileInfo fileInfo) {
		if (StringUtils.isEmpty(id)) {
			this.id = UUID.randomUUID().toString();
		} else {
			this.id = id;
		}
		this.file = file;
		if (fileInfo != null) {
			this.fileInfo = fileInfo;
		} else {
			this.fileInfo = new FileInfo(file);
		}
	}

	public String getId() {
		return id;
	}

	public File getFile() {
		return file;
	}

	public FileInfo getFileInfo() {
		return fileInfo;
	}

	public void setFile(File file) {
		if (file != null && this.file != null && !file.equals(this.file)) {
			throw new IllegalStateException("The file for the RpcFile instance can be set only once.");
		}
		this.file = file;
	}

	@Override
	public String toString() {
		return "RpcFile [id=" + id + ", file=" + file + ", fileInfo=" + fileInfo + "]";
	}
}
