/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.com.rpc.services.filetransfer.common;

import org.apache.commons.lang3.Validate;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.security.DigestInputStream;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

/**
 * This class encapsulates file operations on files for outgoing file transfers
 *
 * @author tilmann
 */
public class OutgoingFile {

	private static final Logger log = LogManager.getLogger(OutgoingFile.class);

	private File file;

	public File getFile() {
		return file;
	}

	private DigestInputStream inputStream;

	public void openWithDigestName(String digestName, File file) throws FileNotFoundException,
		NoSuchAlgorithmException {
		this.file = file;

		MessageDigest streamDigest = MessageDigest.getInstance(digestName);

		inputStream = new DigestInputStream(new BufferedInputStream(new FileInputStream(file)), streamDigest);
	}

	public int read(byte[] data) throws IOException {
		return inputStream.read(data);
	}

	public byte[] closeAndComputeDigest() throws IOException {
		inputStream.close();
		return inputStream.getMessageDigest().digest();
	}

	public void closeSilently() {
		if (inputStream != null) {
			try {
				inputStream.close();
			} catch (IOException e) {
				log.warn("Exception while closing digest input stream.", e);
			}
		}
	}

	public long getSize() {
		return file.length();
	}

	public String getName() {
		return file.getAbsolutePath();
	}

	public static void validateFile(File file) {
		Validate.notNull(file, "file=null");
		Validate.isTrue(file.exists(), file.getAbsolutePath() + " does not exist");
		Validate.isTrue(file.isFile(), file.getAbsolutePath() + " is not a valid file");
		Validate.isTrue(file.canRead(), file.getAbsolutePath() + " is not readable");
	}
}
