/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.com.rpc.xmlrpc.http;

import de.starface.com.rpc.RpcException;
import de.starface.com.rpc.xmlrpc.XmlRpcObjectParser;
import de.starface.com.rpc.xmlrpc.XmlRpcObjectSerializer;
import de.starface.com.xmpp.smack.XmlRpcPacketProvider;

import org.xmlpull.mxp1.MXParser;
import org.xmlpull.v1.XmlPullParser;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;

/**
 * A object parser for {@link XmlRpcObject}s following the concept of the
 * {@link XmlRpcPacketProvider} also using the {@link XmlRpcObjectSerializer}.
 *
 * @author may
 *
 */
public class HttpXmlRpcObjectParser {

	/**
	 * Parses {@link XmlRpcObject}s from a input stream
	 *
	 * @param stream
	 *        The stream to parse
	 * @return the object or null if parsing failed
	 * @throws RpcException
	 *         If the parsing goes wrong
	 */
	public XmlRpcObject parseStreamAsServer(InputStream stream) throws RpcException {
		return parse(stream, true);
	}

	/**
	 * Parses {@link XmlRpcObject}s from a input stream
	 *
	 * @param stream
	 *        The stream to parse
	 * @return the object or null if parsing failed
	 * @throws RpcException
	 *         If the parsing goes wrong
	 */
	public XmlRpcObject parseStreamAsClient(InputStream stream) throws RpcException {
		return parse(stream, false);
	}

	private XmlRpcObject parse(InputStream stream, boolean asServer) throws RpcException {

		try {
			XmlPullParser pullParser = new MXParser();
			BufferedReader reader = new BufferedReader(new InputStreamReader(stream, "UTF-8"));
			pullParser.setFeature(XmlPullParser.FEATURE_PROCESS_NAMESPACES, true);
			pullParser.setInput(reader);

			HttpXmlRpcObjectBuilder objectBuilder = new HttpXmlRpcObjectBuilder();
			XmlRpcObjectParser objectParser = new XmlRpcObjectParser(objectBuilder, "query");

			objectParser.parse(pullParser);

			return objectBuilder.getBuildResult();
		} catch (Exception e) {
			int code = asServer ? RpcException.Codes.Server.XML_PARSE_ERROR
				: RpcException.Codes.Client.XML_PARSE_ERROR;
			throw new RpcException(code,
				"Could not parse a valid XML-RPC methodCall or methodResponse from the transport payload", e);
		} finally {
			try {
				stream.close();
			} catch (IOException e) {
				// at least we tried!
			}
		}
	}
}
