/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.com.rpc.xmlrpc.types;

import java.util.ArrayList;
import java.util.List;

/**
 * The array type in XML-RPC.
 * <ul>
 * <li>Tag: {@code <array>}</li>
 * <li>Type: An {@code <array>} contains a single {@code <data>} element, which can contain any
 * number of {@code <value>}s.</li>
 * <li>{@code <data> <value><i4>12</i4></value> <value><string>Egypt</string></value> <value><boolean>0</boolean></value>
 * <value><i4>-31</i4></value> </data>}</li>
 * </ul>
 * You can mix types as the example above illustrates.
 *
 * {@code <arrays>}s can be recursive, any value may contain an {@code <array>} or any other type,
 * including a {@code <struct>}, described above.
 *
 * @author may
 *
 */
public class XmlRpcArray implements XmlRpcType {
	private final ArrayList<XmlRpcType> _value;

	public XmlRpcArray() {
		_value = new ArrayList<>();
	}

	public XmlRpcArray(List<XmlRpcType> value) {
		_value = new ArrayList<>(value);
	}

	public ArrayList<XmlRpcType> getValue() {
		return new ArrayList<>(_value);
	}

	/**
	 * @see Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((_value == null) ? 0 : _value.hashCode());
		return result;
	}

	/**
	 * @see Object#equals(Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		final XmlRpcArray other = (XmlRpcArray) obj;
		if (_value == null) {
			if (other._value != null) {
				return false;
			}
		} else if (!_value.equals(other._value)) {
			return false;
		}
		return true;
	}

	/**
	 * @see de.starface.com.rpc.xmlrpc.types.XmlRpcType#getJavaType()
	 */
	@Override
	public Class<?> getJavaType() {
		return ArrayList.class;
	}
}
