/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.com.rpc.xmlrpc.types;

/**
 * The boolean type in XML-RPC.
 * <ul>
 * <li>Tag: {@code <boolean>}</li>
 * <li>Type: 0 (false) or 1 (true)</li>
 * <li>Example: 1</li>
 * </ul>
 *
 * @author may
 *
 */
public class XmlRpcBoolean implements XmlRpcType {
	private final Boolean _value;

	public XmlRpcBoolean(Boolean value) {
		_value = Boolean.valueOf(value);
	}

	public XmlRpcBoolean(boolean value) {
		_value = Boolean.valueOf(value);
	}

	public Boolean toBoxedValue() {
		return _value;
	}

	public boolean toPrimitiveValue() {
		return _value.booleanValue();
	}

	/**
	 * @see Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((_value == null) ? 0 : _value.hashCode());
		return result;
	}

	/**
	 * @see Object#equals(Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		final XmlRpcBoolean other = (XmlRpcBoolean) obj;
		if (_value == null) {
			if (other._value != null) {
				return false;
			}
		} else if (!_value.equals(other._value)) {
			return false;
		}
		return true;
	}

	/**
	 * @see de.starface.com.rpc.xmlrpc.types.XmlRpcType#getJavaType()
	 */
	@Override
	public Class<?> getJavaType() {
		return Boolean.class;
	}
}
