/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.com.rpc.xmlrpc.types;

/**
 * The double type in XML-RPC.
 * <ul>
 * <li>Tag: {@code <double>}</li>
 * <li>Type: double-precision signed floating point number</li>
 * <li>Example: -12.214</li>
 * </ul>
 *
 * @author may
 *
 */
public class XmlRpcDouble implements XmlRpcType {
	private final Double _value;

	public XmlRpcDouble(Double value) {
		_value = new Double(value);
	}

	public XmlRpcDouble(double value) {
		_value = new Double(value);
	}

	public Double toBoxedValue() {
		return new Double(_value);
	}

	public double toPrimitiveValue() {
		return _value.doubleValue();
	}

	/**
	 * @see Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((_value == null) ? 0 : _value.hashCode());
		return result;
	}

	/**
	 * @see Object#equals(Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		final XmlRpcDouble other = (XmlRpcDouble) obj;
		if (_value == null) {
			if (other._value != null) {
				return false;
			}
		} else if (!_value.equals(other._value)) {
			return false;
		}
		return true;
	}

	/**
	 * @see de.starface.com.rpc.xmlrpc.types.XmlRpcType#getJavaType()
	 */
	@Override
	public Class<?> getJavaType() {
		return Double.class;
	}
}
