/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.com.rpc.xmlrpc.types;

/**
 * The string type in XML-RPC.
 * <ul>
 * <li>Tag: {@code <string>}</li>
 * <li>Type: string</li>
 * <li>Example: hello world</li>
 * </ul>
 *
 * @author may
 *
 */
public class XmlRpcString implements XmlRpcType {
	private final String _value;

	public XmlRpcString(String value) {
		_value = value;
	}

	public String toStringValue() {
		return _value;
	}

	/**
	 * @see Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((_value == null) ? 0 : _value.hashCode());
		return result;
	}

	/**
	 * @see Object#equals(Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		final XmlRpcString other = (XmlRpcString) obj;
		if (_value == null) {
			if (other._value != null) {
				return false;
			}
		} else if (!_value.equals(other._value)) {
			return false;
		}
		return true;
	}

	/**
	 * @see de.starface.com.rpc.xmlrpc.types.XmlRpcType#getJavaType()
	 */
	@Override
	public Class<?> getJavaType() {
		return String.class;
	}
}
