/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.com.rpc.xmlrpc.types;

import java.util.HashMap;
import java.util.Map;

/**
 * The struct type in XML-RPC.
 * <ul>
 * <li>Tag: {@code <struct>}</li>
 * <li>Type: A {@code <struct>} contains {@code <member>}s and each {@code <member>} contains a
 * {@code <name>} and a {@code <value>}.</li>
 * <li>Example: {@code <member> <name>lowerBound</name> <value><i4>18</i4></value> </member>
 * <member> <name>upperBound</name> <value><i4>139</i4></value> </member>}</li>
 * </ul>
 * {@code <struct>}s can be recursive, any {@code <value>} may contain a {@code <struct>} or any
 * other type, including an {@code <array>}, described below.
 *
 * @author may
 *
 */
public class XmlRpcStruct implements XmlRpcType {
	private final HashMap<String, XmlRpcType> _value;

	public XmlRpcStruct() {
		_value = new HashMap<>();
	}

	public XmlRpcStruct(HashMap<String, XmlRpcType> value) {
		_value = new HashMap<>(value);
	}

	public HashMap<String, XmlRpcType> getValue() {
		return new HashMap<>(_value);
	}

	/**
	 * @see Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((_value == null) ? 0 : _value.hashCode());
		return result;
	}

	/**
	 * @see Object#equals(Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		final XmlRpcStruct other = (XmlRpcStruct) obj;
		if (_value == null) {
			if (other._value != null) {
				return false;
			}
		} else if (!_value.equals(other._value)) {
			return false;
		}
		return true;
	}

	/**
	 * @see de.starface.com.rpc.xmlrpc.types.XmlRpcType#getJavaType()
	 */
	@Override
	public Class<?> getJavaType() {
		return Map.class;
	}
}
