/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.com.rpc.common.client;

import de.starface.com.rpc.RpcException;
import de.starface.com.rpc.client.AsynchronousExecutor;
import de.starface.com.rpc.common.RpcTransportToken;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.junit.jupiter.api.Test;

import java.util.concurrent.atomic.AtomicBoolean;

import static org.junit.jupiter.api.Assertions.assertEquals;

class AsynchronousExecutorTest {
	private final Logger log = LogManager.getLogger(getClass());
	private static RpcTransportToken TOKEN_USER_ONE = new RpcTransportToken() {

		@Override
		public String getIdentifierString() {
			return "user1@localhost";
		}
	};

	private static RpcTransportToken TOKEN_USER_TWO = new RpcTransportToken() {

		@Override
		public String getIdentifierString() {
			return "user2@localhost";
		}
	};

	@Test
	void normalExecution() throws Exception {
		AtomicBoolean user1Executed = new AtomicBoolean(false);
		Runnable runnableUser1 = createRunnable("User1 executed", user1Executed, 0, 0);
		AsynchronousExecutor asynchronousExecutor = new AsynchronousExecutor();
		asynchronousExecutor.execute(runnableUser1, TOKEN_USER_ONE);
		Thread.sleep(10);
		assertEquals(true, user1Executed.get());
	}

	@Test
	void concurrentExecution() throws Exception {
		AtomicBoolean user1Executed = new AtomicBoolean(false);
		Runnable runnableUser1 = createRunnable("User1 executed", user1Executed, 50, 0);
		AtomicBoolean user2Executed = new AtomicBoolean(false);
		Runnable runnableUser2 = createRunnable("User2 executed", user2Executed, 0, 0);
		AsynchronousExecutor asynchronousExecutor = new AsynchronousExecutor();
		asynchronousExecutor.execute(runnableUser1, TOKEN_USER_ONE);
		asynchronousExecutor.execute(runnableUser2, TOKEN_USER_TWO);
		Thread.sleep(10);

		assertEquals(false, user1Executed.get());
		assertEquals(true, user2Executed.get());

		Thread.sleep(45);

		assertEquals(true, user1Executed.get());
		assertEquals(true, user2Executed.get());
	}

	@Test
	void blockingExecution() {
		AsynchronousExecutor asynchronousExecutor = new AsynchronousExecutor();
		AtomicBoolean user1Executed = new AtomicBoolean(false);
		boolean exception = false;
		try {
			for (int i = 0; i < 1100; i++) {
				Runnable runnableUser1 = createRunnable("User1 executed", user1Executed, 1000, i);
				asynchronousExecutor.execute(runnableUser1, TOKEN_USER_ONE);
			}
		} catch (RpcException e) {
			log.info(e.getMessage());
			exception = true;
		}
		assertEquals(true, exception);

	}

	private Runnable createRunnable(String message, AtomicBoolean executedFlag, long waitTime,
									final int currentNumber) {
		Runnable runnable = new Runnable() {
			@Override
			public void run() {
				try {
					Thread.sleep(waitTime);
				} catch (InterruptedException e) {
					log.info("InterruptedException " + currentNumber);
				}
				executedFlag.set(true);
				log.info(message);
			}
		};
		return runnable;
	}
}
