/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.com.rpc.unittests.common.valuetranslation;

import de.starface.com.rpc.annotation.RpcValue;
import de.starface.com.rpc.annotation.RpcValueObject;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

@RpcValueObject
public class VTTestSubBean {
	@RpcValue
	String stringField;

	@RpcValue("integerField")
	int intField;

	@RpcValue
	Integer IntegerField;

	@RpcValue
	boolean booleanField;

	@RpcValue
	Boolean BooleanField;

	@RpcValue
	double doubleField;

	@RpcValue
	Double DoubleField;

	@RpcValue
	Date DateField;

	@RpcValue
	VTTestEnum EnumField;

	public enum VTTestEnum {
		A,
		B,
		C;
	}

	protected static VTTestSubBean createVTTestSubBean() {
		VTTestSubBean bean = new VTTestSubBean();
		bean.initValues();
		return bean;
	}

	@SuppressWarnings("deprecation")
	protected void initValues() {
		booleanField = true;
		BooleanField = false;
		stringField = "TestString";
		intField = 123456;
		IntegerField = Integer.MAX_VALUE;
		doubleField = 123.456;
		DoubleField = Double.MAX_VALUE;
		DateField = new Date(1980, 2, 1, 21, 30, 10);
		EnumField = VTTestEnum.B;
	}

	Map<String, Object> convertToMap() {
		Map<String, Object> map = new HashMap<>();
		map.put("stringField", stringField);
		map.put("integerField", intField);
		map.put("IntegerField", IntegerField);
		map.put("booleanField", booleanField);
		map.put("BooleanField", BooleanField);
		map.put("doubleField", doubleField);
		map.put("DoubleField", DoubleField);
		map.put("DateField", DateField);
		map.put("EnumField", EnumField != null ? EnumField.name() : "");
		return map;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((BooleanField == null) ? 0 : BooleanField.hashCode());
		result = prime * result + ((DateField == null) ? 0 : DateField.hashCode());
		result = prime * result + ((DoubleField == null) ? 0 : DoubleField.hashCode());
		result = prime * result + ((EnumField == null) ? 0 : EnumField.hashCode());
		result = prime * result + ((IntegerField == null) ? 0 : IntegerField.hashCode());
		result = prime * result + (booleanField ? 1231 : 1237);
		long temp;
		temp = Double.doubleToLongBits(doubleField);
		result = prime * result + (int) (temp ^ (temp >>> 32));
		result = prime * result + intField;
		result = prime * result + ((stringField == null) ? 0 : stringField.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		VTTestSubBean other = (VTTestSubBean) obj;
		if (BooleanField == null) {
			if (other.BooleanField != null) {
				return false;
			}
		} else if (!BooleanField.equals(other.BooleanField)) {
			return false;
		}
		if (DateField == null) {
			if (other.DateField != null) {
				return false;
			}
		} else if (!DateField.equals(other.DateField)) {
			return false;
		}
		if (DoubleField == null) {
			if (other.DoubleField != null) {
				return false;
			}
		} else if (!DoubleField.equals(other.DoubleField)) {
			return false;
		}
		if (EnumField != other.EnumField) {
			return false;
		}
		if (IntegerField == null) {
			if (other.IntegerField != null) {
				return false;
			}
		} else if (!IntegerField.equals(other.IntegerField)) {
			return false;
		}
		if (booleanField != other.booleanField) {
			return false;
		}
		if (Double.doubleToLongBits(doubleField) != Double.doubleToLongBits(other.doubleField)) {
			return false;
		}
		if (intField != other.intField) {
			return false;
		}
		if (stringField == null) {
			if (other.stringField != null) {
				return false;
			}
		} else if (!stringField.equals(other.stringField)) {
			return false;
		}
		return true;
	}

	@Override
	public String toString() {
		return "VTTestSubBean [stringField=" + stringField + ", intField=" + intField + ", IntegerField="
			+ IntegerField + ", booleanField=" + booleanField + ", BooleanField=" + BooleanField
			+ ", doubleField=" + doubleField + ", DoubleField=" + DoubleField + ", DateField=" + DateField
			+ ", EnumField=" + EnumField + "]";
	}
}
