/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.examples.uci30.swing;

import de.starface.examples.uci30.swing.model.ConnectionManager;
import de.starface.examples.uci30.swing.model.UciConnectionManager;
import de.starface.examples.uci30.swing.view.DemoClientFrame;
import de.starface.examples.uci30.swing.viewmodel.DemoClientViewModel;

import java.io.PrintStream;
import java.io.PrintWriter;
import java.io.StringWriter;

import javax.swing.*;

public class DemoClient {

	/**
	 * Main entry of the class.
	 */
	public static void main(String[] args) {
		Thread.setDefaultUncaughtExceptionHandler(createUncaughtExceptionHandler(System.err));

		installLnF();
		createAndShowGui();
	}

	private static Thread.UncaughtExceptionHandler createUncaughtExceptionHandler(PrintStream err) {
		return (t, e) -> {
			StringWriter stacktraceWriter = new StringWriter();
			e.printStackTrace(new PrintWriter(stacktraceWriter));
			String message = "Uncaught exception: " + e.getMessage() + " " + stacktraceWriter;
			err.println(message);
			err.flush();
			JOptionPane.showMessageDialog(null, message);
		};
	}

	private static void installLnF() {
		String[] lookAndFeels = {"com.sun.java.swing.plaf.windows.WindowsLookAndFeel",
			"com.sun.java.swing.plaf.nimbus.NimbusLookAndFeel",
			"javax.swing.plaf.nimbus.NimbusLookAndFeel", // Java 9+
			UIManager.getCrossPlatformLookAndFeelClassName()};

		for (String lnfClassname : lookAndFeels) {
			try {
				UIManager.setLookAndFeel(lnfClassname);
				break;
			} catch (Exception e) {
				System.err.println("INFO: cannot set LookAndFeel " + lnfClassname + " on this platform\n" + e);
			}
		}
	}

	private static void createAndShowGui() {
		DemoClientFrame frame = new DemoClientFrame();
		frame.redirectStandardOutputToWindow();

		ConnectionManager connectionManager = new UciConnectionManager();
		DemoClientViewModel viewModel = new DemoClientViewModel(connectionManager, frame);

		frame.setViewModel(viewModel);

		frame.setDefaultCloseOperation(DemoClientFrame.EXIT_ON_CLOSE);
		frame.getContentPane().setPreferredSize(frame.getSize());
		frame.pack();
		frame.setLocationRelativeTo(null);
		frame.setVisible(true);
	}
}
