/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.examples.uci30.swing.view;

import com.jgoodies.binding.adapter.Bindings;
import com.jgoodies.binding.beans.BeanAdapter;

import de.starface.examples.uci30.swing.view.binding.MultiListSelectionAdapter;
import de.starface.examples.uci30.swing.viewmodel.CcbsCallViewModel;

import org.dyno.visual.swing.layouts.Bilateral;
import org.dyno.visual.swing.layouts.Constraints;
import org.dyno.visual.swing.layouts.GroupLayout;
import org.dyno.visual.swing.layouts.Leading;

import java.awt.*;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.*;
import javax.swing.table.DefaultTableModel;

/**
 * This panel provides a GUI for the UCI communication call functionality
 *
 * @author tilmann
 */
// VS4E -- DO NOT REMOVE THIS LINE!
public class CcbsCallPanel extends JPanel {
	private static final long serialVersionUID = 1L;

	private JLabel ccbsStateLabel;

	private JTextField ccbsStateText;

	private JScrollPane CcbscallTableScrollPane;
	private JTable ccbsCallTable;

	private JButton ccbsCallButton;

	private JButton removeAvailableCcbsButton;

	private JButton allCcbsCallsButton;

	private JButton ccbsAccountStateButton;

	private String selectSingleCall = "Select a single call";

	@SuppressWarnings("unused")
	private static final String PREFERRED_LOOK_AND_FEEL = "com.sun.java.swing.plaf.nimbus.NimbusLookAndFeel";

	public CcbsCallPanel() {
		initComponents();
	}

	private void initComponents() {
		setMinimumSize(new Dimension(850, 500));
		setPreferredSize(new Dimension(850, 500));
		setLayout(new GroupLayout());

		add(getCcbsStateLabel(), new Constraints(new Leading(9, 100, 6), new Leading(10, 6, 6)));
		add(getCcbsCallTableScrollPane(), new Constraints(new Bilateral(9, 10, 500), new Leading(100, 350, 6, 6)));

		add(getCcbsStateText(), new Constraints(new Leading(120, 120, 6, 6), new Leading(10, 6, 6)));

		add(getCcbsAccountStateButton(), new Constraints(new Leading(250, 150, 6, 6), new Leading(10, 6, 6)));
		add(getCcbsCallButton(), new Constraints(new Leading(250, 150, 6, 6), new Leading(40, 6, 6)));
		add(getRemoveAvailableCcbsButton(), new Constraints(new Leading(400, 200, 6, 6), new Leading(40, 6, 6)));
		add(getAllCcbsCallsButton(), new Constraints(new Leading(250, 150, 6, 6), new Leading(70, 6, 6)));
	}

	private JButton getCcbsAccountStateButton() {
		if (ccbsAccountStateButton == null) {
			ccbsAccountStateButton = new JButton();
			ccbsAccountStateButton.setText("getAccountState");
		}
		return ccbsAccountStateButton;
	}

	private JButton getAllCcbsCallsButton() {
		if (allCcbsCallsButton == null) {
			allCcbsCallsButton = new JButton();
			allCcbsCallsButton.setText("getAllCcbsCalls");
		}
		return allCcbsCallsButton;
	}

	private JButton getCcbsCallButton() {
		if (ccbsCallButton == null) {
			ccbsCallButton = new JButton();
			ccbsCallButton.setText("switch ccbs state");
			ccbsCallButton.setEnabled(false);
			ccbsCallButton.setToolTipText(selectSingleCall);
		}
		return ccbsCallButton;
	}

	private JButton getRemoveAvailableCcbsButton() {
		if (removeAvailableCcbsButton == null) {
			removeAvailableCcbsButton = new JButton();
			removeAvailableCcbsButton.setText("removeAvailableCcbs");
			removeAvailableCcbsButton.setToolTipText(selectSingleCall);
		}
		return removeAvailableCcbsButton;
	}

	private JTable getCcbsCallTable() {
		if (ccbsCallTable == null) {
			ccbsCallTable = new JTable();
			ccbsCallTable.setModel(new DefaultTableModel(new Object[][]{}, new String[]{}) {
				private static final long serialVersionUID = 1L;
				Class<?>[] types = new Class<?>[]{};

				@Override
				public Class<?> getColumnClass(int columnIndex) {
					return types[columnIndex];
				}
			});
			ccbsCallTable.setRowSelectionAllowed(true);
			ccbsCallTable.setColumnSelectionAllowed(false);
		}
		return ccbsCallTable;
	}

	private JScrollPane getCcbsCallTableScrollPane() {
		if (CcbscallTableScrollPane == null) {
			CcbscallTableScrollPane = new JScrollPane();
			CcbscallTableScrollPane.setViewportView(getCcbsCallTable());
		}
		return CcbscallTableScrollPane;
	}

	private JLabel getCcbsStateLabel() {
		if (ccbsStateLabel == null) {
			ccbsStateLabel = new JLabel();
			ccbsStateLabel.setText("CCBS State");
		}
		return ccbsStateLabel;
	}

	private JTextField getCcbsStateText() {
		if (ccbsStateText == null) {
			ccbsStateText = new JTextField();
			ccbsStateText.setEditable(false);
			ccbsStateText.setText("INACTIVE");
		}
		return ccbsStateText;
	}

	private PropertyChangeListener enableCcbsCallButton = new PropertyChangeListener() {
		@Override
		public void propertyChange(PropertyChangeEvent e) {
			if (e.getPropertyName() == CcbsCallViewModel.PROPERTY_GET_CCBS_IS_ALLOWED) {
				boolean enableButton = (Boolean) e.getNewValue();
				getCcbsCallButton().setEnabled(enableButton);
				getCcbsCallButton().setToolTipText(enableButton ? null : selectSingleCall);
			}
		}
	};

	public void setViewModel(CcbsCallViewModel viewModel) {
		initBindings(viewModel);
	}

	private void initBindings(final CcbsCallViewModel viewModel) {
		getCcbsCallButton().addActionListener(viewModel.getCcbsCallActionListener());
		getRemoveAvailableCcbsButton().addActionListener(viewModel.getRemoveAvailableCcbsActionListener());
		getAllCcbsCallsButton().addActionListener(viewModel.getAllCcbsCallsActionListener());
		getCcbsAccountStateButton().addActionListener(viewModel.getAccountStateActionListener());
		BeanAdapter<CcbsCallViewModel> beanAdapter = new BeanAdapter<>(viewModel, true);

		viewModel.addPropertyChangeListener(enableCcbsCallButton);

		ccbsCallTable.setModel(viewModel.getCcbsCallTableModel());
		ccbsCallTable.setSelectionModel(new MultiListSelectionAdapter(viewModel.getSelectedRows()));
		ccbsCallTable.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		ccbsCallTable.getSelectionModel().addListSelectionListener(viewModel.getSelectedCallsListener());

		Bindings.bind(ccbsStateText, beanAdapter.getValueModel(CcbsCallViewModel.PROPERTY_CCBS_STATUS));
	}
}
