/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.examples.uci30.swing.view;

import com.jgoodies.binding.adapter.Bindings;
import com.jgoodies.binding.beans.BeanAdapter;
import com.jgoodies.binding.list.SelectionInList;

import de.starface.examples.uci30.swing.model.Config;
import de.starface.examples.uci30.swing.view.binding.MultiListSelectionAdapter;
import de.starface.examples.uci30.swing.viewmodel.ConferenceCallViewModel;
import de.starface.integration.uci.java.v30.values.CallProperties;
import de.starface.integration.uci.java.v30.values.ConferenceCallProperties;

import org.dyno.visual.swing.layouts.Bilateral;
import org.dyno.visual.swing.layouts.Constraints;
import org.dyno.visual.swing.layouts.GroupLayout;
import org.dyno.visual.swing.layouts.Leading;

import java.awt.*;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;

import javax.swing.*;
import javax.swing.table.DefaultTableModel;

/**
 * This panel provides a GUI for the UCI communication call functionality
 *
 * @author dirk
 */
// VS4E -- DO NOT REMOVE THIS LINE!
public class ConferenceCallPanel extends JPanel {

	private static final long serialVersionUID = 1L;

	private JButton allCanHearInRoomButton;
	private JButton allCanSpeakInRoomButton;
	private JButton onlyOneCanSpeakInRoomButton;
	private JButton startManagedConferenceButton;
	private JButton createUnmanagedConferenceButton;
	private JButton getAllConferenceCallsButton;
	private JButton getAllConferenceIdsButton;
	private JButton leaveConferenceButton;
	private JButton quitConferenceButton;
	//	private JButton mediateToConferenceButton;
	private JButton transferToConferenceButton;
	private JButton joinToConferenceButton;
	private JButton callNumberToConferenceButton;

	private JButton listenToConferenceButton;
	private JButton muteToConferenceButton;
	private JButton wantToTalkToConferenceButton;

	private JButton hangupCallButton;

	private JList conferenceIdsList;

	private JScrollPane conferenceScrollPane;

	private JTable callTable;
	private JTable conferenceCallTable;

	private JScrollPane callTableScrollPane;
	private JScrollPane conferenceCallTableScrollPane;

	private JLabel conferenceCallsLabel;
	private JLabel conferenceIdLabel;
	private JLabel callTableLabel;

	private JLabel destinationNumberLabel;

	private JTextField destinationNumberText;

	@SuppressWarnings("unused")
	private static final String PREFERRED_LOOK_AND_FEEL = "com.sun.java.swing.plaf.nimbus.NimbusLookAndFeel";

	public ConferenceCallPanel() {
		initComponents();
	}

	private void initComponents() {
		setMinimumSize(new Dimension(850, 500));
		setPreferredSize(new Dimension(850, 500));
		setLayout(new GroupLayout());

		// Conference List
		add(getConferenceIdLabel(), new Constraints(new Leading(10, 120, 6, 6), new Leading(10, 6, 6)));
		add(getConferenceScrollPane(), new Constraints(new Leading(10, 120, 6, 6), new Leading(40, 90, 6, 6)));

		// Buttons

		add(getGetAllConferenceIdsButton(), new Constraints(new Leading(150, 150, 6, 6), new Leading(10, 6, 6)));
		add(getAllCanHearInRoomButton(), new Constraints(new Leading(150, 150, 6, 6), new Leading(40, 6, 6)));
		add(getAllCanSpeakInRoom(), new Constraints(new Leading(150, 150, 6, 6), new Leading(70, 6, 6)));
		add(getGetAllConferenceCallsButton(), new Constraints(new Leading(150, 250, 6, 6), new Leading(100, 6, 6)));

		add(getStartManagedConferenceButton(), new Constraints(new Leading(300, 150, 6, 6), new Leading(10, 6, 6)));
		add(getCreateUnmanagedConferenceButton(), new Constraints(new Leading(300, 150, 6, 6), new Leading(40, 6,
			6)));
		add(getJoinToConferenceButton(), new Constraints(new Leading(300, 120, 6, 6), new Leading(70, 6, 6)));

		add(getLeaveConferenceButton(), new Constraints(new Leading(450, 120, 6, 6), new Leading(10, 6, 6)));
		add(getQuitConferenceButton(), new Constraints(new Leading(450, 120, 6, 6), new Leading(40, 6, 6)));

		// ConferenceCall Table
		add(getHangupCallButton(), new Constraints(new Leading(9, 120, 6, 6), new Leading(150, 6, 6)));
		add(getListentoConferenceButton(), new Constraints(new Leading(9, 120, 6, 6), new Leading(180, 6, 6)));
		add(getOnlyOneCanSpeakInRoom(), new Constraints(new Leading(9, 150, 6, 6), new Leading(210, 6, 6)));

		add(getWantToTalktoConferenceButton(),
			new Constraints(new Leading(130, 120, 6, 6), new Leading(150, 6, 6)));
		add(getMutetoConferenceButton(), new Constraints(new Leading(130, 120, 6, 6), new Leading(180, 6, 6)));

		add(getConferenceCallsLabel(), new Constraints(new Leading(300, 12, 12), new Leading(110, 6, 6)));
		add(getConferenceCallTableScrollPane(), new Constraints(new Bilateral(300, 10, 500), new Leading(150, 150,
			6, 6)));

		// Call Table
		add(getDestinationNumberLabel(), new Constraints(new Leading(10, 120, 10, 10), new Leading(310, 6, 6)));
		add(getDestinationNumberText(), new Constraints(new Leading(150, 120, 6, 6), new Leading(310, 6, 6)));

		add(getCallNumberToConferenceButton(), new Constraints(new Leading(10, 120, 6, 6), new Leading(340, 6, 6)));
		//		add(getMediateToConferenceButton(), new Constraints(new Leading(130, 120, 6, 6), new Leading(340, 6, 6)));
		add(getTransferToConferenceButton(), new Constraints(new Leading(10, 120, 6, 6), new Leading(370, 6, 6)));

		//Label
		add(getCallTableLabel(), new Constraints(new Leading(300, 12, 12), new Leading(310, 6, 6)));
		add(getCallTableScrollPane(), new Constraints(new Bilateral(300, 10, 500), new Leading(340, 150, 6, 6)));

		setSize(570, 500);

	}

	private JLabel getCallTableLabel() {
		if (callTableLabel == null) {
			callTableLabel = new JLabel("active calls");
		}
		return callTableLabel;

	}

	private JTable getCallTable() {
		if (callTable == null) {
			callTable = new JTable();
			callTable.setModel(new DefaultTableModel(new Object[][]{}, new String[]{}) {
				private static final long serialVersionUID = 1L;
				Class<?>[] types = new Class<?>[]{};

				@Override
				public Class<?> getColumnClass(int columnIndex) {
					return types[columnIndex];
				}
			});
			callTable.setRowSelectionAllowed(true);
			callTable.setColumnSelectionAllowed(false);
			callTable.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		}
		return callTable;
	}

	private JScrollPane getCallTableScrollPane() {
		if (callTableScrollPane == null) {
			callTableScrollPane = new JScrollPane(getCallTable());
		}
		return callTableScrollPane;
	}

	private JTable getConferenceCallTable() {
		if (conferenceCallTable == null) {
			conferenceCallTable = new JTable();
			conferenceCallTable.setModel(new DefaultTableModel(new Object[][]{}, new String[]{}) {
				private static final long serialVersionUID = 1L;
				Class<?>[] types = new Class<?>[]{};

				@Override
				public Class<?> getColumnClass(int columnIndex) {
					return types[columnIndex];
				}
			});
			conferenceCallTable.setRowSelectionAllowed(true);
			conferenceCallTable.setColumnSelectionAllowed(false);
			conferenceCallTable.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		}
		return conferenceCallTable;
	}

	private JScrollPane getConferenceCallTableScrollPane() {
		if (conferenceCallTableScrollPane == null) {
			conferenceCallTableScrollPane = new JScrollPane(getConferenceCallTable());
		}
		return conferenceCallTableScrollPane;
	}

	private JTextField getDestinationNumberText() {
		if (destinationNumberText == null) {
			destinationNumberText = new JTextField();
			destinationNumberText.setText(Config.getValue("destinationNumber"));
			destinationNumberText.addFocusListener(new FocusAdapter() {

				@Override
				public void focusGained(FocusEvent event) {
					destinationNumberTextFocusFocusGained(event);
				}
			});
		}
		return destinationNumberText;
	}

	private JLabel getDestinationNumberLabel() {
		if (destinationNumberLabel == null) {
			destinationNumberLabel = new JLabel();
			destinationNumberLabel.setText("destinationNumber");
		}
		return destinationNumberLabel;
	}

	private JLabel getConferenceCallsLabel() {
		if (conferenceCallsLabel == null) {
			conferenceCallsLabel = new JLabel();
			conferenceCallsLabel.setText("<html>Conference<br>Participants</html>");
		}
		return conferenceCallsLabel;
	}

	private JLabel getConferenceIdLabel() {
		if (conferenceIdLabel == null) {
			conferenceIdLabel = new JLabel();
			conferenceIdLabel.setText("<html> Conference<br>Room Ids</html>");
		}
		return conferenceIdLabel;
	}

	private JScrollPane getConferenceScrollPane() {
		if (conferenceScrollPane == null) {
			conferenceScrollPane = new JScrollPane();
			conferenceScrollPane.setViewportView(getConferenceIdsList());
		}
		return conferenceScrollPane;
	}

	private JList getConferenceIdsList() {
		if (conferenceIdsList == null) {
			conferenceIdsList = new JList();
			DefaultListModel listModel = new DefaultListModel();
			conferenceIdsList.setModel(listModel);
		}
		return conferenceIdsList;
	}

	private JButton getQuitConferenceButton() {
		if (quitConferenceButton == null) {
			quitConferenceButton = new JButton();
			quitConferenceButton.setText("quit");
		}
		return quitConferenceButton;
	}

	private JButton getLeaveConferenceButton() {
		if (leaveConferenceButton == null) {
			leaveConferenceButton = new JButton();
			leaveConferenceButton.setText("leave");
		}
		return leaveConferenceButton;
	}

	private JButton getGetAllConferenceCallsButton() {
		if (getAllConferenceCallsButton == null) {
			getAllConferenceCallsButton = new JButton();
			getAllConferenceCallsButton.setText("getAllConferenceCallsInRoom");
		}
		return getAllConferenceCallsButton;
	}

	private JButton getGetAllConferenceIdsButton() {
		if (getAllConferenceIdsButton == null) {
			getAllConferenceIdsButton = new JButton();
			getAllConferenceIdsButton.setText("getAllConferences");
		}
		return getAllConferenceIdsButton;
	}

	private JButton getOnlyOneCanSpeakInRoom() {
		if (onlyOneCanSpeakInRoomButton == null) {
			onlyOneCanSpeakInRoomButton = new JButton();
			onlyOneCanSpeakInRoomButton.setText("onlyOneCanSpeak");
		}
		return onlyOneCanSpeakInRoomButton;
	}

	private JButton getAllCanSpeakInRoom() {
		if (allCanSpeakInRoomButton == null) {
			allCanSpeakInRoomButton = new JButton();
			allCanSpeakInRoomButton.setText("allCanSpeak");
		}
		return allCanSpeakInRoomButton;
	}

	private JButton getAllCanHearInRoomButton() {
		if (allCanHearInRoomButton == null) {
			allCanHearInRoomButton = new JButton();
			allCanHearInRoomButton.setText("allCanHear");
		}
		return allCanHearInRoomButton;
	}

	private JButton getStartManagedConferenceButton() {
		if (startManagedConferenceButton == null) {
			startManagedConferenceButton = new JButton();
			startManagedConferenceButton.setText("start managed");
		}
		return startManagedConferenceButton;
	}

	private JButton getCreateUnmanagedConferenceButton() {
		if (createUnmanagedConferenceButton == null) {
			createUnmanagedConferenceButton = new JButton();
			createUnmanagedConferenceButton.setText("create unmanaged");
		}
		return createUnmanagedConferenceButton;
	}

	private JButton getHangupCallButton() {
		if (hangupCallButton == null) {
			hangupCallButton = new JButton();
			hangupCallButton.setText("hangup");
		}
		return hangupCallButton;
	}

	//	private JButton getMediateToConferenceButton()
	//	{
	//		if (mediateToConferenceButton == null)
	//		{
	//			mediateToConferenceButton = new JButton();
	//			mediateToConferenceButton.setText("mediate");
	//		}
	//		return mediateToConferenceButton;
	//	}

	private JButton getTransferToConferenceButton() {
		if (transferToConferenceButton == null) {
			transferToConferenceButton = new JButton();
			transferToConferenceButton.setText("transfer");
		}
		return transferToConferenceButton;
	}

	private JButton getCallNumberToConferenceButton() {
		if (callNumberToConferenceButton == null) {
			callNumberToConferenceButton = new JButton();
			callNumberToConferenceButton.setText("directCall");
		}
		return callNumberToConferenceButton;
	}

	private JButton getJoinToConferenceButton() {
		if (joinToConferenceButton == null) {
			joinToConferenceButton = new JButton();
			joinToConferenceButton.setText("join");
		}
		return joinToConferenceButton;
	}

	private JButton getListentoConferenceButton() {
		if (listenToConferenceButton == null) {
			listenToConferenceButton = new JButton();
			listenToConferenceButton.setText("listen");
		}
		return listenToConferenceButton;
	}

	private JButton getMutetoConferenceButton() {
		if (muteToConferenceButton == null) {
			muteToConferenceButton = new JButton();
			muteToConferenceButton.setText("mute");
		}
		return muteToConferenceButton;
	}

	private JButton getWantToTalktoConferenceButton() {
		if (wantToTalkToConferenceButton == null) {
			wantToTalkToConferenceButton = new JButton();
			wantToTalkToConferenceButton.setText("wantTalk");
		}
		return wantToTalkToConferenceButton;
	}

	private void destinationNumberTextFocusFocusGained(FocusEvent event) {
		destinationNumberText.selectAll();
	}

	public void setViewModel(ConferenceCallViewModel viewModel) {
		initBindings(viewModel);
	}

	private void initBindings(final ConferenceCallViewModel viewModel) {

		getAllCanHearInRoomButton().addActionListener(viewModel.getAllCanHearInRoomActionListener());
		getAllCanSpeakInRoom().addActionListener(viewModel.getAllCanSpeakInRoomActionListener());
		getOnlyOneCanSpeakInRoom().addActionListener(viewModel.getOnlyOneCanSpeakInRoomActionListener());
		getStartManagedConferenceButton().addActionListener(viewModel.getStartManagedConferenceActionListener());
		getCreateUnmanagedConferenceButton().addActionListener(
			viewModel.getCreateUnmanagedConferenceActionListener());
		getGetAllConferenceIdsButton().addActionListener(viewModel.getGetAllConferenceActionListener());
		getGetAllConferenceCallsButton().addActionListener(viewModel.getGetAllConferenceCallsActionListener());
		getHangupCallButton().addActionListener(viewModel.getHangupCallActionListener());
		getListentoConferenceButton().addActionListener(viewModel.getListentoConferenceActionListener());
		getMutetoConferenceButton().addActionListener(viewModel.getMutetoConferenceActionListener());
		getWantToTalktoConferenceButton().addActionListener(viewModel.getWantToTalktoConferenceActionListener());

		getJoinToConferenceButton().addActionListener(viewModel.getJoinToConferenceActionListener());
		getLeaveConferenceButton().addActionListener(viewModel.getLeaveConferenceActionListener());
		getQuitConferenceButton().addActionListener(viewModel.getQuitConferenceActionListener());
		getTransferToConferenceButton().addActionListener(viewModel.getTransferToConferenceActionListener());
		//		getMediateToConferenceButton().addActionListener(viewModel.getMediateToConferenceActionListener());
		getCallNumberToConferenceButton().addActionListener(viewModel.getCallNumberToConferenceActionListener());

		BeanAdapter<ConferenceCallViewModel> beanAdapter = new BeanAdapter<>(viewModel,
			true);

		callTable.setModel(viewModel.getCallTableModel());
		callTable.setSelectionModel(new MultiListSelectionAdapter(viewModel.getSelectedCallTableRows()));
		callTable.getSelectionModel().addListSelectionListener(viewModel.getSelectedCallTableRowsListener());
		callTable.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		callTable.removeColumn(callTable.getColumn(CallProperties.timestamp.toString()));
		callTable.removeColumn(callTable.getColumn(CallProperties.groupId.toString()));
		callTable.removeColumn(callTable.getColumn(CallProperties.peerNames.toString()));
		callTable.removeColumn(callTable.getColumn(CallProperties.channelNames.toString()));
		callTable.removeColumn(callTable.getColumn(CallProperties.sipCallIds.toString()));
		callTable.removeColumn(callTable.getColumn(CallProperties.caller.toString()));
		callTable.removeColumn(callTable.getColumn(CallProperties.connectedTimestamp.toString()));
		callTable.removeColumn(callTable.getColumn(CallProperties.recorder.toString()));
		callTable.removeColumn(callTable.getColumn(CallProperties.recorded.toString()));
		callTable.removeColumn(callTable.getColumn(CallProperties.recordOwner.toString()));
		callTable.removeColumn(callTable.getColumn(CallProperties.avatarHash.toString()));
		callTable.removeColumn(callTable.getColumn(CallProperties.doorlineCamUrl.toString()));
		callTable.removeColumn(callTable.getColumn(CallProperties.doorlineDtmfCode.toString()));
		callTable.removeColumn(callTable.getColumn(CallProperties.doorlineImageProviderId.toString()));
		callTable.removeColumn(callTable.getColumn(CallProperties.forwardType.toString()));
		callTable.removeColumn(callTable.getColumn(CallProperties.forwarderCallerIdName.toString()));
		callTable.removeColumn(callTable.getColumn(CallProperties.forwarderCallerIdNumber.toString()));
		callTable.removeColumn(callTable.getColumn(CallProperties.conferenceRoomId.toString()));
		callTable.removeColumn(callTable.getColumn(CallProperties.conferenceAdmin.toString()));
		callTable.removeColumn(callTable.getColumn(CallProperties.consultationCall.toString()));
		callTable.removeColumn(callTable.getColumn(CallProperties.referenceOfConsultation.toString()));
		callTable.removeColumn(callTable.getColumn(CallProperties.duration.toString()));
		callTable.removeColumn(callTable.getColumn(CallProperties.jabberId.toString()));

		conferenceCallTable.setModel(viewModel.getConferenceCallTableModel());
		conferenceCallTable.setSelectionModel(new MultiListSelectionAdapter(
			viewModel.getSelectedConferenceCallTableRows()));
		conferenceCallTable.getSelectionModel().addListSelectionListener(
			viewModel.getSelectedConferenceCallTableRowsListener());
		conferenceCallTable.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		conferenceCallTable.removeColumn(conferenceCallTable.getColumn(ConferenceCallProperties.state.toString()));
		conferenceCallTable.removeColumn(conferenceCallTable.getColumn(ConferenceCallProperties.enteredTimestamp.toString()));

		Bindings.bind(destinationNumberText,
			beanAdapter.getValueModel(ConferenceCallViewModel.PROPERTY_DESTINATION_NUMBER));

		Bindings.bind(
			conferenceIdsList,
			new SelectionInList<String>(
				beanAdapter.getValueModel(ConferenceCallViewModel.PROPERTY_CONFERENCE_IDS),
				beanAdapter.getValueModel(ConferenceCallViewModel.PROPERTY_SELECTED_CONFERENCE_ID)));
	}
}
