/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.examples.uci30.swing.view;

import de.starface.examples.uci30.swing.viewmodel.DemoClientViewModel;
import de.starface.examples.uci30.swing.viewmodel.GeneralViewTasks;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintStream;

import javax.swing.*;
import javax.swing.filechooser.FileFilter;

/**
 * This is a visual UCI client that can be used to manually send the UCI requests and track UCI
 * events that are received.
 *
 * It can also be used as an example on how to use the STARFACE UCI and UciProxy implementation in
 * Java programs.
 *
 * @author tilmann
 */
// VS4E -- DO NOT REMOVE THIS LINE!
public class DemoClientFrame extends JFrame implements GeneralViewTasks {

	private static final long serialVersionUID = 1L;

	private DemoClientViewModel viewModel;

	private JTabbedPane tabPanel;
	private JTextArea textOutput;
	private JScrollPane textOutputScrollPane;
	private CallPanel callTab;
	private CcbsCallPanel ccbsCallTab;
	private MonitorCallPanel monitorCallPanel;
	private ConferenceCallPanel conferenceCallTab;
	private CallListPanel callListTab;
	private FaxPanel faxTab;
	private LoginPanel loginPanel;
	private FaxListPanel faxListTab;
	private VoicemailListPanel voicemailListTab;
	private RedirectPanel redirectTab;
	private GroupPanel groupTab;
	private QueuePanel queueTab;
	private FunctionKeyPanel functionKeyTab;
	private UserStatePanel userStateTab;
	private PhonePanel phoneTab;
	private JPanel jPanelTextOutputPanel;
	private PermissionsPanel permissionsTab;
	private SystemSettingsPanel systemSettingsTab;
	private FmcPanel fmcPhonesTabs;

	public static final String PREFERRED_LOOK_AND_FEEL = "com.sun.java.swing.plaf.nimbus.NimbusLookAndFeel";

	public DemoClientFrame() {
		initComponents();
	}

	private void initComponents() {
		setTitle("STARFACE UCI 3.0 Demo Client");
		setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
		add(getTextOutputScrollPane(), BorderLayout.CENTER);
		add(getTabPanel(), BorderLayout.NORTH);
		add(getLoginPanel(), BorderLayout.SOUTH);
		addWindowListener(new WindowAdapter() {

			@Override
			public void windowClosing(WindowEvent event) {
				windowWindowClosing(event);
			}
		});
		setSize(850, 800);
	}

	private SystemSettingsPanel getSystemSettingsTab() {
		if (systemSettingsTab == null) {
			systemSettingsTab = new SystemSettingsPanel();
			systemSettingsTab.setVisible(false);
			systemSettingsTab.setMinimumSize(new Dimension(570, 450));
			systemSettingsTab.setPreferredSize(new Dimension(570, 450));
		}
		return systemSettingsTab;
	}

	private UserStatePanel getUserStateTab() {
		if (userStateTab == null) {
			userStateTab = new UserStatePanel();
			userStateTab.setVisible(false);
		}
		return userStateTab;
	}

	private PhonePanel getPhoneTab() {
		if (phoneTab == null) {
			phoneTab = new PhonePanel();
			phoneTab.setVisible(false);
			phoneTab.setMinimumSize(new Dimension(570, 450));
			phoneTab.setPreferredSize(new Dimension(570, 450));
		}
		return phoneTab;
	}

	private PermissionsPanel getPermissionsTab() {
		if (permissionsTab == null) {
			permissionsTab = new PermissionsPanel();
			permissionsTab.setVisible(false);
			permissionsTab.setMinimumSize(new Dimension(570, 450));
			permissionsTab.setPreferredSize(new Dimension(570, 450));
		}
		return permissionsTab;

	}

	private FunctionKeyPanel getFunctionKeyTab() {
		if (functionKeyTab == null) {
			functionKeyTab = new FunctionKeyPanel();
			functionKeyTab.setMinimumSize(new Dimension(570, 450));
			functionKeyTab.setPreferredSize(new Dimension(570, 450));
		}
		return functionKeyTab;
	}

	private RedirectPanel getRedirectTab() {
		if (redirectTab == null) {
			redirectTab = new RedirectPanel();
			redirectTab.setMinimumSize(new Dimension(570, 450));
			redirectTab.setPreferredSize(new Dimension(570, 450));
		}
		return redirectTab;
	}

	private GroupPanel getGroupTab() {
		if (groupTab == null) {
			groupTab = new GroupPanel();
			groupTab.setMinimumSize(new Dimension(570, 450));
			groupTab.setPreferredSize(new Dimension(570, 450));
		}
		return groupTab;
	}

	private QueuePanel getQueueTab() {
		if (queueTab == null) {
			queueTab = new QueuePanel();
			queueTab.setMinimumSize(new Dimension(570, 450));
			queueTab.setPreferredSize(new Dimension(570, 450));
		}
		return queueTab;
	}

	private VoicemailListPanel getVoicemailListTab() {
		if (voicemailListTab == null) {
			voicemailListTab = new VoicemailListPanel();
		}
		return voicemailListTab;
	}

	private FaxListPanel getFaxListTab() {
		if (faxListTab == null) {
			faxListTab = new FaxListPanel();
		}
		return faxListTab;
	}

	private JPanel getScrollPanel() {
		JPanel panel = new JPanel();
		panel.add(getScrollPanel());
		return panel;
	}

	private LoginPanel getLoginPanel() {
		if (loginPanel == null) {
			loginPanel = new LoginPanel();
			loginPanel.setMinimumSize(new Dimension(558, 180));
			loginPanel.setPreferredSize(new Dimension(558, 180));
		}
		return loginPanel;
	}

	private FaxPanel getFaxTab() {
		if (faxTab == null) {
			faxTab = new FaxPanel();
		}
		return faxTab;
	}

	private CallListPanel getCallListTab() {
		if (callListTab == null) {
			callListTab = new CallListPanel();
		}
		return callListTab;
	}

	private CallPanel getCallTab() {
		if (callTab == null) {
			callTab = new CallPanel();
		}
		return callTab;
	}

	private CcbsCallPanel getCcbsCallTab() {
		if (ccbsCallTab == null) {
			ccbsCallTab = new CcbsCallPanel();
		}
		return ccbsCallTab;
	}

	private MonitorCallPanel getMonitorCallPanel() {
		if (monitorCallPanel == null) {
			monitorCallPanel = new MonitorCallPanel();
		}
		return monitorCallPanel;
	}

	private ConferenceCallPanel getConferenceCallTab() {
		if (conferenceCallTab == null) {
			conferenceCallTab = new ConferenceCallPanel();
		}
		return conferenceCallTab;
	}

	private FmcPanel getFmcPhonesTab() {
		if (fmcPhonesTabs == null) {
			fmcPhonesTabs = new FmcPanel();
		}
		return fmcPhonesTabs;
	}

	private JPanel getTextOutputScrollPane() {
		if (textOutputScrollPane == null) {
			jPanelTextOutputPanel = new JPanel();
			java.awt.BorderLayout borderLayout = new BorderLayout();
			jPanelTextOutputPanel.setLayout(borderLayout);

			textOutputScrollPane = new JScrollPane();
			textOutputScrollPane.setViewportView(getTextOutput());

			JButton clearTextBoxButton = new JButton("Clear");
			clearTextBoxButton.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent e) {
					getTextOutput().setText("");
				}
			});
			jPanelTextOutputPanel.add(textOutputScrollPane, BorderLayout.CENTER);
			jPanelTextOutputPanel.add(clearTextBoxButton, BorderLayout.SOUTH);
		}
		return jPanelTextOutputPanel;
	}

	private JTextArea getTextOutput() {
		if (textOutput == null) {
			textOutput = new JTextArea();
			textOutput.setEditable(false);
			textOutput.setFont(new Font("Monospaced", Font.PLAIN, 12));
			textOutput.setAutoscrolls(true);
		}
		return textOutput;
	}

	private JTabbedPane getTabPanel() {
		if (tabPanel == null) {
			tabPanel = new JTabbedPane();
			tabPanel.setVisible(false);
			tabPanel.addTab("Call", getCallTab());
			tabPanel.addTab("CcbsCall", getCcbsCallTab());
			tabPanel.addTab("Conference", getConferenceCallTab());
			tabPanel.addTab("CallList", getCallListTab());
			tabPanel.addTab("Fax", getFaxTab());
			tabPanel.addTab("FaxList", getFaxListTab());
			tabPanel.addTab("VoicemailList", getVoicemailListTab());
			tabPanel.addTab("MonitorList", getMonitorCallPanel());
			tabPanel.addTab("Redirect", getRedirectTab());
			tabPanel.addTab("FunctionKey", getFunctionKeyTab());
			tabPanel.addTab("UserState", getUserStateTab());
			tabPanel.addTab("Group", getGroupTab());
			tabPanel.addTab("Queue", getQueueTab());
			tabPanel.addTab("Phones", getPhoneTab());
			tabPanel.addTab("Permissions", getPermissionsTab());
			tabPanel.addTab("System Settings", getSystemSettingsTab());
			tabPanel.addTab("iFMC Phones", getFmcPhonesTab());
		}
		return tabPanel;
	}

	public void redirectStandardOutputToWindow() {
		final PrintStream areaPrintStream = new PrintStream(new OutputStream() {
			@Override
			public void write(byte b[]) throws IOException {
				String aString = new String(b);
				appendString(aString);
			}

			@Override
			public void write(byte b[], int off, int len) throws IOException {
				String aString = new String(b, off, len);
				appendString(aString);
			}

			@Override
			public void write(int i) throws IOException {
				byte[] b = {(byte) i};
				String aString = new String(b);
				appendString(aString);
			}

			private void appendString(final String aString) {
				SwingUtilities.invokeLater(new Runnable() {
					@Override
					public void run() {
						textOutput.append(aString);
						textOutput.setCaretPosition(textOutput.getDocument().getLength());
					}
				});
			}
		});
		System.setOut(areaPrintStream);
		System.setErr(areaPrintStream);
	}

	private void windowWindowClosing(WindowEvent event) {
		viewModel.disconnect();
	}

	private void changeToConnectedMode() {
		loginPanel.setVisible(false);
		tabPanel.setVisible(true);
	}

	private void changeToLoginMode() {
		tabPanel.setVisible(false);
		loginPanel.setVisible(true);
		loginPanel.requestFocus();
	}

	public void setViewModel(DemoClientViewModel viewModel) {
		this.viewModel = viewModel;
		callListTab.setViewModel(viewModel.getCallListViewModel());
		callTab.setViewModel(viewModel.getCallViewModel());
		ccbsCallTab.setViewModel(viewModel.getCcbsCallViewModel());
		monitorCallPanel.setViewModel(viewModel.getMonitorCallViewModel());
		conferenceCallTab.setViewModel(viewModel.getConferenceCallViewModel());
		faxListTab.setViewModel(viewModel.getFaxListViewModel());
		functionKeyTab.setViewModel(viewModel.getFunctionKeyViewModel());
		voicemailListTab.setViewModel(viewModel.getVoicemailListViewModel());
		phoneTab.setViewModel(viewModel.getPhoneViewModel());
		loginPanel.setViewModel(viewModel.getLoginViewModel());
		userStateTab.setViewModel(viewModel.getUserStateViewModel());
		faxTab.setViewModel(viewModel.getFaxViewModel());
		groupTab.setViewModel(viewModel.getGroupViewModel());
		queueTab.setViewModel(viewModel.getQueueViewModel());
		redirectTab.setViewModel(viewModel.getRedirectViewModel());
		permissionsTab.setViewModel(viewModel.getPermissionsVierModel());
		systemSettingsTab.setModel(viewModel.getSystemSettingViewModel());
		fmcPhonesTabs.setViewModel(viewModel.getFmcViewModel());
		subscribeEvents();
		updateDisplayMode();
	}

	private void subscribeEvents() {
		viewModel.setEventListener(new DemoClientViewModel.Events() {
			@Override
			public void connectedChanged() {
				updateDisplayMode();
			}
		});
	}

	private void getMWIState() {

	}

	private void updateDisplayMode() {
		if (viewModel.isConnected()) {
			changeToConnectedMode();
			if (viewModel.getLoginViewModel() != null) {
				setTitle("STARFACE UCI 3.0 Demo Client, " + viewModel.getLoginViewModel().getLoginId() + "@"
					+ viewModel.getLoginViewModel().getXmppHost());
			} else {
				setTitle("STARFACE UCI 3.0 Demo Client");
			}
			getMWIState();
		} else {
			changeToLoginMode();
		}
	}

	@Override
	public File selectFile(FileFilter fileFilter) {
		JFileChooser fileChooser = new JFileChooser();
		fileChooser.setFileFilter(fileFilter);
		int result = fileChooser.showOpenDialog(this);
		if (result == JFileChooser.APPROVE_OPTION) {
			return fileChooser.getSelectedFile();
		} else {
			return null;
		}
	}

	@Override
	public void displayInfo(String title, String content) {
		JOptionPane.showMessageDialog(this, content, title, JOptionPane.INFORMATION_MESSAGE);
	}

}
