/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.examples.uci30.swing.view;

import com.jgoodies.binding.adapter.SingleListSelectionAdapter;
import com.jgoodies.binding.beans.BeanAdapter;

import de.starface.examples.uci30.swing.viewmodel.GroupViewModel;

import org.dyno.visual.swing.layouts.Bilateral;
import org.dyno.visual.swing.layouts.Constraints;
import org.dyno.visual.swing.layouts.GroupLayout;
import org.dyno.visual.swing.layouts.Leading;

import java.awt.*;

import javax.swing.*;
import javax.swing.table.DefaultTableModel;

/**
 * This panel provides a GUI for the UCI communication group functionality
 *
 * @author tilmann
 */
// VS4E -- DO NOT REMOVE THIS LINE!
public class GroupPanel extends JPanel {

	private static final long serialVersionUID = 1L;

	private JTable table;
	private JScrollPane tableScrollPane;
	private JButton setGroupSettingsButton;
	private JButton getGroupSettingsButton;
	private JButton setGroupSettingButton;

	@SuppressWarnings("unused")
	private static final String PREFERRED_LOOK_AND_FEEL = "com.sun.java.swing.plaf.nimbus.NimbusLookAndFeel";

	public GroupPanel() {
		initComponents();
	}

	private void initComponents() {
		setMinimumSize(new Dimension(570, 450));
		setPreferredSize(new Dimension(570, 450));
		setLayout(new GroupLayout());
		add(getGetGroupSettingsButton(), new Constraints(new Leading(6, 184, 6, 6), new Leading(6, 6, 6)));
		add(getTableScrollPane(), new Constraints(new Bilateral(6, 10, 558), new Leading(40, 401, 6, 6)));
		add(getSetGroupSettingsButton(), new Constraints(new Leading(422, 140, 10, 10), new Leading(6, 6, 6)));
		add(getSetGroupSettingButton(), new Constraints(new Leading(276, 140, 6, 6), new Leading(6, 6, 6)));
		setSize(570, 450);
	}

	private JButton getSetGroupSettingButton() {
		if (setGroupSettingButton == null) {
			setGroupSettingButton = new JButton();
			setGroupSettingButton.setText("setGroupSetting");
		}
		return setGroupSettingButton;
	}

	private JButton getSetGroupSettingsButton() {
		if (setGroupSettingsButton == null) {
			setGroupSettingsButton = new JButton();
			setGroupSettingsButton.setText("setGroupSettings");
		}
		return setGroupSettingsButton;
	}

	private JButton getGetGroupSettingsButton() {
		if (getGroupSettingsButton == null) {
			getGroupSettingsButton = new JButton();
			getGroupSettingsButton.setText("getGroupSettings");
		}
		return getGroupSettingsButton;
	}

	private JScrollPane getTableScrollPane() {
		if (tableScrollPane == null) {
			tableScrollPane = new JScrollPane();
			tableScrollPane.setMinimumSize(new Dimension(570, 470));
			tableScrollPane.setPreferredSize(new Dimension(570, 470));
			tableScrollPane.setViewportView(getTable());
		}
		return tableScrollPane;
	}

	private JTable getTable() {
		if (table == null) {
			table = new JTable();
			table.setModel(new DefaultTableModel(new Object[][]{}, new String[]{}) {
				private static final long serialVersionUID = 1L;
				Class<?>[] types = new Class<?>[]{};

				@Override
				public Class<?> getColumnClass(int columnIndex) {
					return types[columnIndex];
				}
			});
			table.setRowSelectionAllowed(true);
			table.setColumnSelectionAllowed(false);
			table.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		}
		return table;
	}

	public void setViewModel(GroupViewModel viewModel) {
		initBindings(viewModel);
	}

	private void initBindings(final GroupViewModel viewModel) {
		setGroupSettingsButton.addActionListener(viewModel.getSetGroupSettingsActionListener());
		setGroupSettingButton.addActionListener(viewModel.getSetGroupSettingActionListener());
		getGroupSettingsButton.addActionListener(viewModel.getGetGroupSettingsActionListener());

		BeanAdapter<GroupViewModel> beanAdapter = new BeanAdapter<>(viewModel, true);

		table.setModel(viewModel.getGroupSettingsTableModel());
		table.setSelectionModel(new SingleListSelectionAdapter(
			beanAdapter.getValueModel(GroupViewModel.PROPERTY_SELECTED_ROW)));
	}

}
