/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.examples.uci30.swing.view;

import com.jgoodies.binding.adapter.Bindings;
import com.jgoodies.binding.beans.BeanAdapter;
import com.jgoodies.binding.value.ValueModel;

import de.starface.examples.uci30.swing.DemoClient;
import de.starface.examples.uci30.swing.model.Config;
import de.starface.examples.uci30.swing.viewmodel.LoginViewModel;

import org.dyno.visual.swing.layouts.Bilateral;
import org.dyno.visual.swing.layouts.Constraints;
import org.dyno.visual.swing.layouts.GroupLayout;
import org.dyno.visual.swing.layouts.Leading;

import java.awt.*;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.util.ArrayList;
import java.util.List;

import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * This panel provides a GUI that can be used to obtain UCI connection information from the user.
 *
 * @author tilmann
 */
// VS4E -- DO NOT REMOVE THIS LINE!
public class LoginPanel extends JPanel {

	private static final long serialVersionUID = 1L;

	private JRadioButton xmppConnectRadio;
	private JLabel xmppHostLabel;
	private JRadioButton httpConnectRadio;
	private JLabel httpHostLabel;
	private JTextField httpHostText;
	private JLabel httpCallbackHostLabel;
	private JTextField httpCallbackHostText;
	private JTextField xmppHostText;
	private JTextField loginIdText;
	private JPasswordField passwordText;
	private JLabel loginIdLabel;
	private JLabel passwordLabel;
	private JButton connectButton;
	@SuppressWarnings("unused")
	private static final String PREFERRED_LOOK_AND_FEEL = "com.sun.java.swing.plaf.nimbus.NimbusLookAndFeel";

	public LoginPanel() {
		initComponents();
		setFocusTraversalPolicy();
	}

	private void setFocusTraversalPolicy() {
		setFocusTraversalPolicy(new FocusTraversalPolicy() {
			private List<Component> order = new ArrayList<>();

			{
				order.add(xmppConnectRadio);
				order.add(xmppHostText);
				order.add(loginIdText);
				order.add(passwordText);
				order.add(connectButton);
				order.add(httpConnectRadio);
				order.add(httpHostText);
				order.add(httpCallbackHostText);
				order.add(loginIdText);
				order.add(passwordText);
				order.add(connectButton);
			}

			@Override
			public Component getComponentAfter(Container aContainer, Component aComponent) {
				int currentIndex = order.indexOf(aComponent);
				int nextIndex = Math.min(currentIndex + 1, order.size() - 1);
				Component nextComponent = order.get(nextIndex);
				while (!nextComponent.isEnabled() && nextIndex < order.size() - 1) {
					nextIndex++;
					nextComponent = order.get(nextIndex);
				}
				return nextComponent;
			}

			@Override
			public Component getComponentBefore(Container aContainer, Component aComponent) {
				int currentIndex = order.indexOf(aComponent);
				int nextIndex = Math.max(currentIndex - 1, 0);
				Component nextComponent = order.get(nextIndex);
				while (!nextComponent.isEnabled() && nextIndex > 0) {
					nextIndex--;
					nextComponent = order.get(nextIndex);
				}
				return nextComponent;
			}

			@Override
			public Component getDefaultComponent(Container aContainer) {
				return xmppHostText;
			}

			@Override
			public Component getFirstComponent(Container aContainer) {
				return xmppConnectRadio;
			}

			@Override
			public Component getLastComponent(Container aContainer) {
				return connectButton;
			}

		});
	}

	private void initComponents() {
		setMinimumSize(new Dimension(570, 180));
		setPreferredSize(new Dimension(570, 180));
		setFocusTraversalPolicyProvider(true);
		setFocusCycleRoot(true);
		setLayout(new GroupLayout());
		add(getXmppConnectRadio(), new Constraints(new Leading(12, 12, 12), new Leading(12, 12, 12)));
		add(getXmppHostLabel(), new Constraints(new Leading(34, 10, 10), new Leading(40, 12, 12)));
		add(getHttpConnectRadio(), new Constraints(new Leading(12, 12, 12), new Leading(78, 10, 10)));
		add(getHttpHostLabel(), new Constraints(new Leading(34, 12, 12), new Leading(105, 10, 10)));
		add(getHttpCallbackHostLabel(), new Constraints(new Leading(34, 12, 12), new Leading(139, 12, 12)));
		add(getHttpCallbackHostText(), new Constraints(new Leading(123, 164, 12, 12), new Leading(133, 12, 12)));
		add(getXmppHostText(), new Constraints(new Leading(123, 164, 12, 12), new Leading(34, 12, 12)));
		add(getLoginIdLabel(), new Constraints(new Leading(317, 12, 12), new Leading(40, 12, 12)));
		add(getPasswordLabel(), new Constraints(new Leading(317, 12, 12), new Leading(74, 12, 12)));
		add(getHttpHostText(), new Constraints(new Leading(123, 164, 12, 12), new Leading(99, 12, 12)));
		add(getLoginIdText(), new Constraints(new Bilateral(389, 12, 12), new Leading(34, 12, 12)));
		add(getPasswordText(), new Constraints(new Bilateral(389, 12, 12), new Leading(68, 12, 12)));
		add(getConnectButton(), new Constraints(new Bilateral(317, 12, 174), new Leading(102, 59, 12, 12)));
		setSize(570, 180);
	}

	private JButton getConnectButton() {
		if (connectButton == null) {
			connectButton = new JButton();
			connectButton.setText("connectAsync");
		}
		return connectButton;
	}

	private JLabel getPasswordLabel() {
		if (passwordLabel == null) {
			passwordLabel = new JLabel();
			passwordLabel.setText("Password");
		}
		return passwordLabel;
	}

	private JLabel getLoginIdLabel() {
		if (loginIdLabel == null) {
			loginIdLabel = new JLabel();
			loginIdLabel.setText("Login ID");
		}
		return loginIdLabel;
	}

	private JPasswordField getPasswordText() {
		if (passwordText == null) {
			passwordText = new JPasswordField();
			passwordText.setText(Config.getValue("password"));
			passwordText.addFocusListener(new FocusAdapter() {

				@Override
				public void focusGained(FocusEvent event) {
					passwordTextFocusFocusGained(event);
				}
			});
		}
		return passwordText;
	}

	private JTextField getLoginIdText() {
		if (loginIdText == null) {
			loginIdText = new JTextField();
			loginIdText.setText(Config.getValue("loginId"));
			loginIdText.addFocusListener(new FocusAdapter() {

				@Override
				public void focusGained(FocusEvent event) {
					loginIdTextFocusFocusGained(event);
				}
			});
		}
		return loginIdText;
	}

	private JTextField getXmppHostText() {
		if (xmppHostText == null) {
			xmppHostText = new JTextField();
			xmppHostText.setText(Config.getValue("xmppHost"));
			xmppHostText.addFocusListener(new FocusAdapter() {

				@Override
				public void focusGained(FocusEvent event) {
					xmppHostTextFocusFocusGained(event);
				}
			});
		}
		return xmppHostText;
	}

	private JTextField getHttpCallbackHostText() {
		if (httpCallbackHostText == null) {
			httpCallbackHostText = new JTextField();
			httpCallbackHostText.setEnabled(false);
			httpCallbackHostText.setText(Config.getValue("httpCallbackHost"));
			httpCallbackHostText.addFocusListener(new FocusAdapter() {

				@Override
				public void focusGained(FocusEvent event) {
					httpCallbackHostTextFocusFocusGained(event);
				}
			});
		}
		return httpCallbackHostText;
	}

	private JLabel getHttpCallbackHostLabel() {
		if (httpCallbackHostLabel == null) {
			httpCallbackHostLabel = new JLabel();
			httpCallbackHostLabel.setText("Callback Host");
		}
		return httpCallbackHostLabel;
	}

	private JTextField getHttpHostText() {
		if (httpHostText == null) {
			httpHostText = new JTextField();
			httpHostText.setEnabled(false);
			httpHostText.setText(Config.getValue("httpHost"));
			httpHostText.addFocusListener(new FocusAdapter() {

				@Override
				public void focusGained(FocusEvent event) {
					httpHostTextFocusFocusGained(event);
				}
			});
		}
		return httpHostText;
	}

	private JLabel getHttpHostLabel() {
		if (httpHostLabel == null) {
			httpHostLabel = new JLabel();
			httpHostLabel.setText("HTTP Host");
		}
		return httpHostLabel;
	}

	private JRadioButton getHttpConnectRadio() {
		if (httpConnectRadio == null) {
			httpConnectRadio = new JRadioButton();
			httpConnectRadio.setText("HTTP Connection");
		}
		return httpConnectRadio;
	}

	private JLabel getXmppHostLabel() {
		if (xmppHostLabel == null) {
			xmppHostLabel = new JLabel();
			xmppHostLabel.setText("XMPP Host");
		}
		return xmppHostLabel;
	}

	private JRadioButton getXmppConnectRadio() {
		if (xmppConnectRadio == null) {
			xmppConnectRadio = new JRadioButton();
			xmppConnectRadio.setText("XMPP Connection");
			xmppConnectRadio.addChangeListener(new ChangeListener() {

				@Override
				public void stateChanged(ChangeEvent event) {
					xmppConnectRadioChangeStateChanged(event);
				}
			});
		}
		return xmppConnectRadio;
	}

	private void xmppConnectRadioChangeStateChanged(ChangeEvent event) {
		updateRadioButtonDependencies();
	}

	private void updateRadioButtonDependencies() {
		if (xmppConnectRadio.isSelected()) {
			xmppHostText.setEnabled(true);
			httpHostText.setEnabled(false);
			httpCallbackHostText.setEnabled(false);
		} else {
			xmppHostText.setEnabled(false);
			httpHostText.setEnabled(true);
			httpCallbackHostText.setEnabled(true);
		}
	}

	void setGuiDemoClient(DemoClient guiDemoClient) {
	}

	private void xmppHostTextFocusFocusGained(FocusEvent event) {
		xmppHostText.selectAll();
	}

	private void loginIdTextFocusFocusGained(FocusEvent event) {
		loginIdText.selectAll();
	}

	private void passwordTextFocusFocusGained(FocusEvent event) {
		passwordText.selectAll();
	}

	private void httpCallbackHostTextFocusFocusGained(FocusEvent event) {
		httpCallbackHostText.selectAll();
	}

	private void httpHostTextFocusFocusGained(FocusEvent event) {
		httpHostText.selectAll();
	}

	public void setViewModel(LoginViewModel viewModel) {
		initBindings(viewModel);
	}

	private void initBindings(LoginViewModel viewModel) {
		connectButton.addActionListener(viewModel.getConnectActionListener());

		BeanAdapter<LoginViewModel> beanAdapter = new BeanAdapter<>(viewModel, true);
		ValueModel useXmppVm = beanAdapter.getValueModel(LoginViewModel.PROPERTY_USE_XMPP);
		Bindings.bind(xmppConnectRadio, useXmppVm, true);
		Bindings.bind(httpConnectRadio, useXmppVm, false);
		Bindings.bind(xmppHostText, beanAdapter.getValueModel(LoginViewModel.PROPERTY_XMPP_HOST));
		Bindings.bind(httpHostText, beanAdapter.getValueModel(LoginViewModel.PROPERTY_HTTP_HOST));
		Bindings.bind(httpCallbackHostText,
			beanAdapter.getValueModel(LoginViewModel.PROPERTY_HTTP_CALL_BACK_HOST));
		Bindings.bind(loginIdText, beanAdapter.getValueModel(LoginViewModel.PROPERTY_LOGIN_ID));
		Bindings.bind(passwordText, beanAdapter.getValueModel(LoginViewModel.PROPERTY_PASSWORD));
	}

}
