/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.examples.uci30.swing.view;

import de.starface.examples.uci30.swing.viewmodel.QueueViewModel;
import de.starface.integration.uci.java.v30.values.QueueAgentStateProperties;
import de.starface.integration.uci.java.v30.values.QueueSettingProperties;

import org.dyno.visual.swing.layouts.Bilateral;
import org.dyno.visual.swing.layouts.Constraints;
import org.dyno.visual.swing.layouts.GroupLayout;
import org.dyno.visual.swing.layouts.Leading;
import org.mortbay.log.Log;

import java.awt.Component;
import java.awt.Dimension;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.table.DefaultTableModel;

import com.jgoodies.binding.adapter.Bindings;
import com.jgoodies.binding.adapter.SingleListSelectionAdapter;
import com.jgoodies.binding.beans.BeanAdapter;

/**
 * This panel provides a GUI for the UCI communication queue functionality
 *
 * @author Mojo
 */
// VS4E -- DO NOT REMOVE THIS LINE!
public class QueuePanel extends JPanel {

	private static final long serialVersionUID = 1L;

	JLabel statisticsCurrentCallersLabel = null;
	JLabel statisticsLabelFreeAgentsLabel = null;
	JLabel statisticsLabelCallsLabel = null;
	JLabel statisticsWaittimeLabel = null;
	JLabel statisticsCallersLabel = null;
	JLabel queueAgentsLabel = null;
	JLabel queueCallsLabel = null;
	JLabel queuesLabel = null;
	private JTable queueSettingsTable;
	private JTable queueAgentTable;
	private JTable queueCallTable;
	private JScrollPane queueTableScrollPane;
	private JScrollPane agentTableScrollPane;
	private JScrollPane callTableScrollPane;
	private JButton getQueueSettingsButton;
	private JButton getQueueAgentsButton;
	private JButton getQueueCallsButton;
	private JButton grabQueueCallerButton;

	@SuppressWarnings("unused")
	private static final String PREFERRED_LOOK_AND_FEEL = "com.sun.java.swing.plaf.nimbus.NimbusLookAndFeel";

	public QueuePanel() {
		initComponents();
	}

	private void initComponents() {
		setMinimumSize(new Dimension(570, 450));
		setPreferredSize(new Dimension(570, 450));
		setLayout(new GroupLayout());

		int xposstd = 10;
		int xposButtons = 100;
		int yposQueuesStart = 50;
		int yposAgentsStart = 250;
		int yposCallsStart = yposAgentsStart + 145;

		add(getStatisticsCurrentCallers(), new Constraints(new Leading(xposstd, 100, 6, 100), new Leading(15, 6, 6)));
		add(getStatisticsLabelFreeAgents(),
			new Constraints(new Leading(xposstd + 100, 100, 6, 100), new Leading(15, 6, 6)));
		add(getStatisticsLabelCalls(), new Constraints(new Leading(xposstd + 200, 100, 6, 100), new Leading(15, 6, 6)));
		add(getStatisticsLabelWaittime(), new Constraints(new Leading(xposstd + 300, 100, 6, 100), new Leading(15, 6, 6)));
		add(getStatisticsLabelCallers(), new Constraints(new Leading(xposstd + 400, 100, 6, 100), new Leading(15, 6, 6)));
		//Queues
		add(getExistingQueuesLabel(), new Constraints(new Leading(xposstd, 100, 6, 100), new Leading(
			yposQueuesStart + 5, 6, 6)));
		add(getGetQueueSettingsButton(), new Constraints(new Leading(xposButtons, 200, 6, 140), new Leading(
			yposQueuesStart, 6, 6)));
		add(getQueueTableScrollPane(), new Constraints(new Bilateral(xposstd, 40, 500), new Leading(
			yposQueuesStart + 30, 80, 6, 6)));
		//Agents
		add(getQueueAgentsLabel(), new Constraints(new Leading(xposstd, 100, 6, 100), new Leading(
			yposAgentsStart + 5, 6, 6)));
		add(getQueueAgentsButton(), new Constraints(new Leading(xposButtons, 200, 10, 140), new Leading(
			yposAgentsStart, 6, 6)));
		add(getAgentTableScrollPane(), new Constraints(new Bilateral(xposstd, 40, 500), new Leading(
			yposAgentsStart + 30, 100, 6, 6)));
		//calls
		add(getQueueCallsLabel(), new Constraints(new Leading(xposstd, 100, 6, 100), new Leading(
			yposCallsStart + 5, 6, 6)));
		add(getQueueCallsButton(), new Constraints(new Leading(xposButtons, 200, 10, 140), new Leading(
			yposCallsStart, 6, 6)));
		add(getGrabQueueCallerButton(), new Constraints(new Leading(xposButtons + 200, 200, 10, 140), new Leading(
				yposCallsStart, 6, 6)));
		add(getCallTableScrollPane(), new Constraints(new Bilateral(xposstd, 40, 500), new Leading(
			yposCallsStart + 30, 150, 6, 6)));

		setSize(570, 450);
	}

	private JLabel getStatisticsCurrentCallers() {
		if (statisticsCurrentCallersLabel == null) {
			statisticsCurrentCallersLabel = new JLabel();
			statisticsCurrentCallersLabel.setText("Callers: 0");
		}
		return statisticsCurrentCallersLabel;
	}

	private JLabel getStatisticsLabelFreeAgents() {
		if (statisticsLabelFreeAgentsLabel == null) {
			statisticsLabelFreeAgentsLabel = new JLabel();
			statisticsLabelFreeAgentsLabel.setText("Free Agents: 0");
		}
		return statisticsLabelFreeAgentsLabel;
	}

	private JLabel getStatisticsLabelCalls() {
		if (statisticsLabelCallsLabel == null) {
			statisticsLabelCallsLabel = new JLabel();
			statisticsLabelCallsLabel.setText("Calls: 0/0");
		}
		return statisticsLabelCallsLabel;
	}

	private JLabel getStatisticsLabelWaittime() {
		if (statisticsWaittimeLabel == null) {
			statisticsWaittimeLabel = new JLabel();
			statisticsWaittimeLabel.setText("Waittime: 0");
		}
		return statisticsWaittimeLabel;
	}

	private JLabel getStatisticsLabelCallers() {
		if (statisticsCallersLabel == null) {
			statisticsCallersLabel = new JLabel();
			statisticsCallersLabel.setText("Callers: 0");
		}
		return statisticsCallersLabel;
	}

	private Component getQueueAgentsLabel() {
		if (queueAgentsLabel == null) {
			queueAgentsLabel = new JLabel();
			queueAgentsLabel.setText("<html> Agents </html>");
		}
		return queueAgentsLabel;
	}

	private Component getQueueCallsLabel() {
		if (queueCallsLabel == null) {
			queueCallsLabel = new JLabel();
			queueCallsLabel.setText("<html> Calls </html>");
		}
		return queueCallsLabel;
	}

	private Component getExistingQueuesLabel() {

		if (queuesLabel == null) {
			queuesLabel = new JLabel();
			queuesLabel.setText("<html> Queues </html>");
		}
		return queuesLabel;
	}

	private JButton getQueueAgentsButton() {
		if (getQueueAgentsButton == null) {
			getQueueAgentsButton = new JButton();
			getQueueAgentsButton.setText("Fetch Agents");
		}
		return getQueueAgentsButton;
	}

	private JButton getQueueCallsButton() {
		if (getQueueCallsButton == null) {
			getQueueCallsButton = new JButton();
			getQueueCallsButton.setText("Fetch Calls");
		}
		return getQueueCallsButton;
	}

	private JButton getGrabQueueCallerButton() {
		if (grabQueueCallerButton == null) {
			grabQueueCallerButton = new JButton();
			grabQueueCallerButton.setText("Grab queue caller");
		}
		return grabQueueCallerButton;
	}

	private JButton getGetQueueSettingsButton() {
		if (getQueueSettingsButton == null) {
			getQueueSettingsButton = new JButton();
			getQueueSettingsButton.setText("Fetch Queues");
		}
		return getQueueSettingsButton;
	}

	private JScrollPane getCallTableScrollPane() {
		if (callTableScrollPane == null) {
			callTableScrollPane = new JScrollPane(getCallTable());
			//			agentTableScrollPane.setMinimumSize(new Dimension(250, 470));
			//			agentTableScrollPane.setPreferredSize(new Dimension(250, 470));
			//			agentTableScrollPane.setViewportView(getAgentTable());
		}
		return callTableScrollPane;
	}

	private JScrollPane getAgentTableScrollPane() {
		if (agentTableScrollPane == null) {
			agentTableScrollPane = new JScrollPane(getAgentTable());
			//			agentTableScrollPane.setMinimumSize(new Dimension(250, 470));
			//			agentTableScrollPane.setPreferredSize(new Dimension(250, 470));
			//			agentTableScrollPane.setViewportView(getAgentTable());
		}
		return agentTableScrollPane;
	}

	private JScrollPane getQueueTableScrollPane() {
		if (queueTableScrollPane == null) {
			queueTableScrollPane = new JScrollPane(getQueueTable());
			//			queueTableScrollPane.setMinimumSize(new Dimension(250, 470));
			//			queueTableScrollPane.setPreferredSize(new Dimension(250, 470));
			//			queueTableScrollPane.setViewportView(getQueueTable());
		}
		return queueTableScrollPane;
	}

	private JTable getAgentTable() {
		if (queueAgentTable == null) {
			queueAgentTable = new JTable();
			queueAgentTable.setModel(new DefaultTableModel(new Object[][]{}, new String[]{}) {
				private static final long serialVersionUID = 1L;
				Class<?>[] types = new Class<?>[]{};

				@Override
				public Class<?> getColumnClass(int columnIndex) {
					return types[columnIndex];
				}
			});
			queueAgentTable.setRowSelectionAllowed(true);
			queueAgentTable.setColumnSelectionAllowed(false);
			//			queueAgentTable.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		}
		return queueAgentTable;
	}

	private JTable getCallTable() {
		if (queueCallTable == null) {
			queueCallTable = new JTable();
			queueCallTable.setModel(new DefaultTableModel(new Object[][]{}, new String[]{}) {
				private static final long serialVersionUID = 1L;
				Class<?>[] types = new Class<?>[]{};

				@Override
				public Class<?> getColumnClass(int columnIndex) {
					return types[columnIndex];
				}
			});
			queueCallTable.setRowSelectionAllowed(true);
			queueCallTable.setColumnSelectionAllowed(false);
			//			queueAgentTable.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		}
		return queueCallTable;
	}

	private JTable getQueueTable() {
		if (queueSettingsTable == null) {
			queueSettingsTable = new JTable();
			queueSettingsTable.setModel(new DefaultTableModel(new Object[][]{}, new String[]{}) {
				private static final long serialVersionUID = 1L;
				Class<?>[] types = new Class<?>[]{};

				@Override
				public Class<?> getColumnClass(int columnIndex) {
					return types[columnIndex];
				}
			});
			queueSettingsTable.setRowSelectionAllowed(true);
			queueSettingsTable.setColumnSelectionAllowed(false);
			queueSettingsTable.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		}
		return queueSettingsTable;
	}

	public void setViewModel(QueueViewModel viewModel) {
		initBindings(viewModel);
	}

	private void initBindings(final QueueViewModel viewModel) {

		getQueueSettingsButton.addActionListener(viewModel.getGetQueueSettingsActionListener());
		getQueueAgentsButton.addActionListener(viewModel.getGetQueueAgentsActionListener());
		getQueueCallsButton.addActionListener(viewModel.getGetQueueCallsActionListener());
		getGrabQueueCallerButton().addActionListener(viewModel.getGrabQueueCallerButtonListener());

		BeanAdapter<QueueViewModel> beanAdapter = new BeanAdapter<>(viewModel, true);
		try {
			Bindings.bind(getStatisticsCurrentCallers(),
				beanAdapter.getValueModel(QueueViewModel.PROPERTY_CALLERS_IN_QUEUE_ID));
			Bindings.bind(getStatisticsLabelFreeAgents(),
				beanAdapter.getValueModel(QueueViewModel.PROPERTY_FREE_AGENTS_IN_QUEUE_ID));
			Bindings.bind(getStatisticsLabelCalls(),
				beanAdapter.getValueModel(QueueViewModel.PROPERTY_CALL_LISTS_QUEUE_ID));
			Bindings.bind(statisticsWaittimeLabel,
				beanAdapter.getValueModel(QueueViewModel.PROPERTY_WAITTIME_QUEUE_ID));
			Bindings.bind(statisticsCallersLabel,
				beanAdapter.getValueModel(QueueViewModel.PROPERTY_CALLERS_TODAY_QUEUE_ID));
	} catch (Exception e) {
		Log.info("Error", e);
	}
		queueAgentTable.setModel(viewModel.getQueueAgentTableModel());
		//		queueAgentTable.setSelectionModel(new SingleListSelectionAdapter(
		//			beanAdapter.getValueModel(QueueViewModel.AGENT_SELECTED_ROW)));
		queueAgentTable.removeColumn(queueAgentTable.getColumn(QueueAgentStateProperties.accountId.toString()));

		queueCallTable.setModel(viewModel.getQueueCallTableModel());
		//		queueAgentTable.setSelectionModel(new SingleListSelectionAdapter(
		//			beanAdapter.getValueModel(QueueViewModel.AGENT_SELECTED_ROW)));
		// queueCallTable.removeColumn(queueCallTable.getColumn(UciQueueCallProperties.callId.toString()));

		queueSettingsTable.setModel(viewModel.getQueueSettingsTableModel());
		queueSettingsTable.setSelectionModel(new SingleListSelectionAdapter(
				beanAdapter.getValueModel(QueueViewModel.PROPERTY_SELECTED_ROW_QUEUE)));
		queueCallTable.setSelectionModel(
				new SingleListSelectionAdapter(beanAdapter.getValueModel(QueueViewModel.PROPERTY_SELECTED_ROW_CALLER)));
		queueSettingsTable.removeColumn(queueSettingsTable.getColumn(QueueSettingProperties.accountId.toString()));
		queueSettingsTable.removeColumn(queueSettingsTable.getColumn(QueueSettingProperties.id.toString()));

	}

}
