/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.examples.uci30.swing.view;

import de.starface.examples.uci30.swing.viewmodel.CallListViewModel;
import de.starface.examples.uci30.swing.viewmodel.RedirectViewModel;

import com.jgoodies.binding.adapter.Bindings;
import com.jgoodies.binding.adapter.SingleListSelectionAdapter;
import com.jgoodies.binding.beans.BeanAdapter;
import com.jgoodies.binding.list.SelectionInList;
import org.dyno.visual.swing.layouts.Constraints;
import org.dyno.visual.swing.layouts.GroupLayout;
import org.dyno.visual.swing.layouts.Leading;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.*;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellEditor;

/**
 * This panel provides a GUI for the UCI communication redirect functionality
 *
 * @author tilmann
 */
// VS4E -- DO NOT REMOVE THIS LINE!
public class RedirectPanel extends JPanel {

	private static final long serialVersionUID = 1L;

	private JTable table;
	private JScrollPane tableScrollPane;
	private JComboBox orderPropertyCombo;
	private JComboBox orderDirectionCombo;
	private JLabel typeRestrictionLabel;
	private JComboBox typeRestrictionCombo;
	private JLabel orderPropertyLabel;
	private JLabel orderDirectionLabel;
	private JButton setRedirectSettingsButton;
	private JButton getRedirectSettingsButton;
	private JButton setRedirectSettingButton;

	public RedirectPanel() {
		initComponents();
	}

	private void initComponents() {
		setMinimumSize(new Dimension(570, 450));
		setPreferredSize(new Dimension(570, 450));
		setLayout(new GroupLayout());
		add(getOrderDirectionLabel(), new Constraints(new Leading(10, 6, 6), new Leading(47, 9, 6, 6)));
		add(getOrderDirectionCombo(), new Constraints(new Leading(94, 124, 6, 6), new Leading(39, 6, 6)));
		add(getOrderPropertyCombo(), new Constraints(new Leading(93, 126, 6, 6), new Leading(7, 6, 6)));
		add(getOrderPropertyLabel(), new Constraints(new Leading(10, 6, 6), new Leading(12, 6, 6)));
		add(getTypeRestrictionLabel(), new Constraints(new Leading(231, 6, 6), new Leading(12, 6, 6)));
		add(getSetRedirectSettingButton(), new Constraints(new Leading(10, 276, 10, 10), new Leading(71, 6, 6)));
		add(getGroupRestrictionLabel(), new Constraints(new Leading(231, 148, 148), new Leading(44, 6, 6)));
		add(getTypeRestrictionCombo(), new Constraints(new Leading(326, 100, 144, 144), new Leading(7, 6, 6)));
		add(getGroupRestrictionCombo(), new Constraints(new Leading(326, 100, 144, 144), new Leading(39, 6, 6)));
		add(getGetRedirectSettingsButton(), new Constraints(new Leading(430, 10, 10), new Leading(6, 58, 6, 6)));
		add(getSetRedirectSettingsButton(), new Constraints(new Leading(292, 274, 6, 6), new Leading(70, 6, 6)));
		add(getTableScrollPane(), new Constraints(new Leading(6, 560, 6, 6), new Leading(105, 336, 6, 6)));
		setSize(570, 450);
	}

	private JComboBox getGroupRestrictionCombo() {
		if (groupRestrictionCombo == null) {
			groupRestrictionCombo = new JComboBox();
			groupRestrictionCombo.setRenderer(new GroupSettingsListCellRenderer());
		}
		return groupRestrictionCombo;
	}

	private JLabel getGroupRestrictionLabel() {
		if (groupRestrictionLabel == null) {
			groupRestrictionLabel = new JLabel();
			groupRestrictionLabel.setText("groupRestriction");
		}
		return groupRestrictionLabel;
	}

	private JButton getSetRedirectSettingButton() {
		if (setRedirectSettingButton == null) {
			setRedirectSettingButton = new JButton();
			setRedirectSettingButton.setText("setRedirectSetting");
		}
		return setRedirectSettingButton;
	}

	private JButton getSetRedirectSettingsButton() {
		if (setRedirectSettingsButton == null) {
			setRedirectSettingsButton = new JButton();
			setRedirectSettingsButton.setText("setRedirectSettings");
		}
		return setRedirectSettingsButton;
	}

	private JButton getGetRedirectSettingsButton() {
		if (getRedirectSettingsButton == null) {
			getRedirectSettingsButton = new JButton();
			getRedirectSettingsButton.setText("getRedirectSettings");
		}
		return getRedirectSettingsButton;
	}

	private JLabel getOrderDirectionLabel() {
		if (orderDirectionLabel == null) {
			orderDirectionLabel = new JLabel();
			orderDirectionLabel.setText("orderDirection");
		}
		return orderDirectionLabel;
	}

	private JLabel getOrderPropertyLabel() {
		if (orderPropertyLabel == null) {
			orderPropertyLabel = new JLabel();
			orderPropertyLabel.setText("orderProperty");
		}
		return orderPropertyLabel;
	}

	private JComboBox getTypeRestrictionCombo() {
		if (typeRestrictionCombo == null) {
			typeRestrictionCombo = new JComboBox();
		}
		return typeRestrictionCombo;
	}

	private JLabel getTypeRestrictionLabel() {
		if (typeRestrictionLabel == null) {
			typeRestrictionLabel = new JLabel();
			typeRestrictionLabel.setText("typeRestriction");
		}
		return typeRestrictionLabel;
	}

	private JComboBox getOrderDirectionCombo() {
		if (orderDirectionCombo == null) {
			orderDirectionCombo = new JComboBox();
		}
		return orderDirectionCombo;
	}

	private JComboBox getOrderPropertyCombo() {
		if (orderPropertyCombo == null) {
			orderPropertyCombo = new JComboBox();
		}
		return orderPropertyCombo;
	}

	private JScrollPane getTableScrollPane() {
		if (tableScrollPane == null) {
			tableScrollPane = new JScrollPane();
			tableScrollPane.setMinimumSize(new Dimension(570, 470));
			tableScrollPane.setPreferredSize(new Dimension(570, 470));
			tableScrollPane.setViewportView(getTable());
		}
		return tableScrollPane;
	}

	private JTable getTable() {
		if (table == null) {
			table = new JTable();
			table.setModel(new DefaultTableModel(new Object[][]{}, new String[]{}) {
				private static final long serialVersionUID = 1L;
				Class<?>[] types = new Class<?>[]{};

				@Override
				public Class<?> getColumnClass(int columnIndex) {
					return types[columnIndex];
				}
			});
			table.setRowSelectionAllowed(true);
			table.setColumnSelectionAllowed(false);
			table.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		}
		return table;
	}

	private final ActionListener endTableEditingActionListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			endTableEditing();
		}
	};

	private JLabel groupRestrictionLabel;

	private JComboBox groupRestrictionCombo;

	private static final String PREFERRED_LOOK_AND_FEEL = "com.sun.java.swing.plaf.nimbus.NimbusLookAndFeel";

	private void endTableEditing() {
		TableCellEditor editor = table.getCellEditor();
		if (editor != null) {
			if (!editor.stopCellEditing()) {
				editor.cancelCellEditing();
			}
		}
	}

	public void setViewModel(RedirectViewModel viewModel) {
		initBindings(viewModel);
	}

	private void initBindings(final RedirectViewModel viewModel) {
		setRedirectSettingsButton.addActionListener(viewModel.getSetRedirectSettingsActionListener());
		setRedirectSettingButton.addActionListener(viewModel.getSetRedirectSettingActionListener());
		getRedirectSettingsButton.addActionListener(viewModel.getGetRedirectSettingsActionListener());

		// We need to add these to finish editing before saving the currently edited redirect setting
		// BUT we must add the listeners after the real actions above because Swing executes these
		//     listeners last to first. Why? I don't know.
		setRedirectSettingButton.addActionListener(endTableEditingActionListener);
		setRedirectSettingsButton.addActionListener(endTableEditingActionListener);

		BeanAdapter<RedirectViewModel> beanAdapter = new BeanAdapter<>(viewModel, true);

		Bindings.bind(
			typeRestrictionCombo,
			new SelectionInList<>(viewModel.getTypeRestrictionValues(),
				beanAdapter.getValueModel(RedirectViewModel.PROPERTY_TYPE_RESTRICTION)));
		Bindings.bind(
			orderDirectionCombo,
			new SelectionInList<>(viewModel.getOrderDirectionValues(),
				beanAdapter.getValueModel(RedirectViewModel.PROPERTY_ORDER_DIRECTION)));
		Bindings.bind(
			orderPropertyCombo,
			new SelectionInList<>(viewModel.getOrderPropertyValues(),
				beanAdapter.getValueModel(RedirectViewModel.PROPERTY_ORDER_PROPERTY)));
		Bindings.bind(
			groupRestrictionCombo,
			new SelectionInList<>(
				beanAdapter.getValueModel(CallListViewModel.PROPERTY_GROUP_RESTRICTION_VALUES),
				beanAdapter.getValueModel(CallListViewModel.PROPERTY_GROUP_RESTRICTION)));

		table.setModel(viewModel.getRedirectSettingsTableModel());
		table.setSelectionModel(new SingleListSelectionAdapter(
			beanAdapter.getValueModel(RedirectViewModel.PROPERTY_SELECTED_ROW)));
		table.getColumnModel().getColumn(7).setCellEditor(new RedirectSettingsDestinationCellEditor());
		table.getColumnModel().getColumn(7).setCellRenderer(new RedirectSettingsDestinationCellRenderer());
		table.getColumnModel().getColumn(6).setCellEditor(new RedirectSettingsDestinationTypeCellEditor());
	}

}
