/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.examples.uci30.swing.view;

import java.awt.*;

import javax.swing.*;
import javax.swing.table.DefaultTableCellRenderer;

/**
 * A table cell renderer for the destination property of redirect settings
 *
 * @author tilmann
 */
public class ToolTipCellRenderer extends DefaultTableCellRenderer {

	public static interface TableToolTipProvider {
		String getToolTipForCell(JTable table, Object value, boolean isSelected, boolean hasFocus, int row,
								 int column);
	}

	private static final long serialVersionUID = 1L;

	private final TableToolTipProvider toolTipProvider;

	public ToolTipCellRenderer(TableToolTipProvider toolTipProvider) {
		this.toolTipProvider = toolTipProvider;
	}

	@Override
	public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected,
												   boolean hasFocus, int row, int column) {
		JComponent c = (JComponent) super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row,
			column);
		String toolTip = toolTipProvider.getToolTipForCell(table, value, isSelected, hasFocus, row, column);
		if (toolTip != null) {
			toolTip = "<html>" + toolTip.replaceAll("\n", "<br>") + "<html>";
		}
		c.setToolTipText(toolTip);
		return c;
	}

}
