/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.examples.uci30.swing.view;

import de.starface.examples.uci30.swing.viewmodel.UserStateViewModel;

import com.jgoodies.binding.adapter.Bindings;
import com.jgoodies.binding.beans.BeanAdapter;
import com.jgoodies.binding.beans.PropertyConnector;
import com.jgoodies.binding.list.SelectionInList;
import org.dyno.visual.swing.layouts.Constraints;
import org.dyno.visual.swing.layouts.GroupLayout;
import org.dyno.visual.swing.layouts.Leading;

import java.awt.*;

import javax.swing.*;

/**
 * This panel provides a GUI for the UCI communication call list functionality
 *
 * @author tilmann
 */
// VS4E -- DO NOT REMOVE THIS LINE!
public class UserStatePanel extends JPanel {
	private static final long serialVersionUID = 1L;

	private JButton getUserStateButton;
	private JFormattedTextField currentChatPresenceMessageText;
	private JLabel currentUserStateLabel;

	private JLabel chatPresenceMessageLabel;

	private JLabel telephonyStateLabel;

	private JLabel avatarHashLabel;

	private JLabel userNameLabel;

	private JComboBox chatPresenceCombo;

	private JComboBox telephonyStateCombo;

	private JTextField avatarHashText;

	private JTextField userNameText;

	private JCheckBox doNotDisturbSettingCheckBox;

	private JCheckBox redirectAlwaysActiveCheckBox;

	private JButton setDoNotDisturbSettingButton;

	private JButton userNameButton;

	private JButton setChatPresenceMessageButton;

	private JLabel avatarLabel;

	private JButton getAvatarImageButton;

	private JButton getAvatarHashByLoginButton;

	private JButton setAvatarImageButton;

	private JButton getPasswordChangeRequiredButton;

	private static final String PREFERRED_LOOK_AND_FEEL = "com.sun.java.swing.plaf.nimbus.NimbusLookAndFeel";

	private JTextField loginInput;

	public UserStatePanel() {
		initComponents();
	}

	private void initComponents() {
		setMinimumSize(new Dimension(570, 500));
		setPreferredSize(new Dimension(570, 520));
		setLayout(new GroupLayout());
		add(getChatPresenceCombo(), new Constraints(new Leading(163, 183, 10, 10), new Leading(6, 6, 6)));
		add(getCurrentUserStateLabel(), new Constraints(new Leading(6, 6, 6), new Leading(11, 6, 6)));
		add(getUserStateLabel(), new Constraints(new Leading(163, 403, 6, 6), new Leading(34, 26, 6, 6)));
		add(getChatPresenceMessageLabel(), new Constraints(new Leading(6, 6, 6), new Leading(39, 6, 6)));
		add(getTelephonyStateCombo(), new Constraints(new Leading(163, 182, 6, 6), new Leading(61, 6, 6)));
		add(getTelephonyStateLabel(), new Constraints(new Leading(6, 6, 6), new Leading(67, 6, 6)));
		add(getAvatarHashText(), new Constraints(new Leading(163, 402, 6, 6), new Leading(89, 6, 6)));
		add(getAvatarHashLabel(), new Constraints(new Leading(6, 6, 6), new Leading(95, 6, 6)));

		add(getUserNameLabel(), new Constraints(new Leading(6, 6, 6), new Leading(119, 6, 6)));
		add(getUserNameText(), new Constraints(new Leading(163, 402, 6, 6), new Leading(119, 6, 6)));
		add(getUserNameButton(), new Constraints(new Leading(570, 150, 6, 6), new Leading(119, 6, 6)));

		add(getDoNotDisturbSettingCheckBox(), new Constraints(new Leading(163, 6, 6), new Leading(149, 6, 6)));
		add(getRedirectAlwaysActiveCheckBox(), new Constraints(new Leading(163, 6, 6), new Leading(171, 6, 6)));
		add(getGetUserStateButton(), new Constraints(new Leading(6, 559, 6, 6), new Leading(199, 74, 6, 6)));
		add(getSetDoNotDisturbSettingButton(), new Constraints(new Leading(6, 278, 10, 10), new Leading(280, 67,
			10, 10)));
		add(getSetChatPresenceMessageButton(), new Constraints(new Leading(290, 276, 6, 6), new Leading(280, 66,
			6, 6)));
		add(getAvatarLabel(), new Constraints(new Leading(12, 72, 6, 6), new Leading(360, 71, 6, 6)));
		add(getGetAvatarImageButton(), new Constraints(new Leading(90, 194, 6, 6), new Leading(360, 72, 6, 6)));
		add(getSetAvatarImageButton(), new Constraints(new Leading(290, 276, 6, 6), new Leading(360, 70, 6, 6)));

		add(new JLabel("Login:"), new Constraints(new Leading(10, 80, 6, 6), new Leading(440, 30, 6, 6)));
		add(getLoginInput(), new Constraints(new Leading(90, 194, 6, 6), new Leading(440, 30, 6, 6)));
		add(getGetAvatarHashByLoginButton(), new Constraints(new Leading(290, 276, 6, 6), new Leading(440, 30, 6, 6)));
		add(getGetPasswordChangeRequiredButton(), new Constraints(new Leading(290, 276, 6, 6), new Leading(480, 30, 6, 6)));


		setSize(570, 520);
	}

	private JButton getUserNameButton() {
		if (userNameButton == null) {
			userNameButton = new JButton();
			userNameButton.setText("getUserName");
		}
		return userNameButton;
	}

	private JButton getSetAvatarImageButton() {
		if (setAvatarImageButton == null) {
			setAvatarImageButton = new JButton();
			setAvatarImageButton.setText("setAvatarImage");
		}
		return setAvatarImageButton;
	}

	private JButton getGetAvatarImageButton() {
		if (getAvatarImageButton == null) {
			getAvatarImageButton = new JButton();
			getAvatarImageButton.setText("getAvatarImage");
		}
		return getAvatarImageButton;
	}

	private JButton getGetAvatarHashByLoginButton() {
		if (getAvatarHashByLoginButton == null) {
			getAvatarHashByLoginButton = new JButton();
			getAvatarHashByLoginButton.setText("getAvatarHashByLogin");
		}
		return getAvatarHashByLoginButton;
	}

	private JButton getGetPasswordChangeRequiredButton() {
		if (getPasswordChangeRequiredButton == null) {
			getPasswordChangeRequiredButton = new JButton();
			getPasswordChangeRequiredButton.setText("getPasswordChangeRequired");
		}
		return getPasswordChangeRequiredButton;
	}

	private JTextField getLoginInput() {
		if (loginInput == null) {
			loginInput = new JTextField();
			loginInput.setText("login");
		}
		return loginInput;
	}

	private JLabel getAvatarLabel() {
		if (avatarLabel == null) {
			avatarLabel = new JLabel();
			avatarLabel.setHorizontalAlignment(SwingConstants.CENTER);
			avatarLabel.setMinimumSize(new Dimension(64, 64));
			avatarLabel.setPreferredSize(new Dimension(64, 64));
			avatarLabel.setMaximumSize(new Dimension(64, 64));
			avatarLabel.setIconTextGap(0);
		}
		return avatarLabel;
	}

	private JLabel getUserNameLabel() {
		if (userNameLabel == null) {
			userNameLabel = new JLabel("UserName: ");
		}
		return userNameLabel;
	}

	private JButton getSetChatPresenceMessageButton() {
		if (setChatPresenceMessageButton == null) {
			setChatPresenceMessageButton = new JButton();
			setChatPresenceMessageButton.setText("setChatPresenceMessage");
		}
		return setChatPresenceMessageButton;
	}

	private JButton getSetDoNotDisturbSettingButton() {
		if (setDoNotDisturbSettingButton == null) {
			setDoNotDisturbSettingButton = new JButton();
			setDoNotDisturbSettingButton.setText("setDoNotDisturbSetting");
		}
		return setDoNotDisturbSettingButton;
	}

	private JCheckBox getRedirectAlwaysActiveCheckBox() {
		if (redirectAlwaysActiveCheckBox == null) {
			redirectAlwaysActiveCheckBox = new JCheckBox();
			redirectAlwaysActiveCheckBox.setText("redirectAlwaysActive");
			redirectAlwaysActiveCheckBox.setEnabled(false);
		}
		return redirectAlwaysActiveCheckBox;
	}

	private JCheckBox getDoNotDisturbSettingCheckBox() {
		if (doNotDisturbSettingCheckBox == null) {
			doNotDisturbSettingCheckBox = new JCheckBox();
			doNotDisturbSettingCheckBox.setText("doNotDisturbSetting");
		}
		return doNotDisturbSettingCheckBox;
	}

	private JTextField getAvatarHashText() {
		if (avatarHashText == null) {
			avatarHashText = new JTextField();
			avatarHashText.setEnabled(false);
		}
		return avatarHashText;
	}

	private JTextField getUserNameText() {
		if (userNameText == null) {
			userNameText = new JTextField();
			userNameText.setEnabled(false);
		}
		return userNameText;
	}

	private JComboBox getTelephonyStateCombo() {
		if (telephonyStateCombo == null) {
			telephonyStateCombo = new JComboBox();
			telephonyStateCombo.setModel(new DefaultComboBoxModel(new Object[]{"item0", "item1", "item2",
				"item3"}));
			telephonyStateCombo.setEnabled(false);
		}
		return telephonyStateCombo;
	}

	private JComboBox getChatPresenceCombo() {
		if (chatPresenceCombo == null) {
			chatPresenceCombo = new JComboBox();
			chatPresenceCombo.setModel(new DefaultComboBoxModel(
				new Object[]{"item0", "item1", "item2", "item3"}));
			chatPresenceCombo.setEnabled(false);
		}
		return chatPresenceCombo;
	}

	private JLabel getAvatarHashLabel() {
		if (avatarHashLabel == null) {
			avatarHashLabel = new JLabel();
			avatarHashLabel.setText("avatarHash:");
		}
		return avatarHashLabel;
	}

	private JLabel getTelephonyStateLabel() {
		if (telephonyStateLabel == null) {
			telephonyStateLabel = new JLabel();
			telephonyStateLabel.setText("telephonyState:");
		}
		return telephonyStateLabel;
	}

	private JLabel getChatPresenceMessageLabel() {
		if (chatPresenceMessageLabel == null) {
			chatPresenceMessageLabel = new JLabel();
			chatPresenceMessageLabel.setText("chatPresenceMessage:");
		}
		return chatPresenceMessageLabel;
	}

	private JLabel getCurrentUserStateLabel() {
		if (currentUserStateLabel == null) {
			currentUserStateLabel = new JLabel();
			currentUserStateLabel.setText("chatPresence:");
		}
		return currentUserStateLabel;
	}

	private JFormattedTextField getUserStateLabel() {
		if (currentChatPresenceMessageText == null) {
			currentChatPresenceMessageText = new JFormattedTextField();
		}
		return currentChatPresenceMessageText;
	}

	private JButton getGetUserStateButton() {
		if (getUserStateButton == null) {
			getUserStateButton = new JButton();
			getUserStateButton.setText("getUserState");
		}
		return getUserStateButton;
	}

	public void setViewModel(UserStateViewModel viewModel) {
		initBindings(viewModel);
	}

	private void initBindings(final UserStateViewModel viewModel) {
		getUserStateButton.addActionListener(viewModel.getGetUserStateActionListener());
		setChatPresenceMessageButton.addActionListener(viewModel.getSetChatPresenceMessageActionListener());
		setDoNotDisturbSettingButton.addActionListener(viewModel.getSetDoNotDisturbSettingActionListener());
		getAvatarImageButton.addActionListener(viewModel.getGetAvatarImageActionListener());
		setAvatarImageButton.addActionListener(viewModel.getSetAvatarImageActionListener());
		userNameButton.addActionListener(viewModel.getUserNameActionListener());
		getAvatarHashByLoginButton.addActionListener(viewModel.getGetAvatarHashByLoginActionListener());
		getPasswordChangeRequiredButton.addActionListener(viewModel.getGetPasswordChangeRequiredActionListener());

		BeanAdapter<UserStateViewModel> beanAdapter = new BeanAdapter<>(viewModel, true);

		Bindings.bind(redirectAlwaysActiveCheckBox,
			beanAdapter.getValueModel(UserStateViewModel.PROPERTY_REDIRECT_ALWAYS_ACTIVE));
		Bindings.bind(doNotDisturbSettingCheckBox,
			beanAdapter.getValueModel(UserStateViewModel.PROPERTY_DO_NOT_DISTURB_SETTING));
		Bindings.bind(currentChatPresenceMessageText,
			beanAdapter.getValueModel(UserStateViewModel.PROPERTY_CHAT_PRESENCE_MESSAGE));
		Bindings.bind(loginInput,
			beanAdapter.getValueModel(UserStateViewModel.PROPERTY_LOGIN_INPUT));
		Bindings.bind(avatarHashText, beanAdapter.getValueModel(UserStateViewModel.PROPERTY_AVATAR_HASH));
		Bindings.bind(chatPresenceCombo, new SelectionInList<>(viewModel.getChatPresenceValues(),
			beanAdapter.getValueModel(UserStateViewModel.PROPERTY_CHAT_PRESENCE)));
		PropertyConnector.connectAndUpdate(beanAdapter.getValueModel(UserStateViewModel.PROPERTY_AVATAR),
			avatarLabel, "icon");
		Bindings.bind(getUserNameText(), beanAdapter.getValueModel(UserStateViewModel.PROPERTY_USER_NAME));

		Bindings.bind(
			telephonyStateCombo,
			new SelectionInList<>(viewModel.getTelephonyStateValues(),
				beanAdapter.getValueModel(UserStateViewModel.PROPERTY_TELEPHONY_STATE)));

	}
}
