/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.examples.uci30.swing.viewmodel;

import de.starface.integration.uci.java.v30.types.CallList;
import de.starface.integration.uci.java.v30.types.CallListEntry;
import de.starface.integration.uci.java.v30.values.CallListEntryProperties;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

/**
 * A swing table model for the call list
 *
 * @author tilmann
 */
public class CallListTableModel implements TableModel {
	private static final CallListEntryProperties[] columnOrder = List.of(
					CallListEntryProperties.id,
					CallListEntryProperties.startTime,
					CallListEntryProperties.callerNumber,
					CallListEntryProperties.calledNumber,
					CallListEntryProperties.direction,
					CallListEntryProperties.result,
					CallListEntryProperties.duration,
					CallListEntryProperties.callDescription,
					CallListEntryProperties.voicemailId,
					CallListEntryProperties.groupId,
					CallListEntryProperties.callerJabberId,
					CallListEntryProperties.calledBack,
					CallListEntryProperties.comment,
					CallListEntryProperties.answeredBy)
			.toArray(CallListEntryProperties[]::new);

	private CallList callList;

	public CallListTableModel() {
		callList = new CallList();
		callList.setEntries(new ArrayList<>());
	}

	private List<TableModelListener> listeners = new ArrayList<>();

	@Override
	public void addTableModelListener(TableModelListener listener) {
		listeners.add(listener);
	}

	@Override
	public Class<?> getColumnClass(int columnIndex) {
		return columnOrder[columnIndex].getType();
	}

	@Override
	public int getColumnCount() {
		return columnOrder.length;
	}

	@Override
	public String getColumnName(int columnIndex) {
		return columnOrder[columnIndex].name();
	}

	@Override
	public int getRowCount() {
		return callList.getEntries().size();
	}

	@Override
	public Object getValueAt(int rowIndex, int columnIndex) {
		return Utils.getProperty(columnOrder[columnIndex], callList.getEntries().get(rowIndex));
	}

	@Override
	public boolean isCellEditable(int rowIndex, int columnIndex) {
		return false;
	}

	@Override
	public void removeTableModelListener(TableModelListener listener) {
		listeners.remove(listener);
	}

	@Override
	public void setValueAt(Object value, int rowIndex, int columnIndex) {
		// Not implemented
	}

	public CallListEntry getEntry(int rowIndex) {
		return callList.getEntries().get(rowIndex);
	}

	public void removeEntry(int rowIndex) {
		callList.getEntries().remove(rowIndex);
		fireTableChanged(new TableModelEvent(this, rowIndex, rowIndex, TableModelEvent.ALL_COLUMNS,
			TableModelEvent.DELETE));
	}

	public void replaceEntry(int rowIndex, CallListEntry callListEntry) {
		callList.getEntries().set(rowIndex, callListEntry);
		fireTableChanged(new TableModelEvent(this, rowIndex, rowIndex, TableModelEvent.ALL_COLUMNS,
				TableModelEvent.UPDATE));
	}

	public void addEntry(CallListEntry callListEntry) {
		callList.getEntries().add(callListEntry);
		fireTableChanged(new TableModelEvent(this));
	}

	public CallList getCallList() {
		return callList;
	}

	public void setCallList(CallList callList) {
		this.callList = callList;
		fireTableChanged(new TableModelEvent(this));
	}

	private void fireTableChanged(TableModelEvent e) {
		synchronized (listeners) {
			for (TableModelListener l : listeners) {
				l.tableChanged(e);
			}
		}
	}

	public void removeEntries(List<Integer> rows) {
		List<Integer> myRows = new ArrayList<>(rows);
		Collections.sort(myRows);
		Collections.reverse(myRows);
		for (int row : myRows) {
			removeEntry(row);
		}
	}
}
