/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.examples.uci30.swing.viewmodel;

import de.starface.integration.uci.java.v30.types.CcbsCall;
import de.starface.integration.uci.java.v30.values.CcbsCallProperties;

import java.util.ArrayList;
import java.util.List;
import java.util.ListIterator;

import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

public class CcbsCallTableModel implements TableModel {

	private static final CcbsCallProperties[] columnOrder = new CcbsCallProperties[10];

	private List<TableModelListener> listeners = new ArrayList<>();

	private ArrayList<CcbsCall> ccbsCalls;

	static {
		columnOrder[0] = CcbsCallProperties.id;
		columnOrder[1] = CcbsCallProperties.ccbsCallState;
		columnOrder[2] = CcbsCallProperties.timestamp;
		columnOrder[3] = CcbsCallProperties.calledNumber;
		columnOrder[4] = CcbsCallProperties.calledName;
		columnOrder[5] = CcbsCallProperties.callerNumber;
		columnOrder[6] = CcbsCallProperties.callerName;
		columnOrder[7] = CcbsCallProperties.groupId;
		columnOrder[8] = CcbsCallProperties.caller;
		columnOrder[9] = CcbsCallProperties.duration;
	}

	public CcbsCallTableModel() {
		ccbsCalls = new ArrayList<>();
	}

	@Override
	public int getRowCount() {
		return ccbsCalls.size();
	}

	@Override
	public int getColumnCount() {
		return columnOrder.length;
	}

	@Override
	public String getColumnName(int columnIndex) {
		return columnOrder[columnIndex].name();
	}

	@Override
	public Class<?> getColumnClass(int columnIndex) {
		return columnOrder[columnIndex].getType();
	}

	@Override
	public boolean isCellEditable(int rowIndex, int columnIndex) {
		return false;
	}

	@Override
	public Object getValueAt(int rowIndex, int columnIndex) {
		if (rowIndex > ccbsCalls.size() - 1) {
			return null;
		}
		return Utils.getProperty(columnOrder[columnIndex], ccbsCalls.get(rowIndex));
	}

	@Override
	public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
		// empty
	}

	@Override
	public void addTableModelListener(TableModelListener listener) {
		listeners.add(listener);
	}

	@Override
	public void removeTableModelListener(TableModelListener listener) {
		listeners.remove(listener);
	}

	private void fireTableChanged(TableModelEvent event) {
		synchronized (listeners) {
			for (TableModelListener l : listeners) {
				l.tableChanged(event);
			}
		}
	}

	public void updateCcbsCall(CcbsCall call) {
		String id = call.getId();
		int rowIndex = 0;
		for (ListIterator<CcbsCall> listIterator = ccbsCalls.listIterator(); listIterator.hasNext(); rowIndex = listIterator.nextIndex()) {
			if (listIterator.next().getId().equals(id)) {
				listIterator.set(call);
				fireTableChanged(new TableModelEvent(this, rowIndex, rowIndex, TableModelEvent.ALL_COLUMNS,
					TableModelEvent.UPDATE));
				break;
			}
		}
		if (rowIndex == ccbsCalls.size()) {
			ccbsCalls.add(call);
			fireTableChanged(new TableModelEvent(this, ccbsCalls.size(), ccbsCalls.size(),
				TableModelEvent.ALL_COLUMNS, TableModelEvent.INSERT));
		}
	}

	public void removeCcbsCall(String callId) {
		int rowIndex = 0;
		for (ListIterator<CcbsCall> listIterator = ccbsCalls.listIterator(); listIterator.hasNext(); rowIndex = listIterator.nextIndex()) {
			if (listIterator.next().getId().equals(callId)) {
				listIterator.remove();
				fireTableChanged(new TableModelEvent(this, rowIndex, rowIndex, TableModelEvent.ALL_COLUMNS,
					TableModelEvent.DELETE));
			}
		}
	}

	public CcbsCall getCcbsCall(int rowIndex) {
		return ccbsCalls.get(rowIndex);
	}
}
