/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.examples.uci30.swing.viewmodel;

import de.starface.integration.uci.java.v30.exceptions.UciException;
import de.starface.integration.uci.java.v30.messages.events.UciFaxListEvents;
import de.starface.integration.uci.java.v30.messages.events.UciGroupEvents;
import de.starface.integration.uci.java.v30.messages.requests.UciFaxListRequests;
import de.starface.integration.uci.java.v30.types.FaxList;
import de.starface.integration.uci.java.v30.types.FaxListEntry;
import de.starface.integration.uci.java.v30.types.FaxListEntryMovedData;
import de.starface.integration.uci.java.v30.types.GroupSetting;
import de.starface.integration.uci.java.v30.values.FaxListEntryFolder;
import de.starface.integration.uci.java.v30.values.FaxListEntryProperties;
import de.starface.integration.uci.java.v30.values.GroupRestriction;
import de.starface.integration.uci.java.v30.values.OrderDirection;
import de.starface.integration.uci.v30.client.UciServiceRequests;
import de.starface.integration.uci.v30.client.utils.EventsProvider;

import org.apache.commons.lang3.StringUtils;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.File;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import javax.swing.table.TableModel;

public class FaxListViewModel implements UciFaxListEvents, UciGroupEvents {
	/// Domain Logic Access
	////////////////////////////////////////////////////////////////

	public FaxListViewModel(EventsProvider<UciGroupEvents> groupEventsProvider) {
		groupEventsProvider.subscribeEvents(this);
	}

	private UciFaxListRequests faxListRequests;

	public void setUciServiceRequests(UciServiceRequests uciServiceRequests) throws UciException {
		this.faxListRequests = uciServiceRequests.getRequests(UciFaxListRequests.class);
		uciServiceRequests.subscribeEventsAsync(this, UciFaxListEvents.class);
	}

	@Override
	public void newFaxListEntry(FaxListEntry faxListEntry) {
		faxListTableModel.addEntry(faxListEntry);
	}

	@Override
	public void deletedFaxListEntries(List<String> deletedFaxCdrDataIds) {
		List<Integer> rowsToDelete = new ArrayList<>();
		for (int i = 0; i < faxListTableModel.getRowCount(); i++) {
			FaxListEntry entry = faxListTableModel.getEntry(i);
			if (deletedFaxCdrDataIds.contains(entry.getId())) {
				rowsToDelete.add(i);
			}
		}
		faxListTableModel.removeEntries(rowsToDelete);
	}

	@Override
	public void movedFaxListEntries(List<FaxListEntryMovedData> movedFaxCdrEntries) {
		faxListTableModel.moveEntries(movedFaxCdrEntries);
	}

	@Override
	public void groupSettingsChanged(List<GroupSetting> groupSettings) {
		setGroupRestrictionValues(groupSettings);
	}

	/// View Actions
	////////////////////////////////////////////////////////////////

	private void doBrowseFaxFileUrl() {
		if (!StringUtils.isEmpty(faxFileUrl)) {
			setBrowseUrlIsAllowed(false);
			Desktop desktop = Desktop.getDesktop();
			try {
				desktop.browse(new URL(faxFileUrl).toURI());
			} catch (Exception e) {
				// Too simple error management. MUST be improved in real client!
				e.printStackTrace();
			}
		}
	}

	private void doOpenFaxFile() {
		if (!StringUtils.isEmpty(faxFile)) {
			Desktop desktop = Desktop.getDesktop();
			try {
				desktop.open(new File(faxFile));
			} catch (Exception e) {
				// Too simple error management. MUST be improved in real client!
				e.printStackTrace();
			}
		}
	}

	private void doGetFaxList() {
		if (faxListRequests != null) {
			try {
				Date receivedBefore = this.receivedBefore != null ? this.receivedBefore : new Date();
				Date receivedAfter = this.receivedAfter != null ? this.receivedAfter : new Date(0);

				String groupRestriction = this.groupRestriction instanceof GroupSetting ? ((GroupSetting) this.groupRestriction).getId()
					: (String) this.groupRestriction;

				// Retrieve call list sending getCallList request
				FaxList faxList = faxListRequests.getFaxList(receivedBefore, receivedAfter, folderRestriction,
					groupRestriction, orderProperty, orderDirection, countOffset, countLimit);

				faxListTableModel.setFaxList(faxList);
			} catch (UciException e) {
				// Too simple error management. MUST be improved in real client!
				e.printStackTrace();
			}
		}
	}

	private void doDeleteFaxListEntry() {
		if (isEntrySelected()) {
			try {
				// delete fax list entry sending deleteFaxListEntry request
				faxListRequests.deleteFaxListEntry(getSelectedEntryId());

				faxListTableModel.removeEntry(selectedRows.get(0));
			} catch (UciException e) {
				// Too simple error management. MUST be improved in real client!
				e.printStackTrace();
			}
		}
	}

	private void doDeleteFaxListEntries() {
		if (!selectedRows.isEmpty() && faxListRequests != null) {
			List<String> idsToDelete = computeSelectedEntryIds();
			try {
				faxListRequests.deleteFaxListEntries(idsToDelete);
				faxListTableModel.removeEntries(selectedRows);
			} catch (UciException e) {
				// Too simple error management. MUST be improved in real client!
				e.printStackTrace();
			}
		}
	}

	private void doMoveFaxListEntry() {
		if (isEntrySelected()) {
			try {
				// Move fax list entry sending moveFaxListEntry request
				faxListRequests.moveFaxListEntry(getSelectedEntryId(), destinationFolder);

				if (destinationFolder != folderRestriction && folderRestriction != null) {
					faxListTableModel.removeEntry(selectedRows.get(0));
				}
			} catch (UciException e) {
				// Too simple error management. MUST be improved in real client!
				e.printStackTrace();
			}
		}
	}

	private void doMoveFaxListEntries() {
		if (!selectedRows.isEmpty() && faxListRequests != null) {
			List<String> idsToMove = computeSelectedEntryIds();
			try {
				faxListRequests.moveFaxListEntries(idsToMove, destinationFolder);

				if (destinationFolder != folderRestriction && folderRestriction != null) {
					faxListTableModel.removeEntries(selectedRows);
				}
			} catch (UciException e) {
				// Too simple error management. MUST be improved in real client!
				e.printStackTrace();
			}
		}
	}

	private void doGetFaxFileUrl() {
		if (isEntrySelected()) {
			try {
				final String fileUrl = faxListRequests.getFaxFileUrl(getSelectedEntryId());
				setFaxFileUrl(fileUrl);
				setBrowseUrlIsAllowed(true);
			} catch (UciException e) {
				// Too simple error management. MUST be improved in real client!
				e.printStackTrace();
			}
		}
	}

	private void doGetFaxFile() {
		if (isEntrySelected()) {
			try {
				final File file = faxListRequests.getFaxFile(getSelectedEntryId());
				setFaxFile(file.getAbsolutePath());
			} catch (IndexOutOfBoundsException | UciException e) {
				// Too simple error management. MUST be improved in real client!
				e.printStackTrace();
			}
		}
	}

	private boolean isEntrySelected() {
		return selectedRows.size() == 1;
	}

	private String getSelectedEntryId() {
		return faxListTableModel.getEntry(selectedRows.get(0)).getId();
	}

	private List<String> computeSelectedEntryIds() {
		List<String> entryIds = new ArrayList<>(selectedRows.size());
		for (Integer i : selectedRows) {
			entryIds.add(faxListTableModel.getEntry(i).getId());
		}
		return entryIds;
	}

	private final ActionListener getFaxListActionListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			doGetFaxList();
		}
	};

	private final ActionListener deleteFaxListEntryActionListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			doDeleteFaxListEntry();
		}
	};

	private final ActionListener deleteFaxListEntriesActionListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			doDeleteFaxListEntries();
		}
	};

	private final ActionListener getFaxFileUrlActionListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			doGetFaxFileUrl();
		}
	};

	private final ActionListener getFaxFileActionListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			doGetFaxFile();
		}
	};

	private final ActionListener moveFaxListEntryActionListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			doMoveFaxListEntry();
		}
	};

	private final ActionListener moveFaxListEntriesActionListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			doMoveFaxListEntries();
		}
	};

	private final ActionListener browseFaxFileUrlActionListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			doBrowseFaxFileUrl();
		}
	};

	private final ActionListener openFaxFileActionListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			doOpenFaxFile();
		}
	};

	/// View Data
	////////////////////////////////////////////////////////////////

	private final FaxListTableModel faxListTableModel = new FaxListTableModel();

	boolean browseUrlIsAllowed = false;

	private Date receivedBefore = null;

	private Date receivedAfter = null;

	private FaxListEntryFolder folderRestriction = null;

	private List<Object> groupRestrictionValues;

	{
		setGroupRestrictionValues(Collections.<GroupSetting>emptyList());
	}

	private Object groupRestriction = "";

	private FaxListEntryProperties orderProperty = null;

	private OrderDirection orderDirection = null;

	private int countOffset = 0;

	private int countLimit = 14;

	private List<Integer> selectedRows = new ArrayList<>();

	private String faxFileUrl;

	private String faxFile;

	private FaxListEntryFolder destinationFolder = FaxListEntryFolder.OLD;

	private final PropertyChangeSupport changeSupport = new PropertyChangeSupport(this);

	public void addPropertyChangeListener(PropertyChangeListener listener) {
		changeSupport.addPropertyChangeListener(listener);
	}

	public void removePropertyChangeListener(PropertyChangeListener listener) {
		changeSupport.removePropertyChangeListener(listener);
	}

	/// Property Getters and Setters
	////////////////////////////////////////////////////////////////

	public TableModel getFaxListTableModel() {
		return faxListTableModel;
	}

	public String getFaxFileUrl() {
		return faxFileUrl;
	}

	public static final String PROPERTY_FAX_FILE_URL = "faxFileUrl";

	public void setFaxFileUrl(String faxFileUrl) {
		String old = this.faxFileUrl;
		this.faxFileUrl = faxFileUrl;
		changeSupport.firePropertyChange(PROPERTY_FAX_FILE_URL, old, faxFileUrl);
	}

	public static final String PROPERTY_BROWSE_URL_IS_ALLOWED = "BrowseUrlIsAllowed";

	public boolean getBrowseUrlIsAllowed() {
		return browseUrlIsAllowed;
	}

	public void setBrowseUrlIsAllowed(boolean browseUrlIsAllowed) {
		boolean old = this.browseUrlIsAllowed;
		this.browseUrlIsAllowed = browseUrlIsAllowed;
		changeSupport.firePropertyChange(PROPERTY_BROWSE_URL_IS_ALLOWED, old, browseUrlIsAllowed);
	}

	public String getFaxFile() {
		return faxFile;
	}

	public static final String PROPERTY_FAX_FILE = "faxFile";

	public void setFaxFile(String faxFile) {
		String old = this.faxFile;
		this.faxFile = faxFile;
		changeSupport.firePropertyChange(PROPERTY_FAX_FILE, old, faxFile);
	}

	public Date getReceivedBefore() {
		return receivedBefore;
	}

	public static final String PROPERTY_RECEIVED_BEFORE = "receivedBefore";

	public void setReceivedBefore(Date receivedBefore) {
		Date old = this.receivedBefore;
		this.receivedBefore = receivedBefore;
		changeSupport.firePropertyChange(PROPERTY_RECEIVED_BEFORE, old, receivedBefore);
	}

	public Date getReceivedAfter() {
		return receivedAfter;
	}

	public static final String PROPERTY_RECEIVED_AFTER = "receivedAfter";

	public void setReceivedAfter(Date receivedAfter) {
		Date old = this.receivedAfter;
		this.receivedAfter = receivedAfter;
		changeSupport.firePropertyChange(PROPERTY_RECEIVED_AFTER, old, receivedAfter);
	}

	public FaxListEntryFolder getFolderRestriction() {
		return folderRestriction;
	}

	public static final String PROPERTY_FOLDER_RESTRICTION = "folderRestriction";

	public void setFolderRestriction(FaxListEntryFolder folderRestriction) {
		FaxListEntryFolder old = this.folderRestriction;
		this.folderRestriction = folderRestriction;
		changeSupport.firePropertyChange(PROPERTY_FOLDER_RESTRICTION, old, folderRestriction);
	}

	public FaxListEntryFolder[] getFolderRestrictionValues() {
		return Utils.createArrayWithLeadingNull(FaxListEntryFolder.values());
	}

	public static final String PROPERTY_GROUP_RESTRICTION = "groupRestriction";

	public Object getGroupRestriction() {
		return groupRestriction;
	}

	public void setGroupRestriction(Object groupRestriction) {
		Object old = this.groupRestriction;
		this.groupRestriction = groupRestriction;
		changeSupport.firePropertyChange(PROPERTY_GROUP_RESTRICTION, old, groupRestriction);
	}

	public static final String PROPERTY_GROUP_RESTRICTION_VALUES = "groupRestrictionValues";

	public List<Object> getGroupRestrictionValues() {
		return groupRestrictionValues;
	}

	private void setGroupRestrictionValues(List<GroupSetting> groupSettings) {
		List<Object> old = groupRestrictionValues;
		groupRestrictionValues = new ArrayList<>();
		groupRestrictionValues.add("");
		groupRestrictionValues.add(GroupRestriction.NON_GROUP);
		groupRestrictionValues.addAll(groupSettings);
		if (changeSupport != null) {
			changeSupport.firePropertyChange(PROPERTY_GROUP_RESTRICTION_VALUES, old, groupRestrictionValues);
		}
	}

	public FaxListEntryProperties getOrderProperty() {
		return orderProperty;
	}

	public static final String PROPERTY_ORDER_PROPERTY = "orderProperty";

	public void setOrderProperty(FaxListEntryProperties orderProperty) {
		FaxListEntryProperties old = this.orderProperty;
		this.orderProperty = orderProperty;
		changeSupport.firePropertyChange(PROPERTY_ORDER_PROPERTY, old, orderProperty);
	}

	public FaxListEntryProperties[] getOrderPropertyValues() {
		return Utils.createArrayWithLeadingNull(FaxListEntryProperties.values());
	}

	public OrderDirection getOrderDirection() {
		return orderDirection;
	}

	public static final String PROPERTY_ORDER_DIRECTION = "orderDirection";

	public void setOrderDirection(OrderDirection orderDirection) {
		OrderDirection old = this.orderDirection;
		this.orderDirection = orderDirection;
		changeSupport.firePropertyChange(PROPERTY_ORDER_DIRECTION, old, orderDirection);
	}

	public OrderDirection[] getOrderDirectionValues() {
		return Utils.createArrayWithLeadingNull(OrderDirection.values());
	}

	public int getCountOffset() {
		return countOffset;
	}

	public static final String PROPERTY_COUNT_OFFSET = "countOffset";

	public void setCountOffset(int countOffset) {
		int old = this.countOffset;
		this.countOffset = countOffset;
		changeSupport.firePropertyChange(PROPERTY_COUNT_OFFSET, old, countOffset);
	}

	public int getCountLimit() {
		return countLimit;
	}

	public static final String PROPERTY_COUNT_LIMIT = "countLimit";

	public void setCountLimit(int countLimit) {
		int old = this.countLimit;
		this.countLimit = countLimit;
		changeSupport.firePropertyChange(PROPERTY_COUNT_LIMIT, old, countLimit);
	}

	public List<Integer> getSelectedRows() {
		return selectedRows;
	}

	public FaxListEntryFolder getDestinationFolder() {
		return destinationFolder;
	}

	public static final String PROPERTY_DESTINATION_FOLDER = "destinationFolder";

	public void setDestinationFolder(FaxListEntryFolder destinationFolder) {
		FaxListEntryFolder old = this.destinationFolder;
		this.destinationFolder = destinationFolder;
		changeSupport.firePropertyChange(PROPERTY_DESTINATION_FOLDER, old, destinationFolder);
	}

	public FaxListEntryFolder[] getDestinationFolderValues() {
		return FaxListEntryFolder.values();
	}

	public ActionListener getGetFaxListActionListener() {
		return getFaxListActionListener;
	}

	public ActionListener getDeleteFaxListEntryActionListener() {
		return deleteFaxListEntryActionListener;
	}

	public ActionListener getGetFaxFileUrlActionListener() {
		return getFaxFileUrlActionListener;
	}

	public ActionListener getGetFaxFileActionListener() {
		return getFaxFileActionListener;
	}

	public ActionListener getMoveFaxListEntryActionListener() {
		return moveFaxListEntryActionListener;
	}

	public ActionListener getDeleteFaxListEntriesActionListener() {
		return deleteFaxListEntriesActionListener;
	}

	public ActionListener getMoveFaxListEntriesActionListener() {
		return moveFaxListEntriesActionListener;
	}

	public ActionListener getBrowseFaxFileUrlActionListener() {
		return browseFaxFileUrlActionListener;
	}

	public ActionListener getOpenFaxFileActionListener() {
		return openFaxFileActionListener;
	}
}
