/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.examples.uci30.swing.viewmodel;

import de.starface.integration.uci.java.v30.types.FmcPhone;
import de.starface.integration.uci.java.v30.values.FmcPhoneProperties;

import java.util.ArrayList;
import java.util.List;

import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

/**
 * A swing table model for the fmc phones
 *
 * @author tpfeifer
 */
public class FmcPhonesTableModel implements TableModel {
	private static final FmcPhoneProperties[] columnOrder;

	static {
		columnOrder = new FmcPhoneProperties[6];
		columnOrder[0] = FmcPhoneProperties.fmcId;
		columnOrder[1] = FmcPhoneProperties.telephoneId;
		columnOrder[2] = FmcPhoneProperties.number;
		columnOrder[3] = FmcPhoneProperties.delay;
		columnOrder[4] = FmcPhoneProperties.active;
		columnOrder[5] = FmcPhoneProperties.confirm;
	}

	private List<TableModelListener> listeners = new ArrayList<>(1);

	private List<FmcPhone> fmcPhones = new ArrayList<>();

	@Override
	public void addTableModelListener(TableModelListener l) {
		synchronized (listeners) {
			listeners.add(l);
		}
	}

	@Override
	public Class<?> getColumnClass(int columnIndex) {
		switch (columnIndex) {
			default:
				return columnOrder[columnIndex].getType();
		}
	}

	@Override
	public int getColumnCount() {
		return columnOrder.length;
	}

	@Override
	public String getColumnName(int columnIndex) {
		return columnOrder[columnIndex].name();
	}

	@Override
	public int getRowCount() {
		return fmcPhones.size();
	}

	@Override
	public Object getValueAt(int rowIndex, int columnIndex) {
		switch (columnIndex) {
			default:
				return Utils.getProperty(columnOrder[columnIndex], fmcPhones.get(rowIndex));
		}
	}

	@Override
	public boolean isCellEditable(int rowIndex, int columnIndex) {
		switch (columnIndex) {
			case 0:
				return false;
			case 1:
				return false;
			default:
				return true;
		}
	}

	public List<FmcPhone> getFmcPhones() {
		return fmcPhones;
	}

	public void setFmcPhones(List<FmcPhone> fmcPhones) {
		this.fmcPhones = fmcPhones;
		fireTableChanged(new TableModelEvent(this));
	}

	public FmcPhone getFmcPhone(int rowIndex) {
		return fmcPhones.get(rowIndex);
	}

	@Override
	public void removeTableModelListener(TableModelListener l) {
		synchronized (listeners) {
			listeners.remove(l);
		}
	}

	@Override
	public void setValueAt(Object value, int rowIndex, int columnIndex) {
		switch (columnIndex) {
			default:
				Utils.setProperty(columnOrder[columnIndex], fmcPhones.get(rowIndex), value);
		}
	}

	private void fireTableChanged(TableModelEvent e) {
		synchronized (listeners) {
			for (TableModelListener l : listeners) {
				l.tableChanged(e);
			}
		}
	}

}
