/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.examples.uci30.swing.viewmodel;

import de.starface.integration.uci.java.v30.types.FunctionKey;
import de.starface.integration.uci.java.v30.values.FunctionKeyProperties;
import de.starface.integration.uci.java.v30.values.FunctionKeyState;

import java.util.ArrayList;
import java.util.List;

import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

/**
 * A swing table model for the call list
 *
 * @author tilmann
 */
public class FunctionKeyTableModel implements TableModel {
	private static final FunctionKeyProperties[] columnOrder = new FunctionKeyProperties[6];

	private List<TableModelListener> listeners = new ArrayList<>();

	private String searchId = "";

	static {
		columnOrder[0] = FunctionKeyProperties.id;
		columnOrder[1] = FunctionKeyProperties.name;
		columnOrder[2] = FunctionKeyProperties.internalPhoneNumber;
		columnOrder[3] = FunctionKeyProperties.email;
		columnOrder[4] = FunctionKeyProperties.type;
		columnOrder[5] = FunctionKeyProperties.state;
	}

	private List<FunctionKey> functionKeys = new ArrayList<>();

	@Override
	public void addTableModelListener(TableModelListener listener) {
		listeners.add(listener);
	}

	@Override
	public Class<?> getColumnClass(int columnIndex) {
		return columnOrder[columnIndex].getType();
	}

	@Override
	public int getColumnCount() {
		return columnOrder.length;
	}

	@Override
	public String getColumnName(int columnIndex) {
		return columnOrder[columnIndex].name();
	}

	@Override
	public int getRowCount() {
		return functionKeys.size();
	}

	@Override
	public Object getValueAt(int rowIndex, int columnIndex) {
		return Utils.getProperty(columnOrder[columnIndex], functionKeys.get(rowIndex));
	}

	@Override
	public boolean isCellEditable(int rowIndex, int columnIndex) {
		return false;
	}

	@Override
	public void removeTableModelListener(TableModelListener listener) {
		listeners.remove(listener);
	}

	@Override
	public void setValueAt(Object value, int rowIndex, int columnIndex) {
		// Not implemented
	}

	public void setFunctionKeyState(FunctionKey newKey) {
		if (functionKeys == null) {
			return;
		}

		for (int i = 0; i < functionKeys.size(); i++) {
			FunctionKey oldKey = functionKeys.get(i);
			if (oldKey.getId().equals(newKey.getId())) {
				FunctionKeyState oldState = oldKey.getState();
				FunctionKeyState newState = newKey.getState();
				if (oldState != newState) {
					oldKey.setState(newState);
					fireTableChanged(new TableModelEvent(this, i));
				}
			}
		}
	}

	public FunctionKey getFunctionKey(int rowIndex) {
		return functionKeys.get(rowIndex);
	}

	private void fireTableChanged(TableModelEvent e) {
		synchronized (listeners) {
			for (TableModelListener l : listeners) {
				l.tableChanged(e);
			}
		}
	}

	public void setFunctionKeys(List<FunctionKey> functionKeys) {
		this.functionKeys = functionKeys;
		fireTableChanged(new TableModelEvent(this));
	}

	/**
	 * @return the searchId
	 */
	public String getSearchId() {
		return searchId;
	}

	/**
	 * @param searchId
	 *        the searchId to set
	 */
	public void setSearchId(String searchId) {
		this.searchId = searchId;
	}

}
