/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.examples.uci30.swing.viewmodel;

import de.starface.examples.uci30.swing.view.ToolTipCellRenderer;
import de.starface.examples.uci30.swing.view.ToolTipCellRenderer.TableToolTipProvider;
import de.starface.integration.uci.java.v30.exceptions.UciException;
import de.starface.integration.uci.java.v30.messages.events.UciFunctionKeyEvents;
import de.starface.integration.uci.java.v30.messages.requests.UciFunctionKeyRequests;
import de.starface.integration.uci.java.v30.types.CallInfo;
import de.starface.integration.uci.java.v30.types.ContactInfo;
import de.starface.integration.uci.java.v30.types.FunctionKey;
import de.starface.integration.uci.java.v30.types.FunctionKeySearchResult;
import de.starface.integration.uci.java.v30.types.Image;
import de.starface.integration.uci.java.v30.types.Mailbox;
import de.starface.integration.uci.java.v30.types.RedirectSetting;
import de.starface.integration.uci.java.v30.types.UserState;
import de.starface.integration.uci.java.v30.values.FunctionKeyType;
import de.starface.integration.uci.java.v30.values.RedirectSettingDestinationType;
import de.starface.integration.uci.java.v30.values.RedirectSettingType;
import de.starface.integration.uci.v30.client.UciServiceRequests;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.ArrayList;
import java.util.List;

import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

public class FunctionKeyViewModel implements UciFunctionKeyEvents {
	/// Domain Logic Access
	////////////////////////////////////////////////////////////////

	private UciFunctionKeyRequests functionKeyRequests;

	private PhoneToUseProvider phoneToUseProvider;

	public FunctionKeyViewModel(PhoneToUseProvider phoneToUseProvider, GeneralViewTasks generalViewTasks) {
		this.phoneToUseProvider = phoneToUseProvider;
		this.generalViewTasks = generalViewTasks;
	}

	public void setUciServiceRequests(UciServiceRequests uciServiceRequests) throws UciException {
		this.functionKeyRequests = uciServiceRequests.getRequests(UciFunctionKeyRequests.class);
		uciServiceRequests.subscribeEventsAsync(this, UciFunctionKeyEvents.class);
	}

	@Override
	public void receiveFunctionKeyState(FunctionKey functionKey) {
		functionKeyTableModel.setFunctionKeyState(functionKey);
	}

	@Override
	public void functionKeysChanged(List<FunctionKey> functionKeys) {
		functionKeyTableModel.setFunctionKeys(functionKeys);
	}

	@Override
	public void functionKeySearchTerminates(String searchId) {
		removeSearchResult(searchId);
	}

	/// View Actions
	////////////////////////////////////////////////////////////////

	static int cnt = 0;

	private void doGetFunctionKeys() {
		try {
			List<FunctionKey> functionKeys = functionKeyRequests.getFunctionKeys();
			functionKeyTableModel.setFunctionKeys(functionKeys);
		} catch (UciException e) {
			// Too simple error management. MUST be improved in real client!
			e.printStackTrace();
		} catch (NullPointerException e) {
			e.printStackTrace();
		}
	}

	private void doGetCallInfoForKey() {
		if (selectedRow >= 0) {
			FunctionKey functionKey = functionKeyTableModel.getFunctionKey(selectedRow);
			if (functionKey.getType() == FunctionKeyType.BUSY_LAMP_FIELD
				|| functionKey.getType() == FunctionKeyType.QUICK_DIAL) {
				try {
					List<CallInfo> callInfos = functionKeyRequests.getCallInfoForKey(functionKey.getId());
					generalViewTasks.displayInfo("Call Information", renderCallInfos(callInfos));
				} catch (UciException e) {
					// Too simple error management. MUST be improved in real client!
					e.printStackTrace();
				}
			} else {
				generalViewTasks.displayInfo("Call Information not available",
					"This information is only available for function keys of type Busy Lamp Field");
			}
		}
	}

	private void doGetContactInfoForKey() {
		if (selectedRow >= 0) {
			FunctionKey functionKey = functionKeyTableModel.getFunctionKey(selectedRow);
			if (functionKey.getType() == FunctionKeyType.BUSY_LAMP_FIELD
				|| functionKey.getType() == FunctionKeyType.QUICK_DIAL) {
				try {
					ContactInfo contactInfo = functionKeyRequests.getContactInfoForKey(functionKey.getId());
					generalViewTasks.displayInfo("Contact Information", renderContactInfo(contactInfo));
				} catch (UciException e) {
					// Too simple error management. MUST be improved in real client!
					e.printStackTrace();
				}
			} else {
				generalViewTasks.displayInfo("Contact Information not available",
					"This information is only available for function keys of type Busy Lamp Field");
			}
		}
	}

	private void doGetRedirectSettingsForKey() {
		if (selectedRow >= 0) {
			FunctionKey functionKey = functionKeyTableModel.getFunctionKey(selectedRow);
			if (functionKey.getType() == FunctionKeyType.BUSY_LAMP_FIELD
				|| functionKey.getType() == FunctionKeyType.QUICK_DIAL) {
				try {
					List<RedirectSetting> redirectSettings = functionKeyRequests.getRedirectSettingsForKey(functionKey.getId());
					generalViewTasks.displayInfo("Redirect Settings", renderRedirectSettings(redirectSettings));
				} catch (UciException e) {
					// Too simple error management. MUST be improved in real client!
					e.printStackTrace();
				}
			} else {
				generalViewTasks.displayInfo("Redirect Settings not available",
					"This information is only available for function keys of type Busy Lamp Field");
			}
		}
	}

	private void doTerminateSearchRequest() {
		if (selectedSearchResultRow >= 0) {
			try {
				functionKeyRequests.terminateSearchRequest(searchResults.get(selectedSearchResultRow).getSearchId());
			} catch (UciException e) {
				// Too simple error management. MUST be improved in real client!
				e.printStackTrace();
			}
		}
	}

	private void doSearchFunctionKeys() {
		if (!searchTerm.isEmpty()) {
			try {
				final FunctionKeySearchResult searchResult;
				switch (searchType) {
					case JABBERID:
						searchResult = functionKeyRequests.searchFunctionKeysByJabberId(searchTerm, searchId);
						break;
					case EMAIL:
						searchResult = functionKeyRequests.searchFunctionKeysByEmail(searchTerm, searchId);
						break;
					case ACCOUNTID:
						searchResult = functionKeyRequests.searchFunctionKeysByAccountId(Integer.parseInt(searchTerm), searchId);
						break;
					case TERM:
						//fall through to default case
					default:
						searchResult = functionKeyRequests.searchFunctionKeys(searchTerm, searchId);
				}
				setSelectedSearchResultRow(-1);
				setSelectedSearchResultRow(addSearchResult(searchResult));
				//updateSearchResultTableModel(searchResult);
			} catch (UciException e) {
				// Too simple error management. MUST be improved in real client!
				e.printStackTrace();
			}
		}
	}

	private void doGetImageForKey() {
		if (selectedRow >= 0) {
			FunctionKey functionKey = functionKeyTableModel.getFunctionKey(selectedRow);
			if (functionKey.getType() == FunctionKeyType.BUSY_LAMP_FIELD
				|| functionKey.getType() == FunctionKeyType.QUICK_DIAL) {
				try {
					Image image = functionKeyRequests.getImageForKey(functionKey.getId());
					if (image == null) {
						setAvatar(null);
					} else {
						Icon icon = UserStateViewModel.scaleImageToIcon(image, 58);
						if (icon != null) {
							setAvatar(icon);
						}
					}
				} catch (UciException e) {
					// Too simple error management. MUST be improved in real client!
					e.printStackTrace();
				}
			} else {
				generalViewTasks.displayInfo("Image not available",
					"This information is only available for function keys of type Busy Lamp Field");
			}
		}
	}

	private static String renderCallInfos(List<CallInfo> callInfos) {
		if (callInfos.isEmpty()) {
			return "No ongoing calls at the moment.";
		}
		StringBuilder sb = new StringBuilder();
		for (CallInfo callInfo : callInfos) {
			sb.append(renderCallInfo(callInfo)).append("\n");
		}
		return sb.toString();
	}

	private static String renderCallInfo(CallInfo callInfo) {
		StringBuilder sb = new StringBuilder();
		sb.append(callInfo.getDescription()).append(" ");
		sb.append("(").append(callInfo.getNumber()).append(") ");
		sb.append(callInfo.getState()).append(" ");
		sb.append(callInfo.getDuration()).append(" sec");
		return sb.toString();
	}

	private static String renderRedirectSettings(List<RedirectSetting> redirectSettings) {
		if (redirectSettings.isEmpty()) {
			return "No active redirect settings available.";
		}

		StringBuilder sbAlways = new StringBuilder();
		for (RedirectSetting redirectSetting : redirectSettings) {
			if (redirectSetting.getType() == RedirectSettingType.ALWAYS) {
				sbAlways.append(renderRedirectSetting(redirectSetting)).append("\n");
			}
		}
		if (sbAlways.length() > 0) {
			sbAlways.insert(0, "Always: \n");
		}

		StringBuilder sbBusy = new StringBuilder();
		for (RedirectSetting redirectSetting : redirectSettings) {
			if (redirectSetting.getType() == RedirectSettingType.BUSY) {
				sbBusy.append(renderRedirectSetting(redirectSetting)).append("\n");
			}
		}
		if (sbBusy.length() > 0) {
			sbBusy.insert(0, "Busy: \n");
		}

		StringBuilder sbTimeout = new StringBuilder();
		for (RedirectSetting redirectSetting : redirectSettings) {
			if (redirectSetting.getType() == RedirectSettingType.TIMEOUT) {
				sbTimeout.append(renderRedirectSetting(redirectSetting)).append("\n");
			}
		}
		if (sbTimeout.length() > 0) {
			sbTimeout.insert(0, "Timeout: \n");
		}

		return sbAlways.toString() + sbBusy.toString() + sbTimeout.toString();
	}

	private static String renderRedirectSetting(RedirectSetting redirectSetting) {
		StringBuilder sb = new StringBuilder();
		sb.append(redirectSetting.getCalledNumber()).append(" -> ").append(
			renderRedirectDestination(redirectSetting));
		return sb.toString();
	}

	private static String renderRedirectDestination(RedirectSetting redirectSetting) {
		if (redirectSetting.getDestinationType() == RedirectSettingDestinationType.VOICEMAIL) {
			return getMailboxName(redirectSetting.getDestination(), redirectSetting.getMailboxes());
		} else {
			return redirectSetting.getDestination();
		}
	}

	private static String getMailboxName(String id, List<Mailbox> mailboxes) {
		for (Mailbox mailbox : mailboxes) {
			if (id.equals(mailbox.getId())) {
				return mailbox.getName();
			}
		}
		return "Unknown";
	}

	private static String renderContactInfo(ContactInfo contactInfo) {
		StringBuilder sb = new StringBuilder();
		sb.append("FirstName: ").append(contactInfo.getFirstName()).append("\n");
		sb.append("FamilyName: ").append(contactInfo.getFamilyName()).append("\n");
		sb.append("JobTitle: ").append(contactInfo.getJobTitle()).append("\n");
		sb.append("Company: ").append(contactInfo.getCompany()).append("\n");
		sb.append("Street: ").append(contactInfo.getStreet()).append("\n");
		sb.append("PostCode: ").append(contactInfo.getPostCode()).append("\n");
		sb.append("City: ").append(contactInfo.getCity()).append("\n");
		sb.append("Email: ").append(contactInfo.getEmail()).append("\n");
		sb.append("JabberId: ").append(contactInfo.getJabberId()).append("\n");
		sb.append("Phone: ").append(contactInfo.getPhone()).append("\n");
		sb.append("MobilePhone: ").append(contactInfo.getMobilePhone()).append("\n");
		sb.append("HomePhone: ").append(contactInfo.getHomePhone()).append("\n");
		sb.append("Phone2: ").append(contactInfo.getPhone2()).append("\n");
		sb.append("InternalPhone: ").append(contactInfo.getInternalPhone()).append("\n");
		sb.append("Fax: ").append(contactInfo.getFax());
		return sb.toString();
	}

	private void doPressFunctionKey() {
		if (selectedRow >= 0) {
			FunctionKey functionKey = functionKeyTableModel.getFunctionKey(selectedRow);
			try {
				if (functionKey.getType() == FunctionKeyType.BUSY_LAMP_FIELD
					|| functionKey.getType() == FunctionKeyType.QUICK_DIAL) {
					functionKeyRequests.pressKey(functionKey.getId(), phoneToUseProvider.getPhoneToUse());
				} else {
					functionKeyRequests.pressKey(functionKey.getId());
				}
			} catch (UciException e) {
				// Too simple error management. MUST be improved in real client!
				e.printStackTrace();
			}
		}
	}

	private final ActionListener getFunctionKeyActionListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			doGetFunctionKeys();
		}
	};

	private final ActionListener pressFunctionKeyActionListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			doPressFunctionKey();
		}
	};

	private final ActionListener getCallInfoForKeyActionListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			doGetCallInfoForKey();
		}
	};

	private final ActionListener getContactInfoForKeyActionListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			doGetContactInfoForKey();
		}
	};

	private final ActionListener getRedirectSettingsForKeyActionListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			doGetRedirectSettingsForKey();
		}
	};

	private final ActionListener getImageForKeyActionListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			doGetImageForKey();
		}
	};

	private final ActionListener terminateSearchRequestListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			doTerminateSearchRequest();
		}
	};

	private final ActionListener searchFunctionKeysListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			doSearchFunctionKeys();
		}
	};

	private final ListSelectionListener searchResultListSelectionListener = new ListSelectionListener() {
		@Override
		public void valueChanged(ListSelectionEvent e) {
			if (selectedSearchResultRow >= 0 && selectedSearchResultRow < searchResults.size()) {
				FunctionKeySearchResult searchResult = searchResults.get(selectedSearchResultRow);
				updateSearchResultTableModel(searchResult);
			} else {
				clearSearchResultTableModel();
			}
		}
	};

	/**
	 *
	 * @param searchResult
	 *
	 *        if searchId exists, update its keys; else, add the keys
	 *
	 *        returns the index that the result was added at
	 */
	private int addSearchResult(FunctionKeySearchResult searchResult) {
		int index = -1;

		for (int i = 0; i < searchResults.size(); i++) {
			if (searchResults.get(i).getSearchId().equals(searchResult.getSearchId())) {
				index = i;
				searchResults.get(i).setResultList(searchResult.getResultList());
				break;
			}
		}

		if (index == -1) {
			index = searchResults.size();
			searchResults.add(searchResult);
			updateSearchResults();
		}
		return index;
	}

	private void removeSearchResult(String searchId) {
		for (int i = 0; i < searchResults.size(); i++) {
			if (searchResults.get(i).getSearchId().equals(searchId)) {
				searchResults.remove(i);
				break;
			}
		}
		updateSearchResults();
		if (searchResultTableModel.getSearchId().equals(searchId)) {
			setSelectedSearchResultRow(-1);
		}
	}

	private void updateSearchResults() {
		List<String> tmp = new ArrayList<>();
		if (searchResults != null) {
			for (FunctionKeySearchResult result : searchResults) {
				if (result == null) {
					continue;
				}
				tmp.add(result.getSearchTerm() + " [" + result.getSearchId() + "]");
			}
		}

		setSearchResultList(tmp);
	}

	private void updateSearchResultTableModel(FunctionKeySearchResult searchResult) {
		searchResultTableModel.setSearchId(searchResult.getSearchId());
		List<FunctionKey> keys = new ArrayList<>(searchResult.getResultList().values());
		searchResultTableModel.setFunctionKeys(keys);
	}

	private void clearSearchResultTableModel() {
		searchResultTableModel.setSearchId("");
		searchResultTableModel.setFunctionKeys(new ArrayList<>());
	}

	/// View Data
	////////////////////////////////////////////////////////////////

	private final GeneralViewTasks generalViewTasks;

	private final FunctionKeyTableModel functionKeyTableModel = new FunctionKeyTableModel();

	private final FunctionKeyTableModel searchResultTableModel = new FunctionKeyTableModel();

	private List<FunctionKeySearchResult> searchResults = new ArrayList<>();

	private List<String> searchResultList = new ArrayList<>();

	private int selectedRow = -1;

	private String selectedSearchResult = "";

	private int selectedSearchResultRow = -1;

	private Icon image;

	private String searchId = "";

	private String searchTerm = "";

	public SearchType getSearchType() {
		return searchType;
	}

	public void setSearchType(final SearchType searchType) {
		this.searchType = searchType;
	}

	private SearchType searchType = SearchType.TERM;

	public enum SearchType {
		TERM,
		ACCOUNTID,
		JABBERID,
		EMAIL
	}

	private PropertyChangeSupport changeSupport = new PropertyChangeSupport(this);

	private ToolTipCellRenderer.TableToolTipProvider toolTipProvider = new TableToolTipProvider() {
		@Override
		public String getToolTipForCell(JTable table, Object value, boolean isSelected, boolean hasFocus, int row,
										int column) {
			FunctionKey fk = functionKeyTableModel.getFunctionKey(row);
			if (fk.getType() != FunctionKeyType.BUSY_LAMP_FIELD) {
				return null;
			} else {
				return createToolTipText(fk.getUserState());
			}
		}
	};

	private String createToolTipText(UserState userState) {
		return "telephonyState: " + userState.getTelephonyState() + "\ndoNotDisturbSetting: "
			+ userState.isDoNotDisturbSetting() + "\nredirectAlwaysActive: " + userState.isRedirectAlwaysActive()
			+ "\nchatPresence: " + userState.getChatPresence() + "\nchatPresenceMessage: "
			+ userState.getChatPresenceMessage() + "\navatarHash: " + userState.getAvatarHash();
	}

	public void addPropertyChangeListener(PropertyChangeListener listener) {
		changeSupport.addPropertyChangeListener(listener);
	}

	public void removePropertyChangeListener(PropertyChangeListener listener) {
		changeSupport.removePropertyChangeListener(listener);
	}

	/// Property Getters and Setters
	////////////////////////////////////////////////////////////////

	public static final String PROPERTY_SEARCH_ID = "searchId";

	/**
	 * @return the searchId
	 */
	public String getSearchId() {
		return searchId;
	}

	/**
	 * @param searchId
	 *        the searchId to set
	 */
	public void setSearchId(String searchId) {
		this.searchId = searchId;
	}

	public static final String PROPERTY_SEARCH_TERM = "searchTerm";

	/**
	 * @return the searchTerm
	 */
	public String getSearchTerm() {
		return searchTerm;
	}

	/**
	 * @param searchTerm
	 *        the searchTerm to set
	 */
	public void setSearchTerm(String searchTerm) {
		this.searchTerm = searchTerm;
	}

	public static final String PROPERTY_SELECTED_SEARCH_RESULT = "selectedSearchResult";

	/**
	 * @return the selectedSearchResult
	 */
	public String getSelectedSearchResult() {
		return selectedSearchResult;
	}

	/**
	 * @param selectedSearchResult
	 *        the selectedSearchResult to set
	 */
	public void setSelectedSearchResult(String selectedSearchResult) {
		this.selectedSearchResult = selectedSearchResult;
	}

	public List<String> getSearchResultList() {
		return searchResultList;
	}

	public static final String PROPERTY_SEARCH_RESULT_LIST = "searchResultList";

	public void setSearchResultList(List<String> searchResultList) {
		List<String> old = this.searchResultList;
		this.searchResultList = searchResultList;
		changeSupport.firePropertyChange(PROPERTY_SEARCH_RESULT_LIST, old, searchResultList);
	}

	public static final String PROPERTY_SELECTED_SEARCH_RESULT_ROW = "selectedSearchResultRow";

	public int getSelectedSearchResultRow() {
		return selectedSearchResultRow;
	}

	public void setSelectedSearchResultRow(int selectedSearchResultRow) {
		int old = this.selectedSearchResultRow;
		this.selectedSearchResultRow = selectedSearchResultRow;
		changeSupport.firePropertyChange(PROPERTY_SELECTED_SEARCH_RESULT_ROW, old, selectedSearchResultRow);
	}

	public static final String PROPERTY_SELECTED_ROW = "selectedRow";

	public int getSelectedRow() {
		return selectedRow;
	}

	public void setSelectedRow(int selectedRow) {
		int old = this.selectedRow;
		this.selectedRow = selectedRow;
		changeSupport.firePropertyChange(PROPERTY_SELECTED_ROW, old, selectedRow);
	}

	public static final String PROPERTY_IMAGE = "image";

	public Icon getImage() {
		return image;
	}

	public void setAvatar(Icon image) {
		Icon old = this.image;
		this.image = image;
		changeSupport.firePropertyChange(PROPERTY_IMAGE, old, image);
	}

	public FunctionKeyTableModel getFunctionKeyTableModel() {
		return functionKeyTableModel;
	}

	public FunctionKeyTableModel getSearchResultTableModel() {
		return searchResultTableModel;
	}

	public ActionListener getPressFunctionKeyActionListener() {
		return pressFunctionKeyActionListener;
	}

	public ActionListener getGetFunctionKeysActionListener() {
		return getFunctionKeyActionListener;
	}

	public ActionListener getGetCallInfoForKeysActionListener() {
		return getCallInfoForKeyActionListener;
	}

	public ActionListener getGetContactInfoForKeysActionListener() {
		return getContactInfoForKeyActionListener;
	}

	public ActionListener getGetRedirectSettingsForKeysActionListener() {
		return getRedirectSettingsForKeyActionListener;
	}

	public ActionListener getGetImageForKeysActionListener() {
		return getImageForKeyActionListener;
	}

	public ActionListener getTerminateSearchRequestListener() {
		return terminateSearchRequestListener;
	}

	public ActionListener getSearchFunctionKeysListener() {
		return searchFunctionKeysListener;
	}

	public ListSelectionListener getSearchResultListSelectionListener() {
		return searchResultListSelectionListener;
	}

	public ToolTipCellRenderer.TableToolTipProvider getToolTipProvider() {
		return toolTipProvider;
	}
}
