/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.examples.uci30.swing.viewmodel;

import de.starface.integration.uci.java.v30.types.GroupSetting;
import de.starface.integration.uci.java.v30.types.PhoneNumber;
import de.starface.integration.uci.java.v30.values.GroupSettingProperties;

import java.util.ArrayList;
import java.util.List;

import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

/**
 * A swing table model for the redirect settings
 *
 * @author tilmann
 */
public class GroupSettingsTableModel implements TableModel {
	private static final GroupSettingProperties[] columnOrder;

	static {
		columnOrder = new GroupSettingProperties[4];
		columnOrder[0] = GroupSettingProperties.name;
		columnOrder[1] = GroupSettingProperties.logonId;
		columnOrder[2] = GroupSettingProperties.phoneNumbers;
		columnOrder[3] = GroupSettingProperties.loggedOn;
	}

	private List<TableModelListener> listeners = new ArrayList<>(1);

	private List<GroupSetting> groupSettings = new ArrayList<>();

	@Override
	public void addTableModelListener(TableModelListener l) {
		synchronized (listeners) {
			listeners.add(l);
		}
	}

	@Override
	public Class<?> getColumnClass(int columnIndex) {
		switch (columnIndex) {
			case 2:
				return String.class;
			default:
				return columnOrder[columnIndex].getType();
		}
	}

	@Override
	public int getColumnCount() {
		return columnOrder.length;
	}

	@Override
	public String getColumnName(int columnIndex) {
		return columnOrder[columnIndex].name();
	}

	@Override
	public int getRowCount() {
		return groupSettings.size();
	}

	@Override
	public Object getValueAt(int rowIndex, int columnIndex) {
		switch (columnIndex) {
			case 2:
				return phoneNumberListToString(groupSettings.get(rowIndex).getPhoneNumbers());
			default:
				return Utils.getProperty(columnOrder[columnIndex], groupSettings.get(rowIndex));
		}
	}

	private String phoneNumberListToString(List<PhoneNumber> phoneNumbers) {
		StringBuilder sb = new StringBuilder();
		if (!phoneNumbers.isEmpty()) {
			sb.append(phoneNumbers.get(0).getNumber());
		}
		for (int i = 1; i < phoneNumbers.size(); i++) {
			sb.append(", ");
			sb.append(phoneNumbers.get(i).getNumber());
		}
		return sb.toString();
	}

	@Override
	public boolean isCellEditable(int rowIndex, int columnIndex) {
		switch (columnIndex) {
			case 3:
				return true;
			default:
				return false;
		}
	}

	public List<GroupSetting> getGroupSettings() {
		return groupSettings;
	}

	public void setGroupSettings(List<GroupSetting> groupSettings) {
		this.groupSettings = groupSettings;
		fireTableChanged(new TableModelEvent(this));
	}

	public GroupSetting getGroupSetting(int rowIndex) {
		return groupSettings.get(rowIndex);
	}

	@Override
	public void removeTableModelListener(TableModelListener l) {
		synchronized (listeners) {
			listeners.remove(l);
		}
	}

	@Override
	public void setValueAt(Object value, int rowIndex, int columnIndex) {
		switch (columnIndex) {
			case 3:
				Utils.setProperty(columnOrder[columnIndex], groupSettings.get(rowIndex), value);
			default:
		}
	}

	private void fireTableChanged(TableModelEvent e) {
		synchronized (listeners) {
			for (TableModelListener l : listeners) {
				l.tableChanged(e);
			}
		}
	}

}
