/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.examples.uci30.swing.viewmodel;

import de.starface.integration.uci.java.v30.exceptions.UciException;
import de.starface.integration.uci.java.v30.messages.events.UciMonitorListEvents;
import de.starface.integration.uci.java.v30.messages.requests.UciMonitorListRequests;
import de.starface.integration.uci.java.v30.types.MonitorList;
import de.starface.integration.uci.java.v30.types.MonitorListEntry;
import de.starface.integration.uci.java.v30.values.MonitorListEntryProperties;
import de.starface.integration.uci.java.v30.values.OrderDirection;
import de.starface.integration.uci.v30.client.UciServiceRequests;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.File;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.TableModel;

public class MonitorCallViewModel implements UciMonitorListEvents
{
	private UciMonitorListRequests uciMonitorListRequests;

	public MonitorCallViewModel() {
	}

	public void setUciServiceRequests(UciServiceRequests uciServiceRequests) throws UciException {
		this.uciMonitorListRequests = uciServiceRequests.getRequests(UciMonitorListRequests.class);
		uciServiceRequests.subscribeEventsAsync(this, UciMonitorListEvents.class);
	}

	private final PropertyChangeSupport changeSupport = new PropertyChangeSupport(this);

	public void addPropertyChangeListener(PropertyChangeListener listener) {
		changeSupport.addPropertyChangeListener(listener);
	}

	public void removePropertyChangeListener(PropertyChangeListener listener) {
		changeSupport.removePropertyChangeListener(listener);
	}

	///////// PROPERTYS

	private MonitorListTableModel monitorListTableModel = new MonitorListTableModel();

	private List<Integer> selectedRows = new ArrayList<>();

	private boolean downloadEntryIsAllowed = false;
	private boolean deleteEntriesIsAllowed = false;
	private boolean deleteEntryIsAllowed = false;
	private boolean getDownloadPathIsAllowed = false;

	public static final String GET_DOWNLOAD_PATH_IS_ALLOWED = "getDownloadPathIsAllowed";

	public boolean getGetDownloadPathIsAllowed() {
		return getDownloadPathIsAllowed;
	}

	public void setGetDownloadPathIsAllowed(boolean getDownloadPathIsAllowed) {
		boolean old = this.getDownloadPathIsAllowed;
		this.getDownloadPathIsAllowed = getDownloadPathIsAllowed;
		changeSupport.firePropertyChange(GET_DOWNLOAD_PATH_IS_ALLOWED, old, getDownloadPathIsAllowed);
	}

	public static final String DELETE_ENTRY_IS_ALLOWED = "deleteEntryIsAllowed";

	public boolean getDeleteEntryIsAllowed() {
		return deleteEntryIsAllowed;
	}

	public void setDeleteEntryIsAllowed(boolean deleteEntryIsAllowed) {
		boolean old = this.deleteEntryIsAllowed;
		this.deleteEntryIsAllowed = deleteEntryIsAllowed;
		changeSupport.firePropertyChange(DELETE_ENTRY_IS_ALLOWED, old, deleteEntryIsAllowed);
	}

	public static final String DELETE_ENTRIES_IS_ALLOWED = "deleteEntriesIsAllowed";

	public boolean getDeleteEntriesIsAllowed() {
		return deleteEntriesIsAllowed;
	}

	public void setDeleteEntriesIsAllowed(boolean deleteEntriesIsAllowed) {
		boolean old = this.deleteEntriesIsAllowed;
		this.deleteEntriesIsAllowed = deleteEntriesIsAllowed;
		changeSupport.firePropertyChange(DELETE_ENTRIES_IS_ALLOWED, old, deleteEntriesIsAllowed);
	}

	public static final String DOWNLOAD_ENTRY_IS_ALLOWED = "downloadEntryIsAllowed";

	public boolean getdownloadEntryIsAllowed() {
		return downloadEntryIsAllowed;
	}

	public void setDownloadEntryIsAllowed(boolean downloadEntryIsAllowed) {
		boolean old = this.downloadEntryIsAllowed;
		this.downloadEntryIsAllowed = downloadEntryIsAllowed;
		changeSupport.firePropertyChange(DOWNLOAD_ENTRY_IS_ALLOWED, old, downloadEntryIsAllowed);
	}

	public TableModel getMonitorListTableModel() {
		return monitorListTableModel;
	}

	public List<Integer> getSelectedRows() {
		return selectedRows;
	}

	private List<String> selectedMonitorEntryIds = new ArrayList<>();

	public List<String> getSelectedMonitorEntryIds() {
		return selectedMonitorEntryIds;
	}

	public void setSelectedMonitorEntryIds(List<String> selectedMonitorEntryIds) {
		//List<MonitorListEntry> old = this.selectedMonitorEntries;
		this.selectedMonitorEntryIds = selectedMonitorEntryIds;
	}

	public static final String DOWNLOAD_PATH_MONITOR = "downloadPathMonitor";

	private String downloadPathMonitor;

	public String getDownloadPathMonitor() {
		return downloadPathMonitor;
	}

	public void setDownloadPathMonitor(String downloadPathMonitor) {
		String old = this.downloadPathMonitor;
		this.downloadPathMonitor = downloadPathMonitor;
		changeSupport.firePropertyChange(DOWNLOAD_PATH_MONITOR, old, downloadPathMonitor);
	}

	public static final String PROPERTY_ORDER_DIRECTION_MONITOR = "orderDirectionMonitor";

	private OrderDirection orderDirectionMonitor = null;

	public OrderDirection getOrderDirectionMonitor() {
		return orderDirectionMonitor;
	}

	public void setOrderDirectionMonitor(OrderDirection orderDirectionMonitor) {
		OrderDirection old = this.orderDirectionMonitor;
		this.orderDirectionMonitor = orderDirectionMonitor;
		changeSupport.firePropertyChange(PROPERTY_ORDER_DIRECTION_MONITOR, old, orderDirectionMonitor);
	}

	public OrderDirection[] getOrderDirectionMonitorValues() {
		return Utils.createArrayWithLeadingNull(OrderDirection.values());
	}

	public static final String MONITOR_LIST_ENTRY_PROPERTIES = "monitorListEntryProperties";

	private MonitorListEntryProperties monitorListEntryProperties = null;

	public MonitorListEntryProperties getMonitorListEntryProperties() {
		return monitorListEntryProperties;
	}

	public void setMonitorListEntryProperties(MonitorListEntryProperties monitorListEntryProperties) {
		MonitorListEntryProperties old = this.monitorListEntryProperties;
		this.monitorListEntryProperties = monitorListEntryProperties;
		changeSupport.firePropertyChange(MONITOR_LIST_ENTRY_PROPERTIES, old, monitorListEntryProperties);
	}

	public MonitorListEntryProperties[] getMonitorListEntryPropertiesValues() {
		return Utils.createArrayWithLeadingNull(MonitorListEntryProperties.values());
	}

	public static final String OFFSET_MONITOR = "offsetMonitor";

	private String offsetMonitor = "0";

	public String getOffsetMonitor() {
		return offsetMonitor;
	}

	public void setOffsetMonitor(String offsetMonitor) {
		String old = this.offsetMonitor;
		this.offsetMonitor = offsetMonitor;
		changeSupport.firePropertyChange(OFFSET_MONITOR, old, offsetMonitor);
	}

	public static final String LIMIT_MONITOR = "limitMonitor";

	private String limitMonitor = "15";

	public String getLimitMonitor() {
		return limitMonitor;
	}

	public void setLimitMonitor(String limitMonitor) {
		String old = this.limitMonitor;
		this.limitMonitor = limitMonitor;
		changeSupport.firePropertyChange(LIMIT_MONITOR, old, limitMonitor);
	}

	public static final String STARTED_AFTER_MONITOR = "startedAfterMonitor";

	private java.util.Date startedAfterMonitor = new java.util.Date(0);

	public Date getStartedAfterMonitor() {
		return startedAfterMonitor;
	}

	public void setStartedAfterMonitor(Date startedAfterMonitor) {
		Date old = this.startedAfterMonitor;
		this.startedAfterMonitor = startedAfterMonitor;
		changeSupport.firePropertyChange(STARTED_AFTER_MONITOR, old, startedAfterMonitor);
	}

	public static final String STARTED_BEFORE_MONITOR = "startedBeforeMonitor";

	private java.util.Date startedBeforeMonitor = new java.util.Date(System.currentTimeMillis());

	public Date getStartedBeforeMonitor() {
		return startedBeforeMonitor;
	}

	public void setStartedBeforeMonitor(Date startedBeforeMonitor) {
		Date old = this.startedBeforeMonitor;
		this.startedBeforeMonitor = startedBeforeMonitor;
		changeSupport.firePropertyChange(STARTED_BEFORE_MONITOR, old, startedBeforeMonitor);
	}

	private void updateSelectedListEntries() {
		setDownloadPathMonitor("");
		List<String> tmp = new ArrayList<>();
		selectedMonitorEntryIds.clear();
		for (int row : selectedRows) {
			tmp.add(monitorListTableModel.getListEntry(row).getId());
		}
		selectedMonitorEntryIds = tmp;
	}

	private void doCheckSelectedRows() {
		if (selectedRows.isEmpty()) {
			//disable buttons
			setGetDownloadPathIsAllowed(false);
			setDownloadEntryIsAllowed(false);
			setDeleteEntryIsAllowed(false);
			setDeleteEntriesIsAllowed(false);
		} else if (selectedRows.size() == 1) {
			setGetDownloadPathIsAllowed(true);
			setDownloadEntryIsAllowed(true);
			setDeleteEntryIsAllowed(true);
			setDeleteEntriesIsAllowed(true);
		} else {
			setGetDownloadPathIsAllowed(false);
			setDownloadEntryIsAllowed(false);
			setDeleteEntryIsAllowed(false);
			setDeleteEntriesIsAllowed(true);
		}
	}

	////////////////////////////////////////////////////////////

	public ListSelectionListener getSelectedRowsListener() {
		return selectedRowsListSelectionListener;
	}

	private final ListSelectionListener selectedRowsListSelectionListener = new ListSelectionListener() {
		@Override
		public void valueChanged(ListSelectionEvent e) {
			updateSelectedListEntries();
			doCheckSelectedRows();
		}
	};

	public ActionListener getAllMonitorsActionListener() {
		return allMonitorsActionListener;
	}

	private ActionListener allMonitorsActionListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			doGetAllMonitorsActionListener();
		}
	};

	public ActionListener getDeleteMonitorEntriesActionListener() {
		return deleteMonitorEntriesActionListener;
	}

	public ActionListener getDownloadMonitorEntryActionListener() {
		return doDownloadMonitorEntryActionListener;
	}

	public ActionListener getGetDownloadFileUrlMonitorEntryActionListener() {
		return doGetDownloadFileUrlMonitorEntryActionListener;
	}

	public ActionListener getDeleteMonitorEntryActionListener() {
		return deleteMonitorEntryActionListener;
	}

	private ActionListener deleteMonitorEntryActionListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			doDeleteMonitorEntry();
		}
	};

	private ActionListener deleteMonitorEntriesActionListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			doDeleteMonitorEntries();
		}
	};

	private void doGetAllMonitorsActionListener() {
		try {
			int offset = Integer.parseInt(offsetMonitor);
			int limit = Integer.parseInt(limitMonitor);
			// Get list entries
			MonitorList monitorList = uciMonitorListRequests.getMonitorList(startedAfterMonitor,
				startedBeforeMonitor, monitorListEntryProperties, orderDirectionMonitor, offset, limit);

			monitorListTableModel.setMonitorList(monitorList.getEntries());
		} catch (UciException ex) {
			// Too simple error management. MUST be improved in real client!
			ex.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	private void doDeleteMonitorEntry() {
		try {
			if (selectedMonitorEntryIds.size() == 1) {
				final String id = selectedMonitorEntryIds.get(0);
				uciMonitorListRequests.deleteMonitorListEntry(id);
				monitorListTableModel.removeEntry(id);
			}
		} catch (UciException ex) {
			// Too simple error management. MUST be improved in real client!
			ex.printStackTrace();
		}
	}

	private void doDeleteMonitorEntries() {
		try {
			if (!selectedMonitorEntryIds.isEmpty()) {
				uciMonitorListRequests.deleteMonitorListEntries(selectedMonitorEntryIds);
				monitorListTableModel.removeEntries(selectedMonitorEntryIds);
			}
		} catch (UciException ex) {
			// Too simple error management. MUST be improved in real client!
			ex.printStackTrace();
		}
	}

	private ActionListener doDownloadMonitorEntryActionListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			doDownloadMonitorEntry();
		}
	};

	private void doDownloadMonitorEntry() {
		try {
			if (selectedMonitorEntryIds.size() == 1) {
				File file = uciMonitorListRequests.getMonitorFile(selectedMonitorEntryIds.get(0));
				Desktop desktop = Desktop.getDesktop();
				try {
					desktop.open(file);
				} catch (Exception exception) {
					// Too simple error management. MUST be improved in real client!
					exception.printStackTrace();
				}
			}
		} catch (UciException ex) {
			// Too simple error management. MUST be improved in real client!
			ex.printStackTrace();
		}
	}

	private ActionListener doGetDownloadFileUrlMonitorEntryActionListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			doGetDownloadFileUrlMonitorEntry();
		}
	};

	private void doGetDownloadFileUrlMonitorEntry() {
		try {
			if (selectedMonitorEntryIds.size() == 1) {
				String fileUrl = uciMonitorListRequests.getMonitorFileUrl(selectedMonitorEntryIds.get(0));
				setDownloadPathMonitor(fileUrl);
			}
		} catch (UciException ex) {
			// Too simple error management. MUST be improved in real client!
			ex.printStackTrace();
		}
	}

	@Override
	public void newMonitorListEntry(MonitorListEntry monitorListEntry) {
		monitorListTableModel.addEntry(monitorListEntry);
	}

	@Override
	public void deletedMonitorListEntries(List<String> deletedMonitorEntryIds) {
		for (String id : deletedMonitorEntryIds) {
			monitorListTableModel.removeEntry(id);
		}
	}
}
