/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.examples.uci30.swing.viewmodel;

import de.starface.integration.uci.java.v30.exceptions.UciException;
import de.starface.integration.uci.java.v30.messages.events.UciFmcPhoneEvents;
import de.starface.integration.uci.java.v30.messages.events.UciPhoneEvents;
import de.starface.integration.uci.java.v30.messages.requests.UciPhoneRequests;
import de.starface.integration.uci.java.v30.types.FmcPhone;
import de.starface.integration.uci.java.v30.types.PhoneNumber;
import de.starface.integration.uci.java.v30.types.PushNotificationSetting;
import de.starface.integration.uci.java.v30.types.SoftPhoneConfig;
import de.starface.integration.uci.java.v30.types.SoftPhoneRegistrationInfo;
import de.starface.integration.uci.java.v30.types.SoftPhoneSetting;
import de.starface.integration.uci.v30.client.UciServiceRequests;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

public class PhoneViewModel implements PhoneToUseProvider, UciPhoneEvents, UciFmcPhoneEvents {
	/// Domain Logic Access
	////////////////////////////////////////////////////////////////

	private UciPhoneRequests phoneRequests;

	private GeneralViewTasks generalViewTasks;

	private static final SoftPhoneSetting SOFTPHONESETTING_WINCLIENT = new SoftPhoneSetting(
		"163C00A2-C2F1-4FFE-9474-49283C379852");
	private static final SoftPhoneSetting SOFTPHONESETTING_MACCLIENT = new SoftPhoneSetting(
		"E22FF5B3-38ED-4A5E-966A-80B4785A2FF7");

	public PhoneViewModel(GeneralViewTasks generalViewTasks) {
		this.generalViewTasks = generalViewTasks;
	}

	public void setUciServiceRequests(UciServiceRequests uciServiceRequests) throws UciException {
		this.phoneRequests = uciServiceRequests.getRequests(UciPhoneRequests.class);
		uciServiceRequests.subscribeEvents(this, UciPhoneEvents.class);
		uciServiceRequests.subscribeEvents(this, UciFmcPhoneEvents.class);
	}

	/**
	 * @see de.starface.examples.uci30.swing.viewmodel.PhoneToUseProvider#getPhoneToUse()
	 */
	@Override
	public String getPhoneToUse() {
		return selectedPhoneId.getPhoneId();
	}

	/// View Actions
	////////////////////////////////////////////////////////////////

	private void doGetPhoneIds() {
		try {
			// Retrieve phone ids sending getPhoneIds request
			updatePhoneIds(phoneRequests.getPhoneIds());
		} catch (UciException e) {
			// Too simple error management. MUST be improved in real client!
			e.printStackTrace();
		}
	}

	private void doGetAvailableDisplayNumbers() {
		try {
			updateDisplayNumbers(phoneRequests.getAvailableDisplayNumbers());
		} catch (UciException e) {
			// Too simple error management. MUST be improved in real client!
			e.printStackTrace();
		}
	}

	private void doTransferSelectedPhoneId() {
		setPrimaryPhoneId(selectedPhoneId.getPhoneId());
	}

	private void doTransferSelectedDisplayNumber() {
		setDisplayNumber(selectedDisplayNumber);
	}

	private void doGetPrimaryPhoneId() {
		try {
			setPrimaryPhoneId(phoneRequests.getPrimaryPhoneId());
		} catch (UciException e) {
			// Too simple error management. MUST be improved in real client!
			e.printStackTrace();
		}
	}

	private void doSetPrimaryPhoneId() {
		try {
			if (!"".equals(primaryPhoneId)) {
				phoneRequests.setPrimaryPhoneId(primaryPhoneId);
			}
		} catch (UciException e) {
			// Too simple error management. MUST be improved in real client!
			e.printStackTrace();
		}
	}

	private void doRegisterSipPhone(SoftPhoneSetting setting, PushNotificationSetting pushNotificationSetting) {
		try {
			SoftPhoneConfig config = phoneRequests.registerSipPhoneWithPushNotifications(setting, pushNotificationSetting);
			generalViewTasks.displayInfo("SoftPhoneConfig", "SipAccount: " + config.getSipUserId()
				+ ",\n Passwort: " + config.getSipPassword() + ",\n Realm: " + config.getRealm());

			generalViewTasks.displayInfo("Additional IPs",
				Arrays.toString(phoneRequests.getAvailableServerAddresses().toArray()));
		} catch (UciException e) {
			// Too simple error management. MUST be improved in real client!
			e.printStackTrace();
		}
	}

	private void isSoftPhoneInUse(SoftPhoneSetting setting, String client) {
		try {
			boolean isInUse = phoneRequests.isSipPhoneInUse(setting);
			String isInUseString = isInUse ? "" : "not";
			generalViewTasks.displayInfo("isSoftPhoneInUse", "SoftPhone " + client + " is " + isInUseString
				+ " in use");
		} catch (UciException e) {
			// Too simple error management. MUST be improved in real client!
			e.printStackTrace();
		}
	}

	private void doGetDisplayNumberId() {
		try {
			String id = phoneRequests.getDisplayNumberId();
			setDisplayNumber(getPhoneNumberViewModelForNumberId(id));
		} catch (UciException e) {
			// Too simple error management. MUST be improved in real client!
			e.printStackTrace();
		}
	}

	private PhoneNumberViewModel getPhoneNumberViewModelForNumberId(String id) {
		doGetAvailableDisplayNumbers();
		if (displayNumbers.isEmpty()) {
			return null;
		}
		for (PhoneNumberViewModel pnvm : displayNumbers) {
			if (pnvm.getPhoneNumber() != null && pnvm.getPhoneNumber().getId().equals(id)) {
				return pnvm;
			}
		}
		return new PhoneNumberViewModel();
	}

	private void doSetDisplayNumberId() {
		try {
			PhoneNumber number = displayNumber != null ? displayNumber.getPhoneNumber() : null;
			if (number != null) {
				phoneRequests.setDisplayNumberId(number.getId());
			} else {
				phoneRequests.setDisplayNumberId("");
			}
		} catch (UciException e) {
			// Too simple error management. MUST be improved in real client!
			e.printStackTrace();
		}
	}

	private void doGetCallWaitingIndication() {
		try {
			setCallWaitingIndication(phoneRequests.getCallWaitingIndication());
		} catch (UciException e) {
			// Too simple error management. MUST be improved in real client!
			e.printStackTrace();
		}
	}

	private void doSetCallWaitingIndication() {
		try {
			phoneRequests.setCallWaitingIndication(callWaitingIndication);
		} catch (UciException e) {
			// Too simple error management. MUST be improved in real client!
			e.printStackTrace();
		}
	}

	private final ActionListener getPhoneIdsActionListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			doGetPhoneIds();
		}
	};

	private final ActionListener transferSelectedPhoneIdActionListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			doTransferSelectedPhoneId();
		}
	};

	private final ActionListener getPrimaryPhoneIdActionListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			doGetPrimaryPhoneId();
		}
	};

	private final ActionListener registerWinSoftPhoneActionListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			doRegisterSipPhone(SOFTPHONESETTING_WINCLIENT, new PushNotificationSetting(
				"dummyService", "dummyToken_" + System.currentTimeMillis()));
		}
	};

	private final ActionListener registerMacSoftPhoneActionListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			doRegisterSipPhone(SOFTPHONESETTING_MACCLIENT, new PushNotificationSetting(
				"dummyService", "dummyToken_" + System.currentTimeMillis()));
		}
	};

	private final ActionListener isWinSoftPhoneInUseActionListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			isSoftPhoneInUse(SOFTPHONESETTING_WINCLIENT, "WinClient");
		}
	};

	private final ActionListener isWacSoftPhoneInUseActionListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			isSoftPhoneInUse(SOFTPHONESETTING_MACCLIENT, "MacClient");
		}
	};

	private final ActionListener setPrimaryPhoneIdPhoneIdActionListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			doSetPrimaryPhoneId();
		}
	};

	private final ActionListener getDisplayNumberIdActionListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			doGetDisplayNumberId();
		}
	};

	private final ActionListener setDisplayNumberIdActionListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			doSetDisplayNumberId();
		}
	};

	private final ActionListener getCallWaitingIndicationActionListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			doGetCallWaitingIndication();
		}
	};

	private final ActionListener setCallWaitingIndicationActionListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			doSetCallWaitingIndication();
		}
	};

	private final ActionListener getGetAvailableDisplayNumbersActionListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			doGetAvailableDisplayNumbers();
		}
	};

	private final ActionListener transferSelectedDisplayNumberActionListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			doTransferSelectedDisplayNumber();
		}
	};

	/// View Data
	////////////////////////////////////////////////////////////////

	private static final String PRIMARY_PHONE_PLACEHOLDER = "|-- Primary Phone --|";

	private static final String NO_DISPLAY_PLACEHOLDER = "|-- No Display --|";

	private List<PhoneIdViewModel> phoneIds;

	{
		updatePhoneIds(new ArrayList<>());
	}

	private String primaryPhoneId = "";

	private PhoneIdViewModel selectedPhoneId = new PhoneIdViewModel();

	private List<PhoneNumberViewModel> displayNumbers;

	private PhoneNumberViewModel selectedDisplayNumber;

	private PhoneNumberViewModel displayNumber;

	private boolean callWaitingIndication;

	private final PropertyChangeSupport changeSupport = new PropertyChangeSupport(this);

	public void addPropertyChangeListener(PropertyChangeListener listener) {
		changeSupport.addPropertyChangeListener(listener);
	}

	public void removePropertyChangeListener(PropertyChangeListener listener) {
		changeSupport.removePropertyChangeListener(listener);
	}

	/// Property Getters and Setters
	////////////////////////////////////////////////////////////////

	public PhoneIdViewModel getSelectedPhoneId() {
		return selectedPhoneId;
	}

	public static final String PROPERTY_SELECTED_PHONE_ID = "selectedPhoneId";

	public void setSelectedPhoneId(PhoneIdViewModel selectedPhoneId) {
		PhoneIdViewModel old = this.selectedPhoneId;
		this.selectedPhoneId = selectedPhoneId;
		if (changeSupport != null) {
			changeSupport.firePropertyChange(PROPERTY_SELECTED_PHONE_ID, old,
				selectedPhoneId);
		}
	}

	public List<PhoneIdViewModel> getPhoneIds() {
		return phoneIds;
	}

	public static final String PROPERTY_PHONE_IDS = "phoneIds";

	public void updatePhoneIds(List<String> phoneIds) {
		List<PhoneIdViewModel> old = this.phoneIds;
		this.phoneIds = new LinkedList<>();
		PhoneIdViewModel nullValue = new PhoneIdViewModel();
		this.phoneIds.add(nullValue);
		for (String phoneId : phoneIds) {
			this.phoneIds.add(new PhoneIdViewModel(phoneId));
		}
		if (changeSupport != null) {
			changeSupport.firePropertyChange(PROPERTY_PHONE_IDS, old, this.phoneIds);
		}
		setSelectedPhoneId(nullValue);
	}

	public ActionListener getGetPhoneIdsActionListener() {
		return getPhoneIdsActionListener;
	}

	public List<PhoneNumberViewModel> getDisplayNumbers() {
		return displayNumbers;
	}

	public static final String PROPERTY_DISPLAY_NUMBERS = "displayNumbers";

	public void updateDisplayNumbers(List<PhoneNumber> displayNumbers) {
		List<PhoneNumberViewModel> old = this.displayNumbers;
		this.displayNumbers = new LinkedList<>();
		PhoneNumberViewModel nullValue = new PhoneNumberViewModel();
		this.displayNumbers.add(nullValue);
		for (PhoneNumber displayNumber : displayNumbers) {
			this.displayNumbers.add(new PhoneNumberViewModel(displayNumber));
		}
		if (changeSupport != null) {
			changeSupport.firePropertyChange(PROPERTY_DISPLAY_NUMBERS, old, displayNumbers);
		}
	}

	public PhoneNumberViewModel getSelectedDisplayNumber() {
		return selectedDisplayNumber;
	}

	public static final String PROPERTY_SELECTED_DISPLAY_NUMBER = "selectedDisplayNumber";

	public void setSelectedDisplayNumber(PhoneNumberViewModel selectedDisplayNumber) {
		PhoneNumberViewModel old = this.selectedDisplayNumber;
		this.selectedDisplayNumber = selectedDisplayNumber;
		if (changeSupport != null) {
			changeSupport.firePropertyChange(PROPERTY_SELECTED_DISPLAY_NUMBER, old,
				selectedDisplayNumber);
		}
	}

	@Override
	public void fmcPhonesChangedEvent(List<FmcPhone> phones) {
		//empty
	}

	public static class PhoneIdViewModel {
		private final String phoneId;

		public PhoneIdViewModel(String phoneId) {
			this.phoneId = phoneId;
		}

		public PhoneIdViewModel() {
			this.phoneId = "";
		}

		public String toString() {
			if ("".equals(phoneId)) {
				return PRIMARY_PHONE_PLACEHOLDER;
			} else {
				return phoneId;
			}
		}

		public String getPhoneId() {
			return phoneId;
		}
	}

	public static class PhoneNumberViewModel {
		private final PhoneNumber phoneNumber;

		public PhoneNumberViewModel(PhoneNumber phoneNumber) {
			this.phoneNumber = phoneNumber;
		}

		public PhoneNumberViewModel() {
			phoneNumber = null;
		}

		public String toString() {
			if (phoneNumber == null) {
				return NO_DISPLAY_PLACEHOLDER;
			} else {
				return phoneNumber.getNumber();
			}
		}

		public PhoneNumber getPhoneNumber() {
			return phoneNumber;
		}
	}

	public String getPrimaryPhoneId() {
		return primaryPhoneId;
	}

	public static final String PROPERTY_PRIMARY_PHONE_ID = "primaryPhoneId";

	public void setPrimaryPhoneId(String primaryPhoneId) {
		String old = this.primaryPhoneId;
		this.primaryPhoneId = primaryPhoneId;
		changeSupport.firePropertyChange(PROPERTY_PRIMARY_PHONE_ID, old, primaryPhoneId);
	}

	public ActionListener getTransferSelectedPhoneIdActionListener() {
		return transferSelectedPhoneIdActionListener;
	}

	public ActionListener getGetPrimaryPhoneIdActionListener() {
		return getPrimaryPhoneIdActionListener;
	}

	public ActionListener getSetPrimaryPhoneIdPhoneIdActionListener() {
		return setPrimaryPhoneIdPhoneIdActionListener;
	}

	public boolean isCallWaitingIndication() {
		return callWaitingIndication;
	}

	public static final String PROPERTY_CALL_WAITING_INDICATION = "callWaitingIndication";

	public void setCallWaitingIndication(boolean callWaitingIndication) {
		boolean old = this.callWaitingIndication;
		this.callWaitingIndication = callWaitingIndication;
		changeSupport.firePropertyChange(PROPERTY_CALL_WAITING_INDICATION, old, callWaitingIndication);
	}

	public ActionListener getGetCallWaitingIndicationActionListener() {
		return getCallWaitingIndicationActionListener;
	}

	public ActionListener getSetCallWaitingIndicationActionListener() {
		return setCallWaitingIndicationActionListener;
	}

	public ActionListener getGetGetAvailableDisplayNumbersActionListener() {
		return getGetAvailableDisplayNumbersActionListener;
	}

	public PhoneNumberViewModel getDisplayNumber() {
		return displayNumber;
	}

	public static final String PROPERTY_DISPLAY_NUMBER = "displayNumber";

	public void setDisplayNumber(PhoneNumberViewModel displayNumber) {
		PhoneNumberViewModel old = this.displayNumber;
		this.displayNumber = displayNumber;
		changeSupport.firePropertyChange(PROPERTY_DISPLAY_NUMBER, old, displayNumber);
	}

	public ActionListener getTransferSelectedDisplayNumberActionListener() {
		return transferSelectedDisplayNumberActionListener;
	}

	public ActionListener getSetDisplayNumberIdActionListener() {
		return setDisplayNumberIdActionListener;
	}

	public ActionListener getGetDisplayNumberIdActionListener() {
		return getDisplayNumberIdActionListener;
	}

	/**
	 * @return Returns the registerMacSoftPhoneActionListener.
	 */
	public ActionListener getRegisterMacSoftPhoneActionListener() {
		return registerMacSoftPhoneActionListener;
	}

	/**
	 * @return Returns the registerWinSoftPhoneActionListener.
	 */
	public ActionListener getRegisterWinSoftPhoneActionListener() {
		return registerWinSoftPhoneActionListener;
	}

	/**
	 * @return Returns the isWinSoftPhoneInUse.
	 */
	public ActionListener getIsWinSoftPhoneInUse() {
		return isWinSoftPhoneInUseActionListener;
	}

	/**
	 * @return Returns the isWacSoftPhoneInUse.
	 */
	public ActionListener getIsWacSoftPhoneInUse() {
		return isWacSoftPhoneInUseActionListener;
	}

	@Override
	public void newSoftPhoneRegisteredEvent(SoftPhoneRegistrationInfo softPhoneRegistrationInfo) {
		generalViewTasks.displayInfo(
			"newSoftPhoneRegisteredEvent",
			"Another SoftClient has just been registered for SipAccount "
				+ softPhoneRegistrationInfo.getSipUserId());
	}

	@Override
	public void phonesChangedEvent(List<String> phones) {
		updatePhoneIds(phones);
	}

	@Override
	public void primaryPhoneChangedEvent(String phone) {
		System.out.println("Primary phone is now: " + phone);
	}
}
