/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.examples.uci30.swing.viewmodel;

import de.starface.integration.uci.java.v30.types.QueueAgentState;
import de.starface.integration.uci.java.v30.values.QueueAgentStateProperties;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

public class QueueAgentTableModel implements TableModel {
	private static final QueueAgentStateProperties[] queueAgentColumnOrder;
	private int currentlySelectedGroupAccountId = 0;

	static {
		queueAgentColumnOrder = new QueueAgentStateProperties[8];
		queueAgentColumnOrder[0] = QueueAgentStateProperties.name;
		queueAgentColumnOrder[1] = QueueAgentStateProperties.accountId;
		queueAgentColumnOrder[2] = QueueAgentStateProperties.loggedIn;
		queueAgentColumnOrder[3] = QueueAgentStateProperties.chatPresence;
		queueAgentColumnOrder[4] = QueueAgentStateProperties.chatPresenceMessage;
		queueAgentColumnOrder[5] = QueueAgentStateProperties.doNotDisturbSetting;
		queueAgentColumnOrder[6] = QueueAgentStateProperties.redirectAlwaysActive;
		queueAgentColumnOrder[7] = QueueAgentStateProperties.telephonyState;
	}

	private List<TableModelListener> listeners = new ArrayList<>(1);
	private List<QueueAgentState> agentStates = new ArrayList<>();

	@Override
	public int getRowCount() {
		return agentStates.size();
	}

	@Override
	public int getColumnCount() {
		return queueAgentColumnOrder.length;
	}

	@Override
	public String getColumnName(int columnIndex) {
		return queueAgentColumnOrder[columnIndex].name();
	}

	@Override
	public Class<?> getColumnClass(int columnIndex) {
		switch (columnIndex) {
			case 2:
				return String.class;
			default:
				return queueAgentColumnOrder[columnIndex].getType();
		}
	}

	@Override
	public boolean isCellEditable(int rowIndex, int columnIndex) {
		return false;
	}

	@Override
	public Object getValueAt(int rowIndex, int columnIndex) {
		switch (columnIndex) {
			default:
				return Utils.getProperty(queueAgentColumnOrder[columnIndex], agentStates.get(rowIndex));
		}

	}

	@Override
	public void setValueAt(Object value, int rowIndex, int columnIndex) {
		switch (columnIndex) {
			case 3:
				Utils.setProperty(queueAgentColumnOrder[columnIndex], agentStates.get(rowIndex), value);
			default:
		}

	}

	@Override
	public void addTableModelListener(TableModelListener l) {
		synchronized (listeners) {
			listeners.add(l);
		}

	}

	@Override
	public void removeTableModelListener(TableModelListener l) {
		synchronized (listeners) {
			listeners.remove(l);
		}

	}

	public void setQueueAgents(List<QueueAgentState> agentStates, int queueAccountId) {
		this.currentlySelectedGroupAccountId = queueAccountId;
		this.agentStates = agentStates;
		fireTableChanged(new TableModelEvent(this));
	}

	private void fireTableChanged(TableModelEvent e) {
		synchronized (listeners) {
			for (TableModelListener l : listeners) {
				l.tableChanged(e);
			}
		}
	}

	public void updateAgentState(QueueAgentState agentState) {
		Iterator<QueueAgentState> states = agentStates.iterator();
		QueueAgentState current = null;
		boolean found = false;
		while (states.hasNext()) {
			current = states.next();
			if (current.getAccountId() == agentState.getAccountId()
				&& current.getQueueAccountId() == agentState.getQueueAccountId()) {
				states.remove();
				found = true;
				break;

			}
		}
		if (found) {
			agentStates.add(agentState);
			fireTableChanged(new TableModelEvent(this));
		}
	}

	public void init() {

	}

	/**
	 * @return Returns the currentlyDisplayedGroupAccountId.
	 */
	public int getCurrentlyDisplayedGroupAccountId() {
		return currentlySelectedGroupAccountId;
	}

	/**
	 * @param currentlyDisplayedGroupAccountId
	 *        The currentlyDisplayedGroupAccountId to set.
	 */
	public void setCurrentlyDisplayedGroupAccountId(int currentlyDisplayedGroupAccountId) {
		this.currentlySelectedGroupAccountId = currentlyDisplayedGroupAccountId;
	}

	public String getAgentName(int accountId) {
		for (QueueAgentState agent : agentStates) {
			if (agent.getAccountId() == accountId) {
				return agent.getName();
			}
		}
		return null;
	}

}
