/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.examples.uci30.swing.viewmodel;

import de.starface.integration.uci.java.v30.types.QueueSetting;
import de.starface.integration.uci.java.v30.values.QueueSettingProperties;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

/**
 * A swing table model for the redirect settings
 *
 * @author Mojo
 */
public class QueueSettingsTableModel implements TableModel {
	private static final QueueSettingProperties[] queueColumnOrder;

	static {
		queueColumnOrder = new QueueSettingProperties[5];
		queueColumnOrder[0] = QueueSettingProperties.name;
		queueColumnOrder[1] = QueueSettingProperties.id;
		queueColumnOrder[2] = QueueSettingProperties.accountId;
		queueColumnOrder[3] = QueueSettingProperties.maxWaitingTime;
		queueColumnOrder[4] = QueueSettingProperties.size;
	}

	private List<TableModelListener> listeners = new ArrayList<>(1);

	private List<QueueSetting> queueSettings = new ArrayList<>();

	@Override
	public void addTableModelListener(TableModelListener l) {
		synchronized (listeners) {
			listeners.add(l);
		}
	}

	@Override
	public Class<?> getColumnClass(int columnIndex) {
		switch (columnIndex) {
			case 2:
				return String.class;
			default:
				return queueColumnOrder[columnIndex].getType();
		}
	}

	@Override
	public int getColumnCount() {
		return queueColumnOrder.length;
	}

	@Override
	public String getColumnName(int columnIndex) {
		return queueColumnOrder[columnIndex].name();
	}

	@Override
	public int getRowCount() {
		return queueSettings.size();
	}

	@Override
	public Object getValueAt(int rowIndex, int columnIndex) {
		switch (columnIndex) {
			default:
				return Utils.getProperty(queueColumnOrder[columnIndex], queueSettings.get(rowIndex));
		}
	}

	@Override
	public boolean isCellEditable(int rowIndex, int columnIndex) {
		switch (columnIndex) {
			case 3:
				return true;
			default:
				return false;
		}
	}

	public List<QueueSetting> getQueueSettings() {
		return queueSettings;
	}

	public void setQueueSettings(List<QueueSetting> QueueSettings) {
		this.queueSettings = QueueSettings;
		fireTableChanged(new TableModelEvent(this));
	}

	public void updateQueueSetting(Integer queueAccountId, QueueSetting queueSetting) {
		Iterator<QueueSetting> it = queueSettings.iterator();
		while (it.hasNext()) {
			QueueSetting oldSetting = it.next();
			if (oldSetting.getAccountId() == queueAccountId) {
				it.remove();
				break;
			}
		}
		if (queueSetting != null) {
			this.queueSettings.add(queueSetting);
		}
		fireTableChanged(new TableModelEvent(this));
	}

	public QueueSetting getQueueSetting(int rowIndex) {
		return queueSettings.get(rowIndex);
	}

	@Override
	public void removeTableModelListener(TableModelListener l) {
		synchronized (listeners) {
			listeners.remove(l);
		}
	}

	@Override
	public void setValueAt(Object value, int rowIndex, int columnIndex) {
		switch (columnIndex) {
			case 3:
				Utils.setProperty(queueColumnOrder[columnIndex], queueSettings.get(rowIndex), value);
			default:
		}
	}

	private void fireTableChanged(TableModelEvent e) {
		synchronized (listeners) {
			for (TableModelListener l : listeners) {
				l.tableChanged(e);
			}
		}
	}

	public String getQueueName(int accountId) {
		for (QueueSetting qs : queueSettings) {
			if (qs.getAccountId() == accountId) {
				return qs.getName();
			}
		}
		return null;
	}

}
