/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.examples.uci30.swing.viewmodel;

import de.starface.integration.uci.java.v30.exceptions.UciException;
import de.starface.integration.uci.java.v30.messages.events.UciQueueEvents;
import de.starface.integration.uci.java.v30.messages.requests.UciQueueRequests;
import de.starface.integration.uci.java.v30.types.QueueAgentState;
import de.starface.integration.uci.java.v30.types.QueueSetting;
import de.starface.integration.uci.java.v30.types.QueueStatistics;
import de.starface.integration.uci.java.v30.types.UciQueueCall;
import de.starface.integration.uci.v30.client.UciServiceRequests;
import de.starface.integration.uci.v30.client.utils.EventsProvider;
import de.starface.integration.uci.v30.client.utils.EventsProviderSupport;

import org.mortbay.log.Log;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.Collections;
import java.util.List;

import javax.swing.table.TableModel;

public class QueueViewModel implements UciQueueEvents, EventsProvider<UciQueueEvents> {

	private final QueueSettingsTableModel queueSettingsTableModel = new QueueSettingsTableModel();
	private final QueueAgentTableModel queueAgentTableModel = new QueueAgentTableModel();
	private final QueueCallTableModel queueCallTableModel = new QueueCallTableModel(this);

	private PropertyChangeSupport changeSupport = new PropertyChangeSupport(this);

	private int selectedRowQueue = -1;

	private int selectedRowCaller = -1;

	public int getSelectedRowQueue() {
		return selectedRowQueue;
	}

	public static final String PROPERTY_SELECTED_ROW_CALLER = "selectedRowCaller";

	public static final String PROPERTY_SELECTED_ROW_QUEUE = "selectedRowQueue";

	public static final String PROPERTY_CALLERS_IN_QUEUE_ID = "callersInQueue";

	private String callersInQueue = "Callers: 0";

	public String getCallersInQueue() {
		return callersInQueue;
	}

	public void setCallersInQueue(String callersInQueue) {
		this.callersInQueue = callersInQueue;
	}

	public static final String PROPERTY_FREE_AGENTS_IN_QUEUE_ID = "freeAgentsInQueue";
	private String freeAgentsInQueue = "Free Agents: 0";

	public String getFreeAgentsInQueue() {
		return freeAgentsInQueue;
	}

	public void setFreeAgentsInQueue(String freeAgentsInQueue) {
		this.freeAgentsInQueue = freeAgentsInQueue;
	}

	public static final String PROPERTY_CALL_LISTS_QUEUE_ID = "callListsQueue";
	private String callListsQueue = "Calls: 0/0";

	public String getCallListsQueue() {
		return callListsQueue;
	}

	public void setCallListsQueue(String callListsQueue) {
		this.callListsQueue = callListsQueue;
	}

	public static final String PROPERTY_WAITTIME_QUEUE_ID = "waittimeQueue";
	private String waittimeQueue = "Waittime: 0s";

	public String getWaittimeQueue() {
		return waittimeQueue;
	}

	public void setWaittimeQueue(String waittimeQueue) {
		this.waittimeQueue = waittimeQueue;
	}

	public static final String PROPERTY_CALLERS_TODAY_QUEUE_ID = "callersTodayQueue";
	private String callersTodayQueue = "Callers: 0";

	public String getCallersTodayQueue() {
		return callersTodayQueue;
	}

	public void setCallersTodayQueue(String callersTodayQueue) {
		this.callersTodayQueue = callersTodayQueue;
	}

	public int getSelectedRowCaller() {
		return selectedRowCaller;
	}

	public void setSelectedRowCaller(int selectedRow) {
		this.selectedRowCaller = selectedRow;
		System.err.println("ROW " + selectedRow);
	}

	public void setSelectedRowQueue(int selectedRow) {
		int old = this.selectedRowQueue;
		this.selectedRowQueue = selectedRow;
		changeSupport.firePropertyChange(PROPERTY_SELECTED_ROW_CALLER, old, selectedRow);
		if (selectedRow != -1) {
			Integer queueId = (Integer) queueSettingsTableModel.getValueAt(selectedRow, 2);
			this.queueAgentTableModel.setCurrentlyDisplayedGroupAccountId(queueId);
			this.doGetQueueAgentsSettings();
			this.queueCallTableModel.setSelectedQueueId(queueId);
		} else {
			this.queueAgentTableModel.setCurrentlyDisplayedGroupAccountId(-1);
			this.queueCallTableModel.setSelectedQueueId(-1);
		}
		try {
			queueStatisticsEvent(
				queueRequests.getQueueStatistics(queueAgentTableModel.getCurrentlyDisplayedGroupAccountId()));
		} catch (UciException e) {
			Log.info("Error ", e);
		}
	}

	/// Domain Logic Access
	////////////////////////////////////////////////////////////////

	public QueueViewModel() {
	}

	private EventsProviderSupport<UciQueueEvents> eventsSupport = new EventsProviderSupport<>(
		UciQueueEvents.class);

	private UciQueueRequests queueRequests;

	public void setUciServiceRequests(UciServiceRequests uciServiceRequests) throws UciException {
		this.queueRequests = uciServiceRequests.getRequests(UciQueueRequests.class);
		uciServiceRequests.subscribeEventsAsync(this, UciQueueEvents.class);
		this.doGetQueueCalls();
	}

	@Override
	public void queueSettingsChanged(List<QueueSetting> queueSettings) {
		queueSettingsTableModel.setQueueSettings(queueSettings);
		eventsSupport.getEventsDistributor().queueSettingsChanged(queueSettings);
	}

	@Override
	public void queueSettingChanged(Integer queueAccountId) {
		QueueSetting possiblyNull;
		try {
			possiblyNull = this.queueRequests.getQueueSetting(queueAccountId);
		} catch (UciException e) {
			Log.warn("Could not fetch settings for group " + queueAccountId);
			possiblyNull = null;
		}
		queueSettingsTableModel.updateQueueSetting(queueAccountId, possiblyNull);
		eventsSupport.getEventsDistributor().queueSettingChanged(queueAccountId);
	}

	@Override
	public void queueAgentStateChanged(QueueAgentState agentState) {
		queueAgentTableModel.updateAgentState(agentState);
	}

	@Override
	public void subscribeEvents(UciQueueEvents listener) {
		eventsSupport.subscribeEvents(listener);
	}

	@Override
	public void unsubscribeEvents(UciQueueEvents listener) {
		eventsSupport.unsubscribeEvents(listener);
	}

	/// View Actions
	////////////////////////////////////////////////////////////////
	private void doGrabQueueCallerButton() {
		try {
			queueRequests.grabQueueCaller(queueAgentTableModel.getCurrentlyDisplayedGroupAccountId(), "",
					this.queueCallTableModel.getValueAt(selectedRowCaller, 0).toString());
		} catch (UciException e) {
			// Too simple error management. MUST be improved in real client!
			e.printStackTrace();
		}
	}
	private void doGetQueueSettings() {
		try {
			// Retrieve the queue settings sending the getQueueSettings request
			List<QueueSetting> queueSettings = queueRequests.getQueueSettings();
			queueSettingsTableModel.setQueueSettings(queueSettings);
			eventsSupport.getEventsDistributor().queueSettingsChanged(queueSettings);
		} catch (UciException e) {
			// Too simple error management. MUST be improved in real client!
			e.printStackTrace();
		}

	}

	private void doGetQueueCalls() {
		List<UciQueueCall> calls;
		try {
			calls = queueRequests.getQueueCalls();
			queueCallTableModel.init(calls);
		} catch (UciException e) {
			e.printStackTrace();
		}
	}

	private void doGetQueueAgentsSettings() {
		try {
			List<QueueAgentState> agentStates = queueRequests.getQueueAgents(queueAgentTableModel.getCurrentlyDisplayedGroupAccountId());
			queueAgentTableModel.setQueueAgents(agentStates,
				queueAgentTableModel.getCurrentlyDisplayedGroupAccountId());
		} catch (UciException e) {
			// Too simple error management. MUST be improved in real client!
			e.printStackTrace();
		}
	}

	private final ActionListener getQueueSettingsActionListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			doGetQueueSettings();
		}
	};

	private final ActionListener getQueueAgentsActionListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			doGetQueueAgentsSettings();
		}
	};

	private final ActionListener getGetQueueCallsActionListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			doGetQueueCalls();
		}
	};

	private final ActionListener getGrabQueueCallerButton = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			doGrabQueueCallerButton();
		}

	};

	/// View Data
	////////////////////////////////////////////////////////////////

	public void addPropertyChangeListener(PropertyChangeListener listener) {
		changeSupport.addPropertyChangeListener(listener);
	}

	public void removePropertyChangeListener(PropertyChangeListener listener) {
		changeSupport.removePropertyChangeListener(listener);
	}

	/// Property Getters and Setters
	////////////////////////////////////////////////////////////////

	public TableModel getQueueSettingsTableModel() {
		return queueSettingsTableModel;
	}

	public TableModel getQueueAgentTableModel() {
		return queueAgentTableModel;
	}

	public TableModel getQueueCallTableModel() {
		return queueCallTableModel;
	}

	public ActionListener getGetQueueSettingsActionListener() {
		return getQueueSettingsActionListener;
	}

	/**
	 * @return Returns the getQueueAgentsActionListener.
	 */
	public ActionListener getGetQueueAgentsActionListener() {
		return getQueueAgentsActionListener;
	}

	public ActionListener getGetQueueCallsActionListener() {
		return getGetQueueCallsActionListener;
	}

	public ActionListener getGrabQueueCallerButtonListener() {
		return getGrabQueueCallerButton;
	}

	@Override
	public void queueAgentChangedEvent(Integer groupAccountId) {
		if (queueAgentTableModel.getCurrentlyDisplayedGroupAccountId() == groupAccountId) {
			try {
				Log.info("The number of agents in queue " + groupAccountId + " changed.");
				queueAgentTableModel.setQueueAgents(
					queueRequests.getQueueAgents(queueAgentTableModel.getCurrentlyDisplayedGroupAccountId()),
					groupAccountId);
			} catch (UciException e) {
				Log.warn("Could not fetch Agent settings ", e);
			}
		}
		//just in case we got removed or added to a group
		doGetQueueSettings();
		doGetQueueCalls();
	}

	@Override
	public void queueGroupChangedEvent(boolean newGroup, Integer groupAccountId) {
		if (newGroup) {
			Log.info("New Queue (" + groupAccountId + ") was created.");
		} else {
			if (queueAgentTableModel.getCurrentlyDisplayedGroupAccountId() == groupAccountId) {
				queueAgentTableModel.setQueueAgents(Collections.EMPTY_LIST, groupAccountId);
			}
			Log.info("Queue (" + groupAccountId + ") was deleted.");
		}

	}

	@Override
	public void queueCallEvent(UciQueueCall call) {
		queueCallTableModel.updateCall(call);
	}

	@Override
	public void queueCallChangedIdEvent(String oldId, String newId) {
		queueCallTableModel.updateCallId(oldId, newId);

	}

	public String getQueueName(int AccountId) {
		return queueSettingsTableModel.getQueueName(AccountId);
	}

	public String getAgentName(int AccountId) {
		return queueAgentTableModel.getAgentName(AccountId);
	}

	@Override
	public void queueStatisticsEvent(QueueStatistics queueStatistics) {
		Log.info(queueStatistics.toString());
		Integer selectedQueue = -1;
		if (selectedRowQueue >= 0) {
			Object selected = queueSettingsTableModel.getValueAt(selectedRowQueue, 1);
			if (selected != null) {
				selectedQueue = (Integer) selected;
			}
		}

		if (selectedQueue.equals(queueStatistics.getId())) {
			changeSupport.firePropertyChange(PROPERTY_CALLERS_IN_QUEUE_ID, callersInQueue,
				"Callers: " + queueStatistics.getCallersInQueue());
			this.callersInQueue = "Callers: " + queueStatistics.getCallersInQueue();
			changeSupport.firePropertyChange(PROPERTY_FREE_AGENTS_IN_QUEUE_ID, callersInQueue,
				"Free Agents: " + queueStatistics.getFreeAgents());
			this.freeAgentsInQueue = "Free Agents: " + queueStatistics.getFreeAgents();
			changeSupport.firePropertyChange(PROPERTY_CALL_LISTS_QUEUE_ID, callListsQueue,
				"Calls: " + queueStatistics.getMissedCalls() + "/" + queueStatistics.getUnansweredCalls());
			this.callListsQueue =
				"Calls: " + queueStatistics.getMissedCalls() + "/" + queueStatistics.getUnansweredCalls();
			changeSupport.firePropertyChange(PROPERTY_WAITTIME_QUEUE_ID, waittimeQueue,
				"Waittime: " + queueStatistics.getAverageWaitingTime() + "s");
			this.waittimeQueue = "Waittime: " + queueStatistics.getAverageWaitingTime() + "s";
			changeSupport.firePropertyChange(PROPERTY_CALLERS_TODAY_QUEUE_ID, callersTodayQueue,
				"Callers: " + queueStatistics.getTotalCalls());
			this.callersTodayQueue = "Callers: " + queueStatistics.getTotalCalls();
		}
	}

}
