/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.v30.client.transport.logging;

import de.starface.integration.uci.v30.client.UciProxyConfigurationFailedException;
import de.starface.integration.uci.v30.client.transport.UcpTransport;
import de.starface.integration.uci.v30.client.transport.UcpTransportFactory;

import org.apache.commons.lang3.Validate;

import java.util.Map;

/**
 * This class is an implementation of {@link UcpTransportFactory} that can be used for development
 * or testing purposes. It creates {@link UcpTransport} instances that log all method calls to a
 * commons logging log.
 *
 * @author tilmann
 */
public class LoggingDummyUcpTransportFactory extends UcpTransportFactory {
	/**
	 * Creates a new {@link LoggingDummyUcpTransport} for the given login ID.
	 *
	 * @param loginId
	 *        the login ID of the STARFACE user
	 * @param password
	 *        the password of the STARFACE user
	 * @return an instance of {@link LoggingDummyUcpTransport} that logs calls to a commons logging
	 *         log.
	 * @throws IllegalArgumentException
	 *         if null or empty loginId given
	 */
	@Override
	public LoggingDummyUcpTransport createUcpTransport(String loginId, String password) {
		Validate.notEmpty(loginId, "loginId=empty");

		return new LoggingDummyUcpTransport(loginId);
	}

	/**
	 * This {@link Configurator} implementation is used to create and configure
	 * {@link LoggingDummyUcpTransportFactory} instances from configuration properties.
	 *
	 * There are no properties to set for this Configurator.
	 *
	 * @author tilmann
	 *
	 */
	public static class Configurator implements UcpTransportFactory.Configurator {
		/**
		 * Creates a new {@link LoggingDummyUcpTransportFactory} instance.
		 *
		 * @param configuration
		 *        the configuration properties to be used
		 * @return a newly created {@link LoggingDummyUcpTransportFactory} instance
		 * @throws UciProxyConfigurationFailedException
		 *         if a problem occurs while creating or configuring the new instance
		 */
		@Override
		public LoggingDummyUcpTransportFactory configureFromMap(Map<String, String> configuration)
			throws UciProxyConfigurationFailedException {
			return new LoggingDummyUcpTransportFactory();
		}
	}
}
