/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.v30.client.utils;

import java.lang.reflect.Method;

/**
 * An exception handler that can be given to {@link EventsProviderSupport} to handle exceptions that
 * occur during events distribution.
 *
 * @author tilmann
 * @param <EventsInterface>
 *        the type of the event listener
 *
 */
public interface EventsProviderExceptionHandler<EventsInterface> {
	/**
	 * Handles an exception that has occurred during invocation of the given listener.
	 *
	 * @param listener
	 *        the listener that caused the exception
	 * @param method
	 *        the method of the listener that was invoked
	 * @param args
	 *        the parameters of the invocation
	 * @param e
	 *        the thrown exception
	 */
	public void handleException(EventsInterface listener, Method method, Object[] args, Exception e);
}
