/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.java.v30.messages.events;

import de.starface.com.rpc.annotation.RpcAsynchronous;
import de.starface.com.rpc.annotation.RpcInterface;
import de.starface.com.rpc.annotation.RpcValueTranslation;
import de.starface.integration.uci.java.v30.UciService;
import de.starface.integration.uci.java.v30.UciServices;
import de.starface.integration.uci.java.v30.messages.requests.UciCallRequests;
import de.starface.integration.uci.java.v30.types.Call;
import de.starface.integration.uci.java.v30.types.CallIdChanges;
import de.starface.integration.uci.java.v30.types.MessageWaitingIndicationInfo;
import de.starface.integration.uci.java.v30.ucp.messages.events.UcpCallEvents;
import de.starface.integration.uci.java.v30.values.CallProperties;

/**
 * This interface defines the call events that can be received by a UCI client implementation in
 * order to handle phone calls.
 *
 * @author tilmann
 */
@UciService(UcpCallEvents.SERVICE_NAME)
@RpcInterface(UcpCallEvents.SERVICE_NAME)
@RpcValueTranslation(version = UciServices.UCI_VERSION)
public interface UciCallEvents {

	/**
	 * The ReceiveCallState event provides a number of properties about the call the state is
	 * changing of. The information provided in the map is defined by its properties as seen in
	 * {@link CallProperties}. If a name or number is not known to STARFACE an empty string is
	 * committed for that property. It is possible that information about the caller or the called
	 * person gets available in the middle of a series of events so that the first call state events
	 * don’t provide any details but later events contain full participant information. The
	 * sequential change of the {@link CallProperties#state} property adheres to the state diagram
	 * given in the UCI specification. It is assumed after a successful return of the
	 * {@link UciCallRequests#placeCallWithPhone(String, String, String)} request.
	 *
	 * @param callProperties
	 *        the different properties of the call.
	 */
	@RpcAsynchronous
	public void receiveCallState(Call callProperties);

	/**
	 * This event provides Information about new VoicemailMessages to the UCI client
	 *
	 * @param messageWaitingIndicationInfo
	 *        the new messageWaitingIndication {@link MessageWaitingIndicationInfo}
	 */
	@RpcAsynchronous
	public void newMessageWaitingIndicationEvent(MessageWaitingIndicationInfo messageWaitingIndicationInfo);

	/**
	 * This event provides Information about the new callId of a call to the UCI client
	 *
	 * @param callIdChangesEvent
	 *        the callChangesIdEvent {@link CallIdChanges}
	 */
	@RpcAsynchronous
	public void callChangesIdEvent(CallIdChanges callIdChangesEvent);
}
