/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.java.v30.messages.events;

import de.starface.com.rpc.annotation.RpcAsynchronous;
import de.starface.com.rpc.annotation.RpcInterface;
import de.starface.com.rpc.annotation.RpcValueTranslation;
import de.starface.integration.uci.java.v30.UciService;
import de.starface.integration.uci.java.v30.UciServices;
import de.starface.integration.uci.java.v30.types.CallListEntry;
import de.starface.integration.uci.java.v30.ucp.messages.events.UcpCallListEvents;

import java.util.List;
import java.util.Map;

/**
 * This interface defines the call list events that can be received by a UCI client implementation
 * in order to handle call list entries.
 * 
 * @author dwendland
 */
@RpcInterface(UcpCallListEvents.SERVICE_NAME)
@UciService(UcpCallListEvents.SERVICE_NAME)
@RpcValueTranslation(version = UciServices.UCI_VERSION)
public interface UciCallListEvents {
	/**
	 * This event allows the UCI client to get informed about new call list entries being available.
	 * 
	 * @param callListEntry the new call list entry with a {@link Map} containing the values defined
	 *        in {@link CallListEntry}
	 */
	@RpcAsynchronous
	public void newCallListEntry(CallListEntry callListEntry);

	/**
	 * This event allows the UCI client to get informed about changed call list entries being available.
	 *
	 * @param callListEntry the updated call list entry with a {@link Map} containing the values defined
	 *        in {@link CallListEntry}
	 */
	@RpcAsynchronous
	public void updatedCallListEntry(CallListEntry callListEntry);
	
	/**
	 * This event allows the UCI client to get informed about deleted call list entries.
	 *
	 * @param deletedCallListEntryIds
	 *        a {@link List} containing the ids of deleted call list entries
	 */
	@RpcAsynchronous
	public void deletedCallListEntries(List<String> deletedCallListEntryIds);
}
