/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.java.v30.messages.events;

import de.starface.com.rpc.annotation.RpcAsynchronous;
import de.starface.com.rpc.annotation.RpcInterface;
import de.starface.com.rpc.annotation.RpcValueTranslation;
import de.starface.integration.uci.java.v30.UciService;
import de.starface.integration.uci.java.v30.UciServices;
import de.starface.integration.uci.java.v30.types.CdrDataInfo;
import de.starface.integration.uci.java.v30.ucp.messages.events.UcpCallCdrEvents;
import de.starface.integration.uci.java.v30.values.CdrUpdateType;

/**
 * This interface defines the cdr call events that can be received by a UCI client implementation in
 * order to handle cdrdata.
 *
 * @author dirk
 */
@UciService(UcpCallCdrEvents.SERVICE_NAME)
@RpcInterface(UcpCallCdrEvents.SERVICE_NAME)
@RpcValueTranslation(version = UciServices.UCI_VERSION)
public interface UciCdrDataEvents {
	/**
	 * This event provides Information about new CdrData to the UCI client
	 *
	 * @param cdrDataInfo
	 *        the newCallCdrEvent {@link CdrDataInfo}
	 */
	@RpcAsynchronous
	void callCdrEvent(CdrDataInfo cdrDataInfo);

	/**
	 * This event notifies about changes in existing CDR summary entries.
	 *
	 * @param cdrUpdateType The type of change
	 * @param cdrDataInfo The contents of the updated record
	 */
	@RpcAsynchronous
	void callCdrUpdatedEvent(CdrUpdateType cdrUpdateType, CdrDataInfo cdrDataInfo);
}
