/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.java.v30.messages.events;

import de.starface.com.rpc.annotation.RpcAsynchronous;
import de.starface.com.rpc.annotation.RpcInterface;
import de.starface.com.rpc.annotation.RpcValueTranslation;
import de.starface.integration.uci.java.v30.UciService;
import de.starface.integration.uci.java.v30.UciServices;
import de.starface.integration.uci.java.v30.types.Fax;
import de.starface.integration.uci.java.v30.ucp.messages.events.UcpFaxEvents;
import de.starface.integration.uci.java.v30.values.FaxProperties;

/**
 * This interface defines communication fax events that can be received by a UCI client
 * implementation in order to handle outgoing faxes.
 *
 * @author tilmann
 *
 */
@UciService(UcpFaxEvents.SERVICE_NAME)
@RpcInterface(UcpFaxEvents.SERVICE_NAME)
@RpcValueTranslation(version = UciServices.UCI_VERSION)
public interface UciFaxEvents {
	/**
	 * Much the same as ReceiveCallState the ReceiveFaxState event notifies the UCI client of state
	 * changes to an outgoing fax request by providing several properties about the fax request
	 * including the id of the fax request, the new state of the fax request and the timestamp when
	 * the request did change its state. The state Waiting is never signaled by the server. It is
	 * assumed after a successful call to SendFax.
	 *
	 * @param faxProperties
	 *        the different properties of the fax in a map. The constants of {@link FaxProperties}
	 *        are used as keys in the map.
	 */
	@RpcAsynchronous
	public void receiveFaxState(Fax faxProperties);
}
