/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.java.v30.messages.events;

import de.starface.com.rpc.annotation.RpcAsynchronous;
import de.starface.com.rpc.annotation.RpcInterface;
import de.starface.com.rpc.annotation.RpcValueTranslation;
import de.starface.integration.uci.java.v30.UciService;
import de.starface.integration.uci.java.v30.UciServices;
import de.starface.integration.uci.java.v30.types.FunctionKey;
import de.starface.integration.uci.java.v30.ucp.messages.events.UcpFunctionKeyEvents;

import java.util.List;

/**
 * This interface defines the function key events that can be received by the UCI client
 * implementation in order to handle function key changes entries.
 *
 * @author tilmann
 */
@UciService(UcpFunctionKeyEvents.SERVICE_NAME)
@RpcInterface(UcpFunctionKeyEvents.SERVICE_NAME)
@RpcValueTranslation(version = UciServices.UCI_VERSION)
public interface UciFunctionKeyEvents {

	/**
	 * The ReceiveFunctionKeyState event provides a number of properties about the function key the
	 * state is changing of.
	 *
	 * @param functionKeyProperties
	 *        the different properties of the function key
	 */
	@RpcAsynchronous
	public void receiveFunctionKeyState(FunctionKey functionKeyProperties);

	/**
	 * This event allows the UCI client to get informed when the configuration of the function keys
	 * of the user changed.
	 *
	 * @param functionKeys
	 *        the new function keys the user has
	 */
	@RpcAsynchronous
	public void functionKeysChanged(List<FunctionKey> functionKeys);

	/**
	 * This event allows the UCI client to get informed when the function key searching is
	 * terminated.
	 *
	 * @param searchId
	 *        the id of the terminated search
	 */
	@RpcAsynchronous
	public void functionKeySearchTerminates(String searchId);
}
