/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.java.v30.messages.events;

import de.starface.com.rpc.annotation.RpcAsynchronous;
import de.starface.com.rpc.annotation.RpcInterface;
import de.starface.com.rpc.annotation.RpcValueTranslation;
import de.starface.integration.uci.java.v30.UciService;
import de.starface.integration.uci.java.v30.UciServices;
import de.starface.integration.uci.java.v30.types.GroupSetting;
import de.starface.integration.uci.java.v30.ucp.messages.events.UcpGroupEvents;

import java.util.List;

/**
 * This interface defines the group events that can be received by a UCI client implementation in
 * order to handle group setting changes.
 *
 * @author tilmann
 */
@UciService(UcpGroupEvents.SERVICE_NAME)
@RpcInterface(UcpGroupEvents.SERVICE_NAME)
@RpcValueTranslation(version = UciServices.UCI_VERSION)
public interface UciGroupEvents {

	/**
	 * This event allows the UCI client to get informed when the state or number of the group
	 * settings change.
	 *
	 * @param groupSettings
	 *        the new state of the group setting
	 */
	@RpcAsynchronous
	public void groupSettingsChanged(List<GroupSetting> groupSettings);

}
