/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.java.v30.messages.events;

import de.starface.com.rpc.annotation.RpcAsynchronous;
import de.starface.com.rpc.annotation.RpcInterface;
import de.starface.com.rpc.annotation.RpcValueTranslation;
import de.starface.integration.uci.java.v30.UciService;
import de.starface.integration.uci.java.v30.UciServices;
import de.starface.integration.uci.java.v30.types.RedirectSetting;
import de.starface.integration.uci.java.v30.ucp.messages.events.UcpRedirectEvents;

import java.util.List;

/**
 * This interface defines the redirect events that can be received by a UCI client implementation in
 * order to handle redirect setting changes.
 *
 * @author tilmann
 */
@UciService(UcpRedirectEvents.SERVICE_NAME)
@RpcInterface(UcpRedirectEvents.SERVICE_NAME)
@RpcValueTranslation(version = UciServices.UCI_VERSION)
public interface UciRedirectEvents {

	/**
	 * This event allows the UCI client to get informed when the state or number of the redirect
	 * settings change.
	 *
	 * @param redirectSettings
	 *        the new state of the redirect setting
	 */
	@RpcAsynchronous
	public void redirectSettingsChanged(List<RedirectSetting> redirectSettings);

}
