/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.java.v30.messages.events;

import de.starface.com.rpc.annotation.RpcAsynchronous;
import de.starface.com.rpc.annotation.RpcInterface;
import de.starface.com.rpc.annotation.RpcValueTranslation;
import de.starface.integration.uci.java.v30.UciService;
import de.starface.integration.uci.java.v30.UciServices;
import de.starface.integration.uci.java.v30.types.VoicemailListEntry;
import de.starface.integration.uci.java.v30.types.VoicemailListEntryMovedData;
import de.starface.integration.uci.java.v30.ucp.messages.events.UcpVoicemailListEvents;

import java.util.List;
import java.util.Map;

/**
 * This interface defines voicemail list events that can be received by the UCI client
 * implementation in order to handle voicemail list entries.
 *
 * @author tilmann
 */
@RpcInterface(UcpVoicemailListEvents.SERVICE_NAME)
@UciService(UcpVoicemailListEvents.SERVICE_NAME)
@RpcValueTranslation(version = UciServices.UCI_VERSION)
public interface UciVoicemailListEvents {

	/**
	 * This event allows the UCI client to get informed about new voicemail list entries being
	 * available.
	 *
	 * @param voicemailListEntry
	 *        the new voicemail list entry with a {@link Map} containing the values defined in
	 *        {@link VoicemailListEntry}
	 */
	@RpcAsynchronous
	public void newVoicemailListEntry(VoicemailListEntry voicemailListEntry);
	
	/**
	 * This event allows the UCI client to get informed about deleted voicemail list entries.
	 *
	 * @param deletedVoicemailListEntryIds
	 *        a {@link List} containing the ids of deleted voicemail list entries
	 */
	@RpcAsynchronous
	public void deletedVoicemailListEntries(List<String> deletedVoicemailListEntryIds);
	
	/**
	 * This event allows the UCI client to get informed about moved voicemail list entries.
	 *
	 * @param voicemailListEntryMovedDatas
	 *        a {@link List} of {@link VoicemailListEntryMovedData}
	 */
	@RpcAsynchronous
	public void movedVoicemailListEntries(List<VoicemailListEntryMovedData> voicemailListEntryMovedDatas);
}
