/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.java.v30.messages.requests;

import de.starface.com.rpc.annotation.RpcInterface;
import de.starface.com.rpc.annotation.RpcValueTranslation;
import de.starface.integration.uci.java.v30.UciService;
import de.starface.integration.uci.java.v30.UciServices;
import de.starface.integration.uci.java.v30.exceptions.UciException;
import de.starface.integration.uci.java.v30.types.GlobalLineConfiguration;
import de.starface.integration.uci.java.v30.types.WebServerConfiguration;
import de.starface.integration.uci.java.v30.ucp.messages.requests.UcpSystemRequests;

import java.util.List;

/**
 *
 */
@UciService(UcpSystemRequests.SERVICE_NAME)
@RpcInterface(UcpSystemRequests.SERVICE_NAME)
@RpcValueTranslation(version = UciServices.UCI_VERSION)
public interface UciSystemRequests {
	/**
	 * Liefert den Name der Rechtegruppe
	 *
	 * @param parent
	 *        Name der Rechtegruppe. Leeren String für alle Rechte. Darf nicht null sein.
	 * @return Liste mit allen freigeschalteten Rechten (aus der Gruppe)
	 * @throws UciException
	 */
	List<String> getPermissions(String parent) throws UciException;

	/**
	 * Fragt den Server, eine bestimmte lizenzierbare Funktionalitaet fuer den angemeldeten benutzer
	 * freizuschalten. Wenn serverseitig ausreichende Lizenzen vorhanden sind, dann wird die
	 * Funktion/das Recht automatisch für den Benutzer aktiviert, ohne dass ein Admin im
	 * Adminbereich es explizit freischalten muss.
	 *
	 * @param permission
	 *        Rechtename, darf nicht null sein
	 * @return true wenn Funktionalitaet/Recht fuer den Benutzer jetzt freigeschaltet ist
	 * @throws UciException
	 */
	boolean requestPermission(String permission) throws UciException;

	GlobalLineConfiguration getGlobalLineConfiguration() throws UciException;

	/**
	 * Liefert die Versionsnummer des verbundenen Servers
	 *
	 * @return Version des Servers
	 * @throws UciException
	 */
	public String getServerVersion() throws UciException;

	/**
	 * Liefert die Webserverkonfiguration des verbundenen Servers
	 * @return Webserverkonfiguration
	 * @throws UciException
	 */
	public WebServerConfiguration getWebServerConfiguration() throws UciException;

	/**
	 * Retrieves the server timezone id as used by java.util.TimeZone. See the IANA time zone database for further
	 * information. All Date-Objects have to be adjusted to match this timezone since timezone information are not 
	 * supported by xml-rpc.
	 * For example "Europe/Berlin".
	 * @return Id of the servers TimeZone
	 * @throws UciException
	 */
	public String getTimeZoneId() throws UciException;
}
