/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.java.v30.types;

import de.starface.com.rpc.annotation.RpcValue;
import de.starface.com.rpc.annotation.RpcValueObject;
import de.starface.integration.uci.java.v30.messages.events.UciCcbsCallEvents;
import de.starface.integration.uci.java.v30.values.CcbsCallState;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

/**
 * This object holds the properties of the ccbscall as returned by
 * {@link UciCcbsCallEvents#receiveCcbsCallState(CcbsCall)}.
 *
 * @author tilmann
 */
@RpcValueObject
public class CcbsCall implements Serializable {
	private static final long serialVersionUID = 1L;

	/** The id of the call in form of a UUID as {@link String} */
	@RpcValue
	protected String id;

	/** The state of the call */
	@RpcValue
	protected CcbsCallState ccbsCallState;

	/** The point of time when the call reached the state */
	@RpcValue
	protected Date timestamp;

	/** The phone number that has been called */
	@RpcValue
	protected String calledNumber = "";

	/** The name of the person being called */
	@RpcValue
	protected String calledName = "";

	/** The number of the caller */
	@RpcValue
	protected String callerNumber = "";

	/** The name of the person calling */
	@RpcValue
	protected String callerName = "";

	/**
	 * The id of the group if this call was received by a group (the empty string otherwise)
	 */
	@RpcValue(minVersion = 30)
	protected String groupId = "";

	/** Name of the device **/
	@RpcValue(minVersion = 30)
	protected List<String> peerNames;

	/** The channel name **/
	@RpcValue(minVersion = 30)
	protected List<String> channelNames;

	/** Is caller **/
	@RpcValue(minVersion = 30)
	protected Boolean caller = Boolean.FALSE;

	/** The time between now and the timestamp in milliseconds **/
	@RpcValue(minVersion = 30)
	protected long duration = 0;

	public String getId() {
		return id;
	}

	public void setId(String id) {
		this.id = id;
	}

	public CcbsCallState getCcbsCallState() {
		return ccbsCallState;
	}

	public void setCcbsCallState(CcbsCallState state) {
		this.ccbsCallState = state;
	}

	public Date getTimestamp() {
		return timestamp;
	}

	public void setTimestamp(Date timestamp) {
		this.timestamp = timestamp;
	}

	public String getCalledNumber() {
		return calledNumber;
	}

	public void setCalledNumber(String calledNumber) {
		this.calledNumber = calledNumber;
	}

	public String getCalledName() {
		return calledName;
	}

	public void setCalledName(String calledName) {
		this.calledName = calledName;
	}

	public String getCallerNumber() {
		return callerNumber;
	}

	public void setCallerNumber(String callerNumber) {
		this.callerNumber = callerNumber;
	}

	public String getCallerName() {
		return callerName;
	}

	public void setCallerName(String callerName) {
		this.callerName = callerName;
	}

	public List<String> getChannelNames() {
		return channelNames;
	}

	public void setChannelNames(List<String> channelNames) {
		this.channelNames = channelNames;
	}

	public String getGroupId() {
		return groupId;
	}

	public void setGroupId(String groupId) {
		this.groupId = groupId;
	}

	public List<String> getPeerNames() {
		return peerNames;
	}

	public void setPeerNames(List<String> peerNames) {
		this.peerNames = peerNames;
	}

	public Boolean isCaller() {
		return caller;
	}

	public void setCaller(Boolean caller) {
		this.caller = caller;
	}

	public long getDuration() {
		return duration;
	}

	public void setDuration(long duration) {
		this.duration = duration;
	}

	public boolean isGroupCall() {
		return groupId != null && !"".equals(groupId);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return "CcbsCall [id=" + id + ", ccbsCallState=" + ccbsCallState + ", timestamp=" + timestamp
			+ ", calledNumber=" + calledNumber + ", calledName=" + calledName + ", callerNumber=" + callerNumber
			+ ", callerName=" + callerName + ", groupId=" + groupId + ", peerNames=" + peerNames
			+ ", channelName=" + channelNames + ", caller=" + caller + ", duration=" + duration + "]";
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((calledName == null) ? 0 : calledName.hashCode());
		result = prime * result + ((calledNumber == null) ? 0 : calledNumber.hashCode());
		result = prime * result + ((caller == null) ? 0 : caller.hashCode());
		result = prime * result + ((callerName == null) ? 0 : callerName.hashCode());
		result = prime * result + ((callerNumber == null) ? 0 : callerNumber.hashCode());
		result = prime * result + ((ccbsCallState == null) ? 0 : ccbsCallState.hashCode());
		result = prime * result + ((channelNames == null) ? 0 : channelNames.hashCode());
		result = prime * result + (int) (duration ^ (duration >>> 32));
		result = prime * result + ((groupId == null) ? 0 : groupId.hashCode());
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		result = prime * result + ((peerNames == null) ? 0 : peerNames.hashCode());
		result = prime * result + ((timestamp == null) ? 0 : timestamp.hashCode());
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		CcbsCall other = (CcbsCall) obj;
		if (calledName == null) {
			if (other.calledName != null) {
				return false;
			}
		} else if (!calledName.equals(other.calledName)) {
			return false;
		}
		if (calledNumber == null) {
			if (other.calledNumber != null) {
				return false;
			}
		} else if (!calledNumber.equals(other.calledNumber)) {
			return false;
		}
		if (caller == null) {
			if (other.caller != null) {
				return false;
			}
		} else if (!caller.equals(other.caller)) {
			return false;
		}
		if (callerName == null) {
			if (other.callerName != null) {
				return false;
			}
		} else if (!callerName.equals(other.callerName)) {
			return false;
		}
		if (callerNumber == null) {
			if (other.callerNumber != null) {
				return false;
			}
		} else if (!callerNumber.equals(other.callerNumber)) {
			return false;
		}
		if (ccbsCallState != other.ccbsCallState) {
			return false;
		}
		if (channelNames == null) {
			if (other.channelNames != null) {
				return false;
			}
		} else if (!channelNames.equals(other.channelNames)) {
			return false;
		}
		if (duration != other.duration) {
			return false;
		}
		if (groupId == null) {
			if (other.groupId != null) {
				return false;
			}
		} else if (!groupId.equals(other.groupId)) {
			return false;
		}
		if (id == null) {
			if (other.id != null) {
				return false;
			}
		} else if (!id.equals(other.id)) {
			return false;
		}
		if (peerNames == null) {
			if (other.peerNames != null) {
				return false;
			}
		} else if (!peerNames.equals(other.peerNames)) {
			return false;
		}
		if (timestamp == null) {
			if (other.timestamp != null) {
				return false;
			}
		} else if (!timestamp.equals(other.timestamp)) {
			return false;
		}
		return true;
	}
}
