/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.java.v30.types;

import de.starface.com.rpc.annotation.RpcValue;
import de.starface.com.rpc.annotation.RpcValueObject;
import de.starface.integration.uci.java.v30.messages.events.UciCdrDataEvents;

import java.io.Serializable;
import java.util.Objects;

/**
 * This object holds the properties of the cdr as returned by
 * {@link UciCdrDataEvents#callCdrEvent(CdrDataInfo)}.
 *
 * @author tilmann
 */
@RpcValueObject
public class CdrDataInfo implements Serializable {
	private static final long serialVersionUID = 1L;

	@RpcValue
	private Long id;
	@RpcValue
	private Integer callStepId;
	@RpcValue
	private String callLegUUID;
	@RpcValue
	private Integer cdrAccountId;
	@RpcValue
	private Integer callerAccountId;
	@RpcValue
	private String callerCallerId;
	@RpcValue
	private Integer calledAccountId;
	@RpcValue
	private String calledCallerId;
	@RpcValue
	private Integer serviceId;
	@RpcValue
	private Long startTime;
	@RpcValue
	private Long ringingTime;
	@RpcValue
	private Long linkTime;
	@RpcValue
	private Long callResultTime;
	@RpcValue
	private String cdrCallResult;
	@RpcValue
	private Integer callResultCausedBy;
	@RpcValue
	private String answeredBy;
	@RpcValue
	private Integer lineId;
	@RpcValue
	private String lineName;
	@RpcValue
	private String callbackNumber;
	@RpcValue
	private String answeredElsewhere;
	@RpcValue
	private Boolean incoming;
	@RpcValue
	private Boolean answered;
	@RpcValue
	private Boolean voicemailAvailable;
	@RpcValue
	private Boolean monitorAvailable;
	@RpcValue
	private Boolean faxAvailable;
	@RpcValue
	private Boolean deleted;
	@RpcValue
	private Boolean privateCall;
	@RpcValue
	private String comment;
	@RpcValue
	private String commentAuthor;
	@RpcValue
	private Long commentModified;
	@RpcValue
	private Boolean calledBack;
	@RpcValue
	private String calledBackAuthor;
	@RpcValue
	private Long calledBackModified;

	/**
	 * @return the callStepId
	 */
	public Integer getCallStepId() {
		return callStepId;
	}

	/**
	 * @param callStepId
	 *        the callStepId to set
	 */
	public void setCallStepId(Integer callStepId) {
		this.callStepId = callStepId;
	}

	/**
	 * @return the callLegUUID
	 */
	public String getCallLegUUID() {
		return callLegUUID;
	}

	/**
	 * @param callLegUUID
	 *        the callLegUUID to set
	 */
	public void setCallLegUUID(String callLegUUID) {
		this.callLegUUID = callLegUUID;
	}

	/**
	 * @return the cdrAccountId
	 */
	public Integer getCdrAccountId() {
		return cdrAccountId;
	}

	/**
	 * @param cdrAccountId
	 *        the cdrAccountId to set
	 */
	public void setCdrAccountId(Integer cdrAccountId) {
		this.cdrAccountId = cdrAccountId;
	}

	/**
	 * @return the callerAccountId
	 */
	public Integer getCallerAccountId() {
		return callerAccountId;
	}

	/**
	 * @param callerAccountId
	 *        the callerAccountId to set
	 */
	public void setCallerAccountId(Integer callerAccountId) {
		this.callerAccountId = callerAccountId;
	}

	/**
	 * @return the callerCallerId
	 */
	public String getCallerCallerId() {
		return callerCallerId;
	}

	/**
	 * @param callerCallerId
	 *        the callerCallerId to set
	 */
	public void setCallerCallerId(String callerCallerId) {
		this.callerCallerId = callerCallerId;
	}

	/**
	 * @return the calledAccountId
	 */
	public Integer getCalledAccountId() {
		return calledAccountId;
	}

	/**
	 * @param calledAccountId
	 *        the calledAccountId to set
	 */
	public void setCalledAccountId(Integer calledAccountId) {
		this.calledAccountId = calledAccountId;
	}

	/**
	 * @return the calledCallerId
	 */
	public String getCalledCallerId() {
		return calledCallerId;
	}

	/**
	 * @param calledCallerId
	 *        the calledCallerId to set
	 */
	public void setCalledCallerId(String calledCallerId) {
		this.calledCallerId = calledCallerId;
	}

	/**
	 * @return the serviceId
	 */
	public Integer getServiceId() {
		return serviceId;
	}

	/**
	 * @param serviceId
	 *        the serviceId to set
	 */
	public void setServiceId(Integer serviceId) {
		this.serviceId = serviceId;
	}

	/**
	 * @return the startTime
	 */
	public Long getStartTime() {
		return startTime;
	}

	/**
	 * @param startTime
	 *        the startTime to set
	 */
	public void setStartTime(Long startTime) {
		this.startTime = startTime;
	}

	/**
	 * @return the ringingTime
	 */
	public Long getRingingTime() {
		return ringingTime;
	}

	/**
	 * @param ringingTime
	 *        the ringingTime to set
	 */
	public void setRingingTime(Long ringingTime) {
		this.ringingTime = ringingTime;
	}

	/**
	 * @return the linkTime
	 */
	public Long getLinkTime() {
		return linkTime;
	}

	/**
	 * @param linkTime
	 *        the linkTime to set
	 */
	public void setLinkTime(Long linkTime) {
		this.linkTime = linkTime;
	}

	/**
	 * @return the callResultTime
	 */
	public Long getCallResultTime() {
		return callResultTime;
	}

	/**
	 * @param callResultTime
	 *        the callResultTime to set
	 */
	public void setCallResultTime(Long callResultTime) {
		this.callResultTime = callResultTime;
	}

	/**
	 * @return the cdrCallResult
	 */
	public String getCdrCallResult() {
		return cdrCallResult;
	}

	/**
	 * @param cdrCallResult
	 *        the cdrCallResult to set
	 */
	public void setCdrCallResult(String cdrCallResult) {
		this.cdrCallResult = cdrCallResult;
	}

	/**
	 * @return the callResultCausedBy
	 */
	public Integer getCallResultCausedBy() {
		return callResultCausedBy;
	}

	/**
	 * @param callResultCausedBy
	 *        the callResultCausedBy to set
	 */
	public void setCallResultCausedBy(Integer callResultCausedBy) {
		this.callResultCausedBy = callResultCausedBy;
	}

	/**
	 * @return the lineId
	 */
	public Integer getLineId() {
		return lineId;
	}

	/**
	 * @param lineId
	 *        the lineId to set
	 */
	public void setLineId(Integer lineId) {
		this.lineId = lineId;
	}

	/**
	 * @return the lineName
	 */
	public String getLineName() {
		return lineName;
	}

	/**
	 * @param lineName
	 *        the lineName to set
	 */
	public void setLineName(String lineName) {
		this.lineName = lineName;
	}

	/**
	 * @return the callbackNumber
	 */
	public String getCallbackNumber() {
		return callbackNumber;
	}

	/**
	 * @param callbackNumber
	 *        the callbackNumber to set
	 */
	public void setCallbackNumber(String callbackNumber) {
		this.callbackNumber = callbackNumber;
	}

	/**
	 * @return the answeredElsewhere
	 */
	public String getAnsweredElsewhere() {
		return answeredElsewhere;
	}

	/**
	 * @param answeredElsewhere
	 *        the answeredElsewhere to set
	 */
	public void setAnsweredElsewhere(String answeredElsewhere) {
		this.answeredElsewhere = answeredElsewhere;
	}

	/**
	 * @return the incoming
	 */
	public Boolean isIncoming() {
		return incoming;
	}

	/**
	 * @param incoming
	 *        the incoming to set
	 */
	public void setIncoming(Boolean incoming) {
		this.incoming = incoming;
	}

	/**
	 * @return the answered
	 */
	public Boolean isAnswered() {
		return answered;
	}

	/**
	 * @param answered
	 *        the answered to set
	 */
	public void setAnswered(Boolean answered) {
		this.answered = answered;
	}

	/**
	 * @return the voicemailAvailable
	 */
	public Boolean isVoicemailAvailable() {
		return voicemailAvailable;
	}

	/**
	 * @param voicemailAvailable
	 *        the voicemailAvailable to set
	 */
	public void setVoicemailAvailable(Boolean voicemailAvailable) {
		this.voicemailAvailable = voicemailAvailable;
	}

	/**
	 * @return the monitorAvailable
	 */
	public Boolean isMonitorAvailable() {
		return monitorAvailable;
	}

	/**
	 * @param monitorAvailable
	 *        the monitorAvailable to set
	 */
	public void setMonitorAvailable(Boolean monitorAvailable) {
		this.monitorAvailable = monitorAvailable;
	}

	/**
	 * @return the faxAvailable
	 */
	public Boolean isFaxAvailable() {
		return faxAvailable;
	}

	/**
	 * @param faxAvailable
	 *        the faxAvailable to set
	 */
	public void setFaxAvailable(Boolean faxAvailable) {
		this.faxAvailable = faxAvailable;
	}

	/**
	 * @return the deleted
	 */
	public Boolean isDeleted() {
		return deleted;
	}

	/**
	 * @param deleted
	 *        the deleted to set
	 */
	public void setDeleted(Boolean deleted) {
		this.deleted = deleted;
	}

	/**
	 * @return the privateCall
	 */
	public Boolean isPrivateCall() {
		return privateCall;
	}

	/**
	 * @param privateCall
	 *        the privateCall to set
	 */
	public void setPrivateCall(Boolean privateCall) {
		this.privateCall = privateCall;
	}

	/**
	 * @return the id
	 */
	public Long getId() {
		return id;
	}

	/**
	 * @param id
	 *        the id to set
	 */
	public void setId(Long id) {
		this.id = id;
	}


	public String getAnsweredBy() {
		return answeredBy;
	}

	public void setAnsweredBy(String answeredBy) {
		this.answeredBy = answeredBy;
	}
	public String getComment() {
		return comment;
	}

	public void setComment(String comment) {
		this.comment = comment;
	}

	public String getCommentAuthor() {
		return commentAuthor;
	}

	public void setCommentAuthor(String commentAuthor) {
		this.commentAuthor = commentAuthor;
	}

	public Long getCommentModified() {
		return commentModified;
	}

	public void setCommentModified(Long commentModified) {
		this.commentModified = commentModified;
	}

	public Boolean getCalledBack() {
		return calledBack;
	}

	public void setCalledBack(Boolean calledBack) {
		this.calledBack = calledBack;
	}

	public String getCalledBackAuthor() {
		return calledBackAuthor;
	}

	public void setCalledBackAuthor(String calledBackAuthor) {
		this.calledBackAuthor = calledBackAuthor;
	}

	public Long getCalledBackModified() {
		return calledBackModified;
	}

	public void setCalledBackModified(Long calledBackModified) {
		this.calledBackModified = calledBackModified;
	}

	@Override
	public boolean equals(Object o) {
		if (this == o) {
			return true;
		}
		if (o == null || getClass() != o.getClass()) {
			return false;
		}
		CdrDataInfo that = (CdrDataInfo) o;
		return Objects.equals(id, that.id) && Objects.equals(callStepId, that.callStepId) &&
				Objects.equals(callLegUUID, that.callLegUUID) &&
				Objects.equals(cdrAccountId, that.cdrAccountId) &&
				Objects.equals(callerAccountId, that.callerAccountId) &&
				Objects.equals(callerCallerId, that.callerCallerId) &&
				Objects.equals(calledAccountId, that.calledAccountId) &&
				Objects.equals(calledCallerId, that.calledCallerId) &&
				Objects.equals(serviceId, that.serviceId) &&
				Objects.equals(startTime, that.startTime) &&
				Objects.equals(ringingTime, that.ringingTime) &&
				Objects.equals(linkTime, that.linkTime) &&
				Objects.equals(callResultTime, that.callResultTime) &&
				Objects.equals(cdrCallResult, that.cdrCallResult) &&
				Objects.equals(callResultCausedBy, that.callResultCausedBy) &&
				Objects.equals(answeredBy, that.answeredBy) && Objects.equals(lineId, that.lineId) &&
				Objects.equals(lineName, that.lineName) &&
				Objects.equals(callbackNumber, that.callbackNumber) &&
				Objects.equals(answeredElsewhere, that.answeredElsewhere) &&
				Objects.equals(incoming, that.incoming) && Objects.equals(answered, that.answered) &&
				Objects.equals(voicemailAvailable, that.voicemailAvailable) &&
				Objects.equals(monitorAvailable, that.monitorAvailable) &&
				Objects.equals(faxAvailable, that.faxAvailable) &&
				Objects.equals(deleted, that.deleted) &&
				Objects.equals(privateCall, that.privateCall) &&
				Objects.equals(comment, that.comment) &&
				Objects.equals(commentAuthor, that.commentAuthor) &&
				Objects.equals(commentModified, that.commentModified) &&
				Objects.equals(calledBack, that.calledBack) &&
				Objects.equals(calledBackAuthor, that.calledBackAuthor) &&
				Objects.equals(calledBackModified, that.calledBackModified);
	}

	@Override
	public int hashCode() {
		return Objects.hash(id, callStepId, callLegUUID, cdrAccountId, callerAccountId, callerCallerId, calledAccountId,
				calledCallerId, serviceId, startTime, ringingTime, linkTime, callResultTime, cdrCallResult,
				callResultCausedBy, answeredBy, lineId, lineName, callbackNumber, answeredElsewhere, incoming, answered,
				voicemailAvailable, monitorAvailable, faxAvailable, deleted, privateCall, comment, commentAuthor,
				commentModified, calledBack, calledBackAuthor, calledBackModified);
	}

	@Override
	public String toString() {
		return "CdrDataInfo{" +
				"id=" + id +
				", callStepId=" + callStepId +
				", callLegUUID='" + callLegUUID + '\'' +
				", cdrAccountId=" + cdrAccountId +
				", callerAccountId=" + callerAccountId +
				", callerCallerId='" + callerCallerId + '\'' +
				", calledAccountId=" + calledAccountId +
				", calledCallerId='" + calledCallerId + '\'' +
				", serviceId=" + serviceId +
				", startTime=" + startTime +
				", ringingTime=" + ringingTime +
				", linkTime=" + linkTime +
				", callResultTime=" + callResultTime +
				", cdrCallResult='" + cdrCallResult + '\'' +
				", callResultCausedBy=" + callResultCausedBy +
				", answeredBy='" + answeredBy + '\'' +
				", lineId=" + lineId +
				", lineName='" + lineName + '\'' +
				", callbackNumber='" + callbackNumber + '\'' +
				", answeredElsewhere='" + answeredElsewhere + '\'' +
				", incoming=" + incoming +
				", answered=" + answered +
				", voicemailAvailable=" + voicemailAvailable +
				", monitorAvailable=" + monitorAvailable +
				", faxAvailable=" + faxAvailable +
				", deleted=" + deleted +
				", privateCall=" + privateCall +
				", comment='" + comment + '\'' +
				", commentAuthor='" + commentAuthor + '\'' +
				", commentModified=" + commentModified +
				", calledBack=" + calledBack +
				", calledBackAuthor='" + calledBackAuthor + '\'' +
				", calledBackModified=" + calledBackModified +
				'}';
	}
}
