/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.java.v30.types;

import de.starface.com.rpc.annotation.RpcValue;
import de.starface.com.rpc.annotation.RpcValueObject;
import de.starface.integration.uci.java.v30.messages.events.UciConferenceCallEvents;
import de.starface.integration.uci.java.v30.values.CallState;

import java.io.Serializable;
import java.util.Date;

/**
 * This object holds the properties of the call as returned by
 * {@link UciConferenceCallEvents#receiveConferenceCallState(ConferenceCall)}.
 *
 * @author dirk
 */
@RpcValueObject
public class ConferenceCall implements Serializable {
	private static final long serialVersionUID = 1L;

	/** The id of the call in form of a UUID as {@link String} */
	@RpcValue
	protected String id;

	/** The phone number of the conference participant */
	@RpcValue
	protected String number = "";

	/** The name of the conference participant */
	@RpcValue
	protected String name = "";

	/** Can the participant hear */
	@RpcValue
	protected Boolean listening = true;

	/** Can the participant speak */
	@RpcValue
	protected Boolean muted = false;

	/** Is the participant speaking */
	@RpcValue
	protected Boolean talking = false;

	/** Want the participant speak */
	@RpcValue
	protected Integer wantToSpeak = 0;

	/** Is the participant admin */
	@RpcValue
	protected Boolean admin = false;

	/** The state of the call */
	@RpcValue
	protected CallState state;

	/** The Room Identifikation Number */
	@RpcValue
	protected String roomId;

	/** Entered Room */
	@RpcValue
	protected Date enteredTimestamp;

	/** The time between now and the enteredTimestamp in milliseconds **/
	@RpcValue(minVersion = 30)
	protected long duration = 0;

	/** The jabber id **/
	@RpcValue(minVersion = 30)
	protected String jabberId = "";

	/** The avatar hash **/
	@RpcValue(minVersion = 30)
	protected String avatarHash = "";

	/** The account id */
	@RpcValue(minVersion = 30)
	protected String accountId = "";

	public String getId() {
		return id;
	}

	public void setId(String id) {
		this.id = id;
	}

	/**
	 * @return the number
	 */
	public String getNumber() {
		return number;
	}

	/**
	 * @param number
	 *        the number to set
	 */
	public void setNumber(String number) {
		this.number = number;
	}

	/**
	 * @return the name
	 */
	public String getName() {
		return name;
	}

	/**
	 * @param name
	 *        the name to set
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * @return the wantToSpeak
	 */
	public Integer getWantToSpeak() {
		return wantToSpeak;
	}

	/**
	 * @param wantToSpeak
	 *        the wantToSpeak to set
	 */
	public void setWantToSpeak(Integer wantToSpeak) {
		this.wantToSpeak = wantToSpeak;
	}

	/**
	 * @return the admin
	 */
	public Boolean isAdmin() {
		return admin;
	}

	/**
	 * @param admin
	 *        the admin to set
	 */
	public void setAdmin(Boolean admin) {
		this.admin = admin;
	}

	/**
	 * @return the state
	 */
	public CallState getState() {
		return state;
	}

	/**
	 * @param state
	 *        the state to set
	 */
	public void setState(CallState state) {
		this.state = state;
	}

	/**
	 * @return the listening
	 */
	public Boolean isListening() {
		return listening;
	}

	/**
	 * @param listen
	 *        the listen to set
	 */
	public void setListening(Boolean listen) {
		this.listening = listen;
	}

	/**
	 * @return the mute
	 */
	public Boolean isMuted() {
		return muted;
	}

	/**
	 * @param muted
	 *        the mute to set
	 */
	public void setMuted(Boolean muted) {
		this.muted = muted;
	}

	/**
	 * @return the talking
	 */
	public Boolean isTalking() {
		return talking;
	}

	/**
	 * @param talking
	 *        the talking to set
	 */
	public void setTalking(Boolean talking) {
		this.talking = talking;
	}

	/**
	 * @return the roomId
	 */
	public String getRoomId() {
		return roomId;
	}

	/**
	 * @param roomId
	 *        the roomId to set
	 */
	public void setRoomId(String roomId) {
		this.roomId = roomId;
	}

	/**
	 * @return the enteredRoom
	 */
	public Date getEnteredTimestamp() {
		return enteredTimestamp;
	}

	/**
	 * @param enteredTimestamp
	 *        the enteredTimestamp to set
	 */
	public void setEnteredTimestamp(Date enteredTimestamp) {
		this.enteredTimestamp = enteredTimestamp;
	}

	/**
	 * @return the duration
	 */
	public long getDuration() {
		return duration;
	}

	/**
	 * @param duration
	 *        the duration to set
	 */
	public void setDuration(long duration) {
		this.duration = duration;
	}

	/**
	 * @return the jabberId
	 */
	public String getJabberId() {
		return jabberId;
	}

	/**
	 * @param jabberId
	 *        the jabberId to set
	 */
	public void setJabberId(String jabberId) {
		this.jabberId = jabberId;
	}

	/**
	 * @return the avatarHash
	 */
	public String getAvatarHash() {
		return avatarHash;
	}

	/**
	 * @param avatarHash
	 *        the avatarHash to set
	 */
	public void setAvatarHash(String avatarHash) {
		this.avatarHash = avatarHash;
	}

	@Override
	public boolean equals(final Object o) {
		if (this == o) {
			return true;
		}
		if (!(o instanceof ConferenceCall)) {
			return false;
		}

		final ConferenceCall that = (ConferenceCall) o;

		if (duration != that.duration) {
			return false;
		}
		if (id != null ? !id.equals(that.id) : that.id != null) {
			return false;
		}
		if (number != null ? !number.equals(that.number) : that.number != null) {
			return false;
		}
		if (name != null ? !name.equals(that.name) : that.name != null) {
			return false;
		}
		if (listening != null ? !listening.equals(that.listening) : that.listening != null) {
			return false;
		}
		if (muted != null ? !muted.equals(that.muted) : that.muted != null) {
			return false;
		}
		if (talking != null ? !talking.equals(that.talking) : that.talking != null) {
			return false;
		}
		if (wantToSpeak != null ? !wantToSpeak.equals(that.wantToSpeak) : that.wantToSpeak != null) {
			return false;
		}
		if (admin != null ? !admin.equals(that.admin) : that.admin != null) {
			return false;
		}
		if (state != that.state) {
			return false;
		}
		if (roomId != null ? !roomId.equals(that.roomId) : that.roomId != null) {
			return false;
		}
		if (enteredTimestamp != null ? !enteredTimestamp.equals(that.enteredTimestamp) : that.enteredTimestamp != null) {
			return false;
		}
		if (jabberId != null ? !jabberId.equals(that.jabberId) : that.jabberId != null) {
			return false;
		}
		if (avatarHash != null ? !avatarHash.equals(that.avatarHash) : that.avatarHash != null) {
			return false;
		}
		return accountId != null ? accountId.equals(that.accountId) : that.accountId == null;

	}

	@Override
	public int hashCode() {
		int result = id != null ? id.hashCode() : 0;
		result = 31 * result + (number != null ? number.hashCode() : 0);
		result = 31 * result + (name != null ? name.hashCode() : 0);
		result = 31 * result + (listening != null ? listening.hashCode() : 0);
		result = 31 * result + (muted != null ? muted.hashCode() : 0);
		result = 31 * result + (talking != null ? talking.hashCode() : 0);
		result = 31 * result + (wantToSpeak != null ? wantToSpeak.hashCode() : 0);
		result = 31 * result + (admin != null ? admin.hashCode() : 0);
		result = 31 * result + (state != null ? state.hashCode() : 0);
		result = 31 * result + (roomId != null ? roomId.hashCode() : 0);
		result = 31 * result + (enteredTimestamp != null ? enteredTimestamp.hashCode() : 0);
		result = 31 * result + (int) (duration ^ (duration >>> 32));
		result = 31 * result + (jabberId != null ? jabberId.hashCode() : 0);
		result = 31 * result + (avatarHash != null ? avatarHash.hashCode() : 0);
		result = 31 * result + (accountId != null ? accountId.hashCode() : 0);
		return result;
	}

	@Override
	public String toString() {
		final StringBuilder sb = new StringBuilder("ConferenceCall [");
		sb.append("id=").append(id);
		sb.append(", number=").append(number);
		sb.append(", name=").append(name);
		sb.append(", listening=").append(listening);
		sb.append(", muted=").append(muted);
		sb.append(", talking=").append(talking);
		sb.append(", wantToSpeak=").append(wantToSpeak);
		sb.append(", admin=").append(admin);
		sb.append(", state=").append(state);
		sb.append(", roomId=").append(roomId);
		sb.append(", enteredTimestamp=").append(enteredTimestamp);
		sb.append(", duration=").append(duration);
		sb.append(", jabberId=").append(jabberId);
		sb.append(", avatarHash=").append(avatarHash);
		sb.append(", accountId=").append(accountId);
		sb.append(']');
		return sb.toString();
	}

	public String getAccountId() {
		return accountId;
	}

	public void setAccountId(final String accountId) {
		this.accountId = accountId;
	}
}
