/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.java.v30.types;

import de.starface.com.rpc.annotation.RpcValue;
import de.starface.com.rpc.annotation.RpcValueObject;

import java.io.Serializable;
import java.util.Map;

@RpcValueObject
public class FunctionKeySearchResult implements Serializable {

	/**
	 *
	 */
	private static final long serialVersionUID = 1L;

	/** The id of this searchResult. It's used to make explicit requests on server for this search. */
	@RpcValue
	private String searchId;

	/** The map of function key ids and {@link FunctionKey} objects matching the search term. */
	@RpcValue
	private Map<String, FunctionKey> resultList;

	/** The id of this searchResult. Its used to make explicit requests on server for this search. */
	@RpcValue
	private String searchTerm;

	public FunctionKeySearchResult(Map<String, FunctionKey> resultList, String searchTerm, String searchId) {
		this.searchId = searchId;
		this.resultList = resultList;
		this.searchTerm = searchTerm;
	}

	/** Do not use! For serialization only! */
	public FunctionKeySearchResult() {
	}

	/**
	 * @return the searchId
	 */
	public String getSearchId() {
		return searchId;
	}

	/**
	 * @param searchId
	 *        the searchId to set
	 */
	public void setSearchId(String searchId) {
		this.searchId = searchId;
	}

	/**
	 * @return the resultList
	 */
	public Map<String, FunctionKey> getResultList() {
		return resultList;
	}

	/**
	 * @param resultList
	 *        the resultList to set
	 */
	public void setResultList(Map<String, FunctionKey> resultList) {
		this.resultList = resultList;
	}

	/**
	 * @return the searchTerm
	 */
	public String getSearchTerm() {
		return searchTerm;
	}

	/**
	 * @param searchTerm
	 *        the searchTerm to set
	 */
	public void setSearchTerm(String searchTerm) {
		this.searchTerm = searchTerm;
	}

	@Override
	public String toString() {
		String searchResult = "Search result: uuid =" + searchId + ", search term = " + searchTerm
			+ "\n List of FunctionKeys:\n";

		for (FunctionKey fk : resultList.values()) {
			searchResult = searchResult.concat(fk.toString() + "\n");
		}
		return searchResult;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((searchId == null) ? 0 : searchId.hashCode());
		result = prime * result + ((searchTerm == null) ? 0 : searchTerm.hashCode());
		result = prime * result + ((resultList == null) ? 0 : resultList.hashCode());
		return result;
	}

	/**
	 * Returns true if the parameter is an {@link FunctionKeySearchResult} and the searchIds, search
	 * terms an the resultLists are equals.
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		FunctionKeySearchResult other = (FunctionKeySearchResult) obj;
		if (searchId.equals(other.getSearchId()) && resultList.equals(other.getResultList())
			&& searchTerm.equals(other.getSearchTerm())) {
			return true;
		} else {
			return false;
		}
	}
}
