/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.java.v30.types;

import de.starface.com.rpc.annotation.RpcValue;
import de.starface.com.rpc.annotation.RpcValueObject;

import java.io.Serializable;

@RpcValueObject
public class QueueSetting implements Serializable {
	private static final long serialVersionUID = -4590501213645332276L;
	/**
	 * The ID of this queue for the database
	 */
	@RpcValue(minVersion = 30)
	int id;
	/**
	 * The Account ID of the group behind the queue
	 */
	@RpcValue(minVersion = 30)
	int accountId;
	/**
	 * The size of the queue
	 */
	@RpcValue(minVersion = 30)
	int size;
	/**
	 * The maximum waiting time until a waiting caller becomes disconnected
	 */
	@RpcValue(minVersion = 30)
	int maxWaitingTime;
	/**
	 * The Name of the Queue
	 */
	@RpcValue(minVersion = 30)
	String name;

	public QueueSetting() {
	}

	/**
	 * Returns the unique id of this queue
	 *
	 * @see de.starface.integration.uci.java.v30.types.QueueSetting#getId()
	 */
	public int getId() {
		return id;
	}

	/**
	 * Returns the accountid of this queue(group)
	 *
	 * @see de.starface.integration.uci.java.v30.types.QueueSetting#getAccountId()
	 */
	public int getAccountId() {
		return accountId;
	}

	/**
	 * Returns the maximum amount of callers in the queue
	 *
	 * @see de.starface.integration.uci.java.v30.types.QueueSetting#getSize()
	 */
	public int getSize() {
		return size;
	}

	/**
	 * Returns the maximum waiting time until a caller will get kicked out of the queue
	 *
	 * @see de.starface.integration.uci.java.v30.types.QueueSetting#getMaxWaitingTime()
	 */
	public int getMaxWaitingTime() {
		return maxWaitingTime;
	}

	/**
	 * Returns the Name of the Queue Group
	 *
	 * @see de.starface.integration.uci.java.v30.types.QueueSetting#getName()
	 */
	public String getName() {
		return name;
	}

	/**
	 * Sets the name of the queue
	 *
	 * @param name
	 *        The name to set.
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * @param id
	 *        The id to set.
	 */
	public void setId(int id) {
		this.id = id;
	}

	/**
	 * @param accountId
	 *        The accountId to set.
	 */
	public void setAccountId(int accountId) {
		this.accountId = accountId;
	}

	/**
	 * @param size
	 *        The size to set.
	 */
	public void setSize(int size) {
		this.size = size;
	}

	/**
	 * @param maxWaitingTime
	 *        The maxWaitingTime to set.
	 */
	public void setMaxWaitingTime(int maxWaitingTime) {
		this.maxWaitingTime = maxWaitingTime;
	}

	/**
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + accountId;
		result = prime * result + id;
		result = prime * result + maxWaitingTime;
		result = prime * result + ((name == null) ? 0 : name.hashCode());
		result = prime * result + size;
		return result;
	}

	/**
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		QueueSetting other = (QueueSetting) obj;
		if (accountId != other.accountId) {
			return false;
		}
		if (id != other.id) {
			return false;
		}
		if (maxWaitingTime != other.maxWaitingTime) {
			return false;
		}
		if (name == null) {
			if (other.name != null) {
				return false;
			}
		} else if (!name.equals(other.name)) {
			return false;
		}
		if (size != other.size) {
			return false;
		}
		return true;
	}
}
