/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.java.v30.types;

import de.starface.com.rpc.annotation.RpcValue;
import de.starface.com.rpc.annotation.RpcValueObject;
import de.starface.integration.uci.java.v30.messages.events.UciQueueEvents;

import java.io.Serializable;

/**
 * This object holds the properties of the queuestatistics as returned by
 * {@link UciQueueEvents#queueStatisticsEvent}.
 *
 */
@RpcValueObject
public class QueueStatistics implements Serializable {
	private static final long serialVersionUID = 1L;

	@RpcValue
	protected int accountId;

	@RpcValue
	protected int id;

	@RpcValue
	protected int callersInQueue;

	@RpcValue
	protected int freeAgents;
	
	@RpcValue
	protected int missedCalls;
	
	@RpcValue
	protected int unansweredCalls;

	@RpcValue
	protected int totalCalls;
	
	@RpcValue
	protected int averageWaitingTime;

	public int getAccountId() {
		return accountId;
	}

	public void setAccountId(int accountId) {
		this.accountId = accountId;
	}

	public int getId() {
		return id;
	}

	public void setId(int id) {
		this.id = id;
	}

	public int getCallersInQueue() {
		return callersInQueue;
	}

	public void setCallersInQueue(int callersInQueue) {
		this.callersInQueue = callersInQueue;
	}

	public int getFreeAgents() {
		return freeAgents;
	}

	public void setFreeAgents(int freeAgents) {
		this.freeAgents = freeAgents;
	}

	public int getMissedCalls() {
		return missedCalls;
	}

	public void setMissedCalls(int missedCalls) {
		this.missedCalls = missedCalls;
	}

	public int getTotalCalls() {
		return totalCalls;
	}

	public void setTotalCalls(int totalCalls) {
		this.totalCalls = totalCalls;
	}

	public int getAverageWaitingTime() {
		return averageWaitingTime;
	}

	public void setAverageWaitingTime(int averageWaitingTime) {
		this.averageWaitingTime = averageWaitingTime;
	}

	public int getUnansweredCalls() {
		return unansweredCalls;
	}

	public void setUnansweredCalls(int unansweredCalls) {
		this.unansweredCalls = unansweredCalls;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + accountId;
		result = prime * result + averageWaitingTime;
		result = prime * result + callersInQueue;
		result = prime * result + freeAgents;
		result = prime * result + id;
		result = prime * result + missedCalls;
		result = prime * result + totalCalls;
		result = prime * result + unansweredCalls;
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) return true;
		if (obj == null) return false;
		if (getClass() != obj.getClass()) return false;
		QueueStatistics other = (QueueStatistics) obj;
		if (accountId != other.accountId) return false;
		if (averageWaitingTime != other.averageWaitingTime) return false;
		if (callersInQueue != other.callersInQueue) return false;
		if (freeAgents != other.freeAgents) return false;
		if (id != other.id) return false;
		if (missedCalls != other.missedCalls) return false;
		if (totalCalls != other.totalCalls) return false;
		if (unansweredCalls != other.unansweredCalls) return false;
		return true;
	}

	@Override
	public String toString() {
		return "QueueStatistics [accountId=" + accountId + ", id=" + id + ", callersInQueue=" + callersInQueue +
			", freeAgents=" + freeAgents + ", missedCalls=" + missedCalls + ", unansweredCalls=" + unansweredCalls +
			", totalCalls=" + totalCalls + ", averageWaitingTime=" + averageWaitingTime + "]";
	}
	
}
