/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.java.v30.ucp.messages.events;

import de.starface.com.rpc.annotation.RpcAsynchronous;
import de.starface.com.rpc.annotation.RpcInterface;
import de.starface.integration.uci.java.v30.UciService;
import de.starface.integration.uci.java.v30.types.CdrDataInfo;
import de.starface.integration.uci.java.v30.values.CdrDataProperties;
import de.starface.integration.uci.java.v30.values.CdrUpdateType;

import java.util.Map;

/**
 * This interface defines the call events that can be received by a UCI client implementation in
 * order to handle cdrdata.
 *
 * @author tilmann
 */
@RpcInterface(UcpCallCdrEvents.SERVICE_NAME)
@UciService(UcpCallCdrEvents.SERVICE_NAME)
public interface UcpCallCdrEvents {

	/** The UCI service name used for this interface */
	String SERVICE_NAME = "ucp.v30.events.callcdr";

	/**
	 * This event provides Information about new CallCdr to the UCI client
	 *
	 * @param cdrDataProperties
	 *        the newCallCdrEvent {@link CdrDataProperties}
	 */
	@RpcAsynchronous
	void callCdrEvent(Map<String, Object> cdrDataProperties);

	/**
	 * This event notifies about changes in existing CDR summary entries.
	 *
	 * @param cdrUpdateType The type of change
	 * @param cdrDataInfo The contents of the updated record
	 */
	@RpcAsynchronous
	void callCdrUpdatedEvent(CdrUpdateType cdrUpdateType, CdrDataInfo cdrDataInfo);
}
