/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.java.v30.ucp.messages.events;

import de.starface.com.rpc.annotation.RpcAsynchronous;
import de.starface.com.rpc.annotation.RpcInterface;
import de.starface.integration.uci.java.v30.UciService;
import de.starface.integration.uci.java.v30.types.FaxListEntry;
import de.starface.integration.uci.java.v30.types.FaxListEntryMovedData;

import java.util.List;
import java.util.Map;

/**
 * This interface defines the fax list events that can be received by a UCI client implementation in
 * order to handle fax list entries.
 *
 * @author tilmann
 */
@RpcInterface(UcpFaxListEvents.SERVICE_NAME)
@UciService(UcpFaxListEvents.SERVICE_NAME)
public interface UcpFaxListEvents {

	/** The UCI service name used for this interface */
	public static final String SERVICE_NAME = "ucp.v30.events.faxList";

	/**
	 * This event allows the UCI client to get informed about new fax list entries being available.
	 *
	 * @param faxListEntry
	 *        the new fax list entry with a {@link Map} containing the values defined in
	 *        {@link FaxListEntry}
	 */
	@RpcAsynchronous
	public void newFaxListEntry(Map<String, Object> faxListEntry);
	
	/**
	 * This event allows the UCI client to get informed about deleted fax list entries.
	 *
	 * @param deletedFaxListEntryIds
	 *        a {@link List} containing the ids of deleted fax list entries
	 */
	@RpcAsynchronous
	public void deletedFaxListEntries(List<String> deletedFaxListEntryIds);
	
	/**
	 * This event allows the UCI client to get informed about moved fax list entries.
	 *
	 * @param faxListEntryMovedDatas
	 *        a {@link List} of {@link Map} containing the values defined in
	 *        {@link FaxListEntryMovedData}
	 */
	@RpcAsynchronous
	public void movedFaxListEntries(List<Map<String, Object>> faxListEntryMovedDatas);
}
