/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.java.v30.ucp.messages.events;

import de.starface.com.rpc.annotation.RpcAsynchronous;
import de.starface.com.rpc.annotation.RpcInterface;
import de.starface.integration.uci.java.v30.UciService;
import de.starface.integration.uci.java.v30.values.FunctionKeyProperties;

import java.util.List;
import java.util.Map;

/**
 * This interface defines the function key events that can be received by the UCI client
 * implementation in order to handle function key changes entries.
 *
 * @author tilmann
 */
@RpcInterface(UcpFunctionKeyEvents.SERVICE_NAME)
@UciService(UcpFunctionKeyEvents.SERVICE_NAME)
public interface UcpFunctionKeyEvents {

	/** The UCI service name used for this interface */
	public static final String SERVICE_NAME = "ucp.v30.events.functionKey";

	/**
	 * The ReceiveFunctionKeyState event provides a number of properties about the function key the
	 * state is changing of. The information provided in the map is defined by its properties as
	 * seen in {@link FunctionKeyProperties}.
	 *
	 * @param functionKeyProperties
	 *        the different properties of the function key in a {@link Map}. The constants of
	 *        {@link FunctionKeyProperties} are used as keys in the map.
	 */
	@RpcAsynchronous
	public void receiveFunctionKeyState(Map<String, Object> functionKeyProperties);

	/**
	 * This event allows the UCI client to get informed when the configuration of the function keys
	 * of the user changed.
	 *
	 * @param functionKeys
	 *        the new function keys the user has as {@link List} of {@link Map} containing values as
	 *        defined by {@link FunctionKeyProperties}.
	 */
	@RpcAsynchronous
	public void functionKeysChanged(List<Map<String, Object>> functionKeys);

	/**
	 * This event allows the UCI client to get informed when the function key searching is
	 * terminated.
	 *
	 * @param searchId
	 *        the id of the terminated search
	 */
	@RpcAsynchronous
	public void functionKeySearchTerminates(String searchId);
}
