/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.java.v30.ucp.messages.events;

import de.starface.com.rpc.annotation.RpcAsynchronous;
import de.starface.com.rpc.annotation.RpcInterface;
import de.starface.integration.uci.java.v30.UciService;
import de.starface.integration.uci.java.v30.values.GroupSettingProperties;

import java.util.List;
import java.util.Map;

/**
 * This interface defines the group events that can be received by a UCI client implementation in
 * order to handle group setting changes.
 *
 * @author tilmann
 */
@UciService(UcpGroupEvents.SERVICE_NAME)
@RpcInterface(UcpGroupEvents.SERVICE_NAME)
public interface UcpGroupEvents {

	/** The UCI service name used for this interface */
	public static final String SERVICE_NAME = "ucp.v30.events.group";

	/**
	 * This event allows the UCI client to get informed when the state or number of the group
	 * settings change.
	 *
	 * @param groupSetting
	 *        the new state of the group setting with values for the properties defined in
	 *        {@link GroupSettingProperties}.
	 */
	@RpcAsynchronous
	public void groupSettingChanged(List<Map<String, Object>> groupSetting);

}
