/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.java.v30.ucp.messages.events;

import de.starface.com.rpc.annotation.RpcAsynchronous;
import de.starface.com.rpc.annotation.RpcInterface;
import de.starface.integration.uci.java.v30.UciService;

import java.util.List;
import java.util.Map;

/**
 * This interface defines the redirect events that can be received by a UCI client implementation in
 * order to handle redirect setting changes.
 *
 * @author tilmann
 */
@UciService(UcpRedirectEvents.SERVICE_NAME)
@RpcInterface(UcpRedirectEvents.SERVICE_NAME)
public interface UcpRedirectEvents {

	/** The UCI service name used for this interface */
	public static final String SERVICE_NAME = "ucp.v30.events.redirect";

	/**
	 * This event allows the UCI client to get informed when the state or number of the redirect
	 * settings change.
	 *
	 * @param redirectSettings
	 *        the new state of the redirect setting
	 */
	@RpcAsynchronous
	public void redirectSettingsChanged(List<Map<String, Object>> redirectSettings);

}
