/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.java.v30.ucp.messages.events;

import de.starface.com.rpc.annotation.RpcAsynchronous;
import de.starface.com.rpc.annotation.RpcInterface;
import de.starface.integration.uci.java.v30.UciService;
import de.starface.integration.uci.java.v30.values.UserStateProperties;

import java.util.Map;

/**
 * This interface defines user state events that can be received by the UCI client implementation.
 *
 * @author tilmann
 */
@RpcInterface(UcpUserStateEvents.SERVICE_NAME)
@UciService(UcpUserStateEvents.SERVICE_NAME)
public interface UcpUserStateEvents {

	/** The UCI service name used for this interface */
	public static final String SERVICE_NAME = "ucp.v30.events.userState";

	/**
	 * This event allows the UCI client to get informed when the state of the current user changes.
	 *
	 * @param state
	 *        the new state of the user with a {@link Map} containing values for the keys defined in
	 *        {@link UserStateProperties}.
	 */
	@RpcAsynchronous
	public void userStateChanged(Map<String, Object> state);
}
