/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.java.v30.ucp.messages.requests;

import de.starface.com.rpc.annotation.RpcInterface;
import de.starface.com.rpc.annotation.RpcValueTranslation;
import de.starface.integration.uci.java.v30.UciService;
import de.starface.integration.uci.java.v30.UciServices;
import de.starface.integration.uci.java.v30.exceptions.UciException;

import java.util.Map;

/**
 * This interface defines the FMC phone requests provided by the UCI server.
 *
 * @author shennlich
 */
@RpcInterface(UcpFmcPhoneRequests.SERVICE_NAME)
@UciService(UcpFmcPhoneRequests.SERVICE_NAME)
@RpcValueTranslation(version = UciServices.UCI_VERSION)
public interface UcpFmcPhoneRequests {
	/** The UCI service name used for this interface */
	public static final String SERVICE_NAME = "ucp.v30.requests.fmcPhone";

	/**
	 * Saving a new FMC phone or updating one. <br>
	 * For a new FMC phone, set fmcId and telephoneId to -1!<br>
	 * Devices with delay &lt; 0 will not be acceppted for saving.
	 *
	 * @param phone
	 * @throws UciException
	 */
	public void saveFmcPhone(Map<String, Object> phone) throws UciException;

	/**
	 * Deleting an FMC phone.
	 *
	 * @param phone
	 * @throws UciException
	 */
	public void deleteFmcPhone(Map<String, Object> phone) throws UciException;

	/**
	 * Getting all FMC phones of this user.
	 *
	 * @throws UciException
	 */
	public Map<String, Object> getFmcPhones() throws UciException;
}
