/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.java.v30.values;

import de.starface.integration.uci.java.v30.ucp.messages.events.UcpCallEvents;
import de.starface.integration.uci.java.v30.ucp.messages.requests.UcpCallListRequests;
import de.starface.integration.uci.java.v30.ucp.messages.requests.UcpCallRequests;

import java.util.Date;

/**
 * This enum holds constants with the property names of the call list entries as returned by
 * {@link UcpCallListRequests#getCallList(Date, Date, String, String, String, String, String, int, int)}
 *
 * @author tilmann
 */
public enum CallListEntryProperties {

	/**
	 * The id of the call list entry as {@link String}
	 *
	 * ATTENTION: This is not the same id as used in the call functions of {@link UcpCallRequests}
	 * and {@link UcpCallEvents}
	 */
	id(OrderDirection.ASCENDING, String.class),

	/** The number that was called, if known as {@link String} */
	calledNumber(OrderDirection.ASCENDING, String.class),

	/** The number of the caller, if known as {@link String} */
	callerNumber(OrderDirection.ASCENDING, String.class),

	/** A description of the call (usually the CLIP), if known as {@link String} */
	callDescription(OrderDirection.ASCENDING, String.class),

	/**
	 * The start time of the call as {@link java.util.Date}. Per default dates will be ordered newest
	 * first
	 */
	startTime(OrderDirection.DESCENDING, Date.class),

	/** The duration of the call in seconds as {@link Integer} */
	duration(OrderDirection.ASCENDING, Integer.class),

	/**
	 * The direction of the call as {@link String} with one of the values of
	 * {@link CallListEntryDirection}
	 */
	direction(OrderDirection.ASCENDING, String.class),

	/**
	 * The result of the call as {@link String} with one of the values of
	 * {@link CallListEntryResult}
	 */
	result(OrderDirection.ASCENDING, String.class),

	/** The id of a voicemail that was recorded for the call if existing as {@link String} */
	voicemailId(OrderDirection.ASCENDING, String.class),

	/**
	 * The name of the group if this call was received by a group as {@link String} (the empty
	 * string otherwise)
	 */
	groupId(OrderDirection.ASCENDING, String.class),

	/**
	 * The Jabber-ID of the caller if the caller is known, empty String otherwise
	 */
	callerJabberId(OrderDirection.ASCENDING, String.class),

	/**
	 * If the call was marked as having been called back.
	 */
	calledBack(OrderDirection.ASCENDING, Boolean.class),

	/**
	 * The comment in this call, or null if none.
	 */
	comment(OrderDirection.ASCENDING, String.class),

	/**
	 * The name of the person that picked up the call, or null if none.
	 */
	answeredBy(OrderDirection.ASCENDING, String.class),

	;

	private OrderDirection defaultOrderDirection;

	private Class<?> type;

	private CallListEntryProperties(OrderDirection defaultOrderDirection, Class<?> type) {
		this.defaultOrderDirection = defaultOrderDirection;
		this.type = type;
	}

	/**
	 * @return the default {@link OrderDirection} for the current property
	 */
	public OrderDirection getDefaultOrderDirection() {
		return defaultOrderDirection;
	}

	/**
	 * @return the java type of the property
	 */
	public Class<?> getType() {
		return type;
	}

}
