/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.java.v30.values;

import de.starface.integration.uci.java.v30.ucp.messages.events.UcpCallEvents;

import java.util.Date;
import java.util.List;

/**
 * This enum holds constants with the property names of the call as returned by
 * {@link UcpCallEvents#receiveCallState(java.util.Map)}.
 *
 * @author tilmann
 */
public enum CallProperties {
	/** The id of the call in form of a UUID as {@link String} */
	id(String.class),

	/** The state of the call as {@link String} by name defined in {@link CallState} */
	state(String.class),

	/** The point of time when the call reached the state as {@link Date} */
	timestamp(Date.class),

	/** The phone number that has been called as {@link String} */
	calledNumber(String.class),

	/** The name of the person being called as {@link String} */
	calledName(String.class),

	/** The number of the caller as {@link String} */
	callerNumber(String.class),

	/** The name of the person calling as {@link String} */
	callerName(String.class),

	/**
	 * The id of the group if this call was received by a group as {@link String} (the empty string
	 * otherwise)
	 */
	groupId(String.class),

	/** The channel name **/
	peerNames(List.class),

	/** The channel name **/
	channelNames(List.class),

	/** The sip callId **/
	sipCallIds(List.class),

	/** Is caller **/
	caller(Boolean.class),

	/** The point of time when the call reached the state linked as {@link Date} */
	connectedTimestamp(Date.class),

	/** Is in the current call recorded by the user */
	recorder(Boolean.class),

	/** Is in the current call recording activated */
	recorded(Boolean.class),

	/** Is the user a monitor owner */
	recordOwner(Boolean.class),

	/** AvatarHash of Caller */
	avatarHash(String.class),

	/** Doorline Webcam */
	doorlineCamUrl(String.class),

	/** Doorline DTMF Code */
	doorlineDtmfCode(String.class),

	/** Doorline image provider id */
	doorlineImageProviderId(Integer.class),

	/** Forward type */
	forwardType(String.class),

	/** Forwarder callerIdName of forwarder */
	forwarderCallerIdName(String.class),

	/** Forwarder callerIdNumber of forwarder */
	forwarderCallerIdNumber(String.class),

	/** The conference room identification */
	conferenceRoomId(String.class),

	/** The current call is an conference admin */
	conferenceAdmin(Boolean.class),

	/** The reference of the outher side of consultation */
	referenceOfConsultation(String.class),

	/** The time between now and the connected timestamp/timestamp **/
	duration(Long.class),

	/** Is this call an internal call */
	internalCall(Boolean.class),

	/** Is this call a consultationCall */
	consultationCall(Boolean.class),

	/** The jabber id */
	jabberId(String.class);

	private Class<?> type;

	private CallProperties(Class<?> type) {
		this.type = type;
	}

	/**
	 * @return the java type of the property
	 */
	public Class<?> getType() {
		return type;
	}
}
