/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.java.v30.values;

import de.starface.integration.uci.java.v30.ucp.messages.events.UcpCallEvents;

/**
 * This enum holds constants with the property names of the call as returned by
 * {@link UcpCallEvents#receiveCallState(java.util.Map)}.
 *
 * @author dirk
 */
public enum ConferenceCallProperties {
	/** The id of the call in form of a UUID as {@link String} */
	id(String.class),

	/** The phone number of the conference participant */
	number(String.class),

	/** The name of the conference participant */
	name(String.class),

	/** Can the participant hear */
	listening(Boolean.class),

	/** Can the participant speak */
	muted(Boolean.class),

	/** Is the participant speaking */
	talking(Boolean.class),

	/** Want the participant speak */
	wantToSpeak(Integer.class),

	/** Is the participant admin */
	admin(Boolean.class),

	/** The state of the call as {@link String} by name defined in {@link CallState} */
	state(String.class),

	/** The Room Identifikation Number */
	roomId(String.class),

	/** Entered Room */
	enteredTimestamp(java.util.Date.class),

	/** The time between now and the enteredTimestamp in milliseconds **/
	duration(Long.class),

	/** The jabber id */
	jabberId(String.class),

	/** The avatar hash **/
	avatarHash(String.class);

	private Class<?> type;

	private ConferenceCallProperties(Class<?> type) {
		this.type = type;
	}

	/**
	 * @return the java type of the property
	 */
	public Class<?> getType() {
		return type;
	}
}
