/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.java.v30.values;

import de.starface.integration.uci.java.v30.ucp.messages.events.UcpFaxEvents;
import de.starface.integration.uci.java.v30.ucp.messages.requests.UcpFaxListRequests;
import de.starface.integration.uci.java.v30.ucp.messages.requests.UcpFaxRequests;

import java.util.Date;

/**
 * This enum holds constants with the property names of the fax list entries as returned by
 * {@link UcpFaxListRequests#getFaxList(Date, Date, String, String, String, String, int, int)}
 *
 * @author tilmann
 */
public enum FaxListEntryProperties {
	/**
	 * The id of the fax list entry as {@link String}
	 *
	 * ATTENTION: This is not the same id as used in the fax functions of {@link UcpFaxRequests} and
	 * {@link UcpFaxEvents}
	 */
	id(OrderDirection.ASCENDING, String.class),

	/** The number that was called, if known as {@link String} */
	calledNumber(OrderDirection.ASCENDING, String.class),

	/** The number of the sender, if known as {@link String} */
	callerNumber(OrderDirection.ASCENDING, String.class),

	/** A description of the fax call (usually the CLIP), if known as {@link String} */
	callDescription(OrderDirection.ASCENDING, String.class),

	/**
	 * The folder the fax belongs to as {@link String} holding one of the values defined in
	 * {@link FaxListEntryFolder}.
	 */
	folder(OrderDirection.ASCENDING, String.class),

	/**
	 * The time when the fax was received as {@link java.util.Date}. Dates will be ordered newest
	 * first per default.
	 */
	receptionTime(OrderDirection.DESCENDING, Date.class),

	/** A flag indicating that the fax was not transmitted completely as {@link Boolean} */
	partial(OrderDirection.ASCENDING, Boolean.class),

	/**
	 * The name of the group if this fax was received by a group as {@link String} (the empty string
	 * otherwise)
	 */
	groupId(OrderDirection.ASCENDING, String.class),

	/**
	 * The Jabber-ID of the caller if the caller is known, empty String otherwise
	 */
	callerJabberId(OrderDirection.ASCENDING, String.class);

	private OrderDirection defaultOrderDirection;

	private Class<?> type;

	private FaxListEntryProperties(OrderDirection defaultOrderDirection, Class<?> type) {
		this.defaultOrderDirection = defaultOrderDirection;
		this.type = type;
	}

	/**
	 * @return the default {@link OrderDirection} for the current property
	 */
	public OrderDirection getDefaultOrderDirection() {
		return defaultOrderDirection;
	}

	/**
	 * @return the java type of the property
	 */
	public Class<?> getType() {
		return type;
	}

}
