/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.java.v30.values;

import de.starface.integration.uci.java.v30.ucp.messages.requests.UcpFaxListRequests;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * This enum holds constants with the property names of the fax list as returned by
 * {@link UcpFaxListRequests#getFaxList(Date, Date, String, String, String, String, int, int)}
 *
 * @author tilmann
 */
public enum FaxListProperties {
	/** The entries received before this point of time given as {@link Date} */
	receivedBefore(Date.class),

	/** The entries received after this point of time given as {@link Date} */
	receivedAfter(Date.class),

	/**
	 * The entries are restricted to those in the given folder with one of the values of
	 * {@link FaxListEntryFolder} as {@link String}
	 */
	folderRestriction(String.class),

	/**
	 * The entries are restricted to faxes belonging to the group with the given id or to faxes that
	 * do not belong to any group if {@link GroupRestriction#NON_GROUP} is given. If an empty String
	 * is given no restriction is used.
	 */
	groupRestriction(String.class),

	/**
	 * The entries are ordered by the given property. This may be one of the properties in
	 * {@link FaxListEntryProperties} as {@link String}
	 */
	orderProperty(String.class),

	/**
	 * The entries are ordered in the given direction. This may be one of the values in
	 * {@link OrderDirection} as {@link String}
	 */
	orderDirection(String.class),

	/**
	 * The offset used for the retrieval of this list regarding the given time constraints and
	 * restrictions as {@link Integer}
	 */
	countOffset(Integer.class),

	/**
	 * The limit used for the retrieval of this list regarding the given time constraints and
	 * restrictions as {@link Integer}
	 */
	countLimit(Integer.class),

	/**
	 * The total count of available entries on the server regarding the given time constraints and
	 * restrictions as {@link Integer}.
	 */
	totalCount(Integer.class),

	/**
	 * The fax list entries in this list as {@link List} of {@link Map} with properties as defined
	 * in {@link FaxListEntryProperties}
	 */
	entries(List.class);

	private Class<?> type;

	private FaxListProperties(Class<?> type) {
		this.type = type;
	}

	/**
	 * @return the java type of the property
	 */
	public Class<?> getType() {
		return type;
	}

}
