/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.java.v30.values;

import de.starface.integration.uci.java.v30.ucp.messages.requests.UcpGroupRequests;

import java.util.List;
import java.util.Map;

/**
 * This enum holds constants with the proprty names of the group settings as returned by
 * {@link UcpGroupRequests#getGroupSettings()}
 *
 * @author tilmann
 */
public enum GroupSettingProperties {
	/** The id of the group as {@link String} */
	id(String.class),

	/** The display name of the group for the current user as {@link String} */
	name(String.class),

	/** A flag indicating if the user is currently logged on to the group or not */
	loggedOn(Boolean.class),

	/** The Id that can be used to log on to the group using the telephone */
	logonId(String.class),

	/**
	 * The phone numbers allocated to the group as {@link List} of {@link Map} with properties as
	 * defined in {@link PhoneNumberProperties}
	 */
	phoneNumbers(List.class);

	private Class<?> type;

	private GroupSettingProperties(Class<?> type) {
		this.type = type;
	}

	/**
	 * @return the java type of the property
	 */
	public Class<?> getType() {
		return type;
	}

}
