/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.java.v30.values;

import de.starface.integration.uci.java.v30.ucp.messages.requests.UcpMonitorListRequests;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * This enum holds constants with the property names of the monitor list as returned by
 * {@link UcpMonitorListRequests#getMonitorList(Date, Date, String, String, int, int)}
 *
 * @author tilmann
 */
public enum MonitorListProperties {
	/** The entries start before this point of time given as {@link Date} */
	startBefore(Date.class),

	/** The entries start after this point of time given as {@link Date} */
	startAfter(Date.class),

	/**
	 * The entries are ordered by the given property. This may be one of the properties in
	 * {@link MonitorListEntryProperties} as {@link String}
	 */
	orderProperty(String.class),

	/**
	 * The entries are ordered in the given direction. This may be one of the values in
	 * {@link OrderDirection} as {@link String}
	 */
	orderDirection(String.class),

	/**
	 * The offset used for the retrieval of this list regarding the given time constraints and
	 * restrictions as {@link Integer}
	 */
	countOffset(Integer.class),

	/**
	 * The limit used for the retrieval of this list regarding the given time constraints and
	 * restrictions as {@link Integer}
	 */
	countLimit(Integer.class),

	/**
	 * The total count of available entries on the server regarding the given time constraints and
	 * restrictions as {@link Integer}.
	 */
	totalCount(Integer.class),

	/**
	 * The call list entries in this list as {@link List} of {@link Map} with properties as defined
	 * in {@link VoicemailListEntryProperties}
	 */
	entries(List.class);

	private Class<?> type;

	private MonitorListProperties(Class<?> type) {
		this.type = type;
	}

	/**
	 * @return the java type of the property
	 */
	public Class<?> getType() {
		return type;
	}

}
