/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.java.v30.values;

import de.starface.integration.uci.java.v30.ucp.messages.requests.UcpUserStateRequests;

/**
 * This enum holds constants with the names ot the properties of the user state as returned by
 * {@link UcpUserStateRequests#getUserState()}
 *
 * @author tilmann
 */
public enum UserStateProperties {
	/** A {@link Boolean} flag indicating whether the user has DND set to active */
	doNotDisturbSetting(Boolean.class),

	/** A {@link Boolean} flag indicating whether the user has any redirect always settings enabled */
	redirectAlwaysActive(Boolean.class),

	/**
	 * The current chat presence of the user as {@link String} with one of the values of
	 * {@link ChatPresence}
	 */
	chatPresence(String.class),

	/** The current chat presence message of the user as {@link String} */
	chatPresenceMessage(String.class),

	/**
	 * The SHA-1 hash of the users avatar image as {@link String} if one is set. The image itself
	 * can be queried with {@link UcpUserStateRequests#getAvatarImage()}.
	 */
	avatarHash(String.class),

	/**
	 * The current telephony state of the user as {@link String} with one of the values of
	 * {@link TelephonyState}.
	 */
	telephonyState(String.class);

	private Class<?> type;

	private UserStateProperties(Class<?> type) {
		this.type = type;
	}

	/**
	 * @return the java type of the property
	 */
	public Class<?> getType() {
		return type;
	}
}
